// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.

using System;
using System.Collections.Generic;
using Bicep.Core.Resources;
using System.Linq;
using Bicep.Core.TypeSystem.Az;
using Moq;
using Bicep.Core.TypeSystem;

namespace Bicep.Core.UnitTests.Mock
{
    internal static class FakeResourceTypes
    {
        private static string[] SplitLines(string fakeResourceTypesAndVersions)
        {
            var types = fakeResourceTypesAndVersions
               .ReplaceLineEndings("\n")
               .Split('\n', StringSplitOptions.RemoveEmptyEntries | StringSplitOptions.RemoveEmptyEntries)
               .Select(line => line.Trim())
               .Where(s => !string.IsNullOrWhiteSpace(s));
            return types.ToArray();
        }

        public static IEnumerable<ResourceTypeReference> GetFakeResourceTypeReferences(string fakeResourceTypesAndVersions)
        {
            return GetFakeResourceTypeReferences(SplitLines(fakeResourceTypesAndVersions));
        }

        public static IEnumerable<ResourceTypeReference> GetFakeResourceTypeReferences(IEnumerable<string> fakeResourceTypesAndVersions)
        {
            foreach (string line in fakeResourceTypesAndVersions)
            {
                var resourcType = ResourceTypeReference.Parse(line);
                yield return resourcType;
            }
        }

        public static string[] ResourceScopeTypes
            => SplitLines(ResourceScopeString);

        private const string ResourceScopeString = @"Fake.Genomics/accounts@2417-08-01-preview
Fake.RecoveryServices/vaults@2418-01-10
Fake.RecoveryServices/vaults@2417-07-01
Fake.RecoveryServices/vaults@2416-05-01
Fake.RecoveryServices/vaults@2415-12-15
Fake.RecoveryServices/vaults@2415-11-10
Fake.RecoveryServices/vaults@2415-08-15
Fake.RecoveryServices/vaults@2415-08-10
Fake.RecoveryServices/vaults@2415-06-10
Fake.RecoveryServices/vaults@2415-03-15
Fake.RecoveryServices/vaults@2416-06-01
Fake.RecoveryServices/vaults@2420-02-02
Fake.RecoveryServices/vaults@2420-10-01
Fake.RecoveryServices/vaults@2421-01-01
Fake.RecoveryServices/vaults@2421-02-10
Fake.RecoveryServices/vaults@2421-03-01
Fake.RecoveryServices/vaults@2421-04-01
Fake.RecoveryServices/vaults@2421-06-01
Fake.RecoveryServices/vaults@2421-07-01
Fake.Kusto/clusters@2417-09-07-privatepreview
Fake.Kusto/clusters@2418-09-07-preview
Fake.Kusto/clusters@2419-01-21
Fake.Kusto/clusters@2419-05-15
Fake.Kusto/clusters@2419-09-07
Fake.Kusto/clusters@2419-11-09
Fake.Kusto/clusters@2420-02-15
Fake.Kusto/clusters@2420-06-14
Fake.Kusto/clusters@2420-09-18
Fake.Kusto/clusters@2421-01-01
Fake.Kusto/clusters/databases@2417-09-07-privatepreview
Fake.Kusto/clusters/databases@2418-09-07-preview
Fake.Kusto/clusters/databases@2419-01-21
Fake.Kusto/clusters/databases@2419-05-15
Fake.Kusto/clusters/databases@2419-09-07
Fake.Kusto/clusters/databases@2419-11-09
Fake.Kusto/clusters/databases@2420-02-15
Fake.Kusto/clusters/databases@2420-06-14
Fake.Kusto/clusters/databases@2420-09-18
Fake.Kusto/clusters/databases@2421-01-01
Fake.Kusto/clusters/databases/dataconnections@2419-01-21
Fake.Kusto/clusters/databases/dataconnections@2419-05-15
Fake.Kusto/clusters/databases/dataconnections@2419-09-07
Fake.Kusto/clusters/databases/dataconnections@2419-11-09
Fake.Kusto/clusters/databases/dataconnections@2420-02-15
Fake.Kusto/clusters/databases/dataconnections@2420-06-14
Fake.Kusto/clusters/databases/dataconnections@2420-09-18
Fake.Kusto/clusters/databases/dataconnections@2421-01-01
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2419-09-07
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2419-11-09
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2420-02-15
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2420-06-14
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2420-09-18
Fake.Kusto/clusters/AttachedDatabaseConfigurations@2421-01-01
Fake.Kusto/clusters/dataConnections@2419-11-09
Fake.Kusto/clusters/dataConnections@2420-02-15
Fake.Kusto/clusters/dataConnections@2420-06-14
Fake.Kusto/clusters/dataConnections@2420-09-18
Fake.Kusto/clusters/dataConnections@2421-01-01
Fake.Kusto/clusters/principalAssignments@2419-11-09
Fake.Kusto/clusters/principalAssignments@2420-02-15
Fake.Kusto/clusters/principalAssignments@2420-06-14
Fake.Kusto/clusters/principalAssignments@2420-09-18
Fake.Kusto/clusters/principalAssignments@2421-01-01
Fake.Kusto/clusters/databases/principalAssignments@2419-11-09
Fake.Kusto/clusters/databases/principalAssignments@2420-02-15
Fake.Kusto/clusters/databases/principalAssignments@2420-06-14
Fake.Kusto/clusters/databases/principalAssignments@2420-09-18
Fake.Kusto/clusters/databases/principalAssignments@2421-01-01
Fake.Kusto/clusters/databases/scripts@2421-01-01
Fake.Network/trafficManagerProfiles@2415-11-01
Fake.Network/trafficManagerProfiles@2417-03-01
Fake.Network/trafficManagerProfiles@2417-05-01
Fake.Network/trafficManagerProfiles@2418-04-01
Fake.Compute/availabilitySets@2415-05-01-preview
Fake.Compute/availabilitySets@2415-06-15
Fake.Compute/availabilitySets@2416-04-30-preview
Fake.Compute/availabilitySets@2417-03-30
Fake.Compute/availabilitySets@2417-12-01
Fake.Compute/availabilitySets@2418-06-01
Fake.Compute/availabilitySets@2418-10-01
Fake.Compute/availabilitySets@2419-03-01
Fake.Compute/availabilitySets@2419-07-01
Fake.Compute/availabilitySets@2420-06-01
Fake.Compute/availabilitySets@2420-12-01
Fake.Compute/availabilitySets@2421-03-01
Fake.Compute/availabilitySets@2421-04-01
Fake.Compute/availabilitySets@2421-07-01
Fake.Compute/virtualMachines/extensions@2415-05-01-preview
Fake.Compute/virtualMachines/extensions@2415-06-15
Fake.Compute/virtualMachines/extensions@2416-03-30
Fake.Compute/virtualMachines/extensions@2416-04-30-preview
Fake.Compute/virtualMachines/extensions@2417-03-30
Fake.Compute/virtualMachines/extensions@2417-12-01
Fake.Compute/virtualMachines/extensions@2418-06-01
Fake.Compute/virtualMachines/extensions@2418-10-01
Fake.Compute/virtualMachines/extensions@2419-03-01
Fake.Compute/virtualMachines/extensions@2419-07-01
Fake.Compute/virtualMachines/extensions@2420-06-01
Fake.Compute/virtualMachines/extensions@2420-12-01
Fake.Compute/virtualMachines/extensions@2421-03-01
Fake.Compute/virtualMachines/extensions@2421-04-01
Fake.Compute/virtualMachines/extensions@2421-07-01
Fake.Compute/virtualMachineScaleSets@2415-05-01-preview
Fake.Compute/virtualMachineScaleSets@2415-06-15
Fake.Compute/virtualMachineScaleSets@2416-04-30-preview
Fake.Compute/virtualMachineScaleSets@2417-03-30
Fake.Compute/virtualMachineScaleSets@2417-12-01
Fake.Compute/virtualMachineScaleSets@2418-06-01
Fake.Compute/virtualMachineScaleSets@2418-10-01
Fake.Compute/virtualMachineScaleSets@2419-03-01
Fake.Compute/virtualMachineScaleSets@2419-07-01
Fake.Compute/virtualMachineScaleSets@2420-06-01
Fake.Compute/virtualMachineScaleSets@2420-12-01
Fake.Compute/virtualMachineScaleSets@2421-03-01
Fake.Compute/virtualMachineScaleSets@2421-04-01
Fake.Compute/virtualMachineScaleSets@2421-07-01
Fake.Compute/virtualMachines@2415-05-01-preview
Fake.Compute/virtualMachines@2415-06-15
Fake.Compute/virtualMachines@2416-04-30-preview
Fake.Compute/virtualMachines@2417-03-30
Fake.Compute/virtualMachines@2417-12-01
Fake.Compute/virtualMachines@2418-06-01
Fake.Compute/virtualMachines@2418-10-01
Fake.Compute/virtualMachines@2419-03-01
Fake.Compute/virtualMachines@2419-07-01
Fake.Compute/virtualMachines@2420-06-01
Fake.Compute/virtualMachines@2420-12-01
Fake.Compute/virtualMachines@2421-03-01
Fake.Compute/virtualMachines@2421-04-01
Fake.Compute/virtualMachines@2421-07-01
Fake.Network/dnszones@2415-05-04-preview
Fake.Network/dnszones@2416-04-01
Fake.Network/dnszones@2417-09-01
Fake.Network/dnszones@2417-10-01
Fake.Network/dnszones@2418-03-01-preview
Fake.Network/dnszones@2418-05-01
Fake.Network/dnszones/A@2415-05-04-preview
Fake.Network/dnszones/A@2416-04-01
Fake.Network/dnszones/A@2417-09-01
Fake.Network/dnszones/A@2417-10-01
Fake.Network/dnszones/A@2418-05-01
Fake.Network/dnszones/AAAA@2415-05-04-preview
Fake.Network/dnszones/AAAA@2416-04-01
Fake.Network/dnszones/AAAA@2417-09-01
Fake.Network/dnszones/AAAA@2417-10-01
Fake.Network/dnszones/AAAA@2418-05-01
Fake.Network/dnszones/CNAME@2415-05-04-preview
Fake.Network/dnszones/CNAME@2416-04-01
Fake.Network/dnszones/CNAME@2417-09-01
Fake.Network/dnszones/CNAME@2417-10-01
Fake.Network/dnszones/CNAME@2418-05-01
Fake.Network/dnszones/MX@2415-05-04-preview
Fake.Network/dnszones/MX@2416-04-01
Fake.Network/dnszones/MX@2417-09-01
Fake.Network/dnszones/MX@2417-10-01
Fake.Network/dnszones/MX@2418-05-01
Fake.Network/dnszones/NS@2415-05-04-preview
Fake.Network/dnszones/NS@2416-04-01
Fake.Network/dnszones/NS@2417-09-01
Fake.Network/dnszones/NS@2417-10-01
Fake.Network/dnszones/NS@2418-05-01
Fake.Network/dnszones/PTR@2415-05-04-preview
Fake.Network/dnszones/PTR@2416-04-01
Fake.Network/dnszones/PTR@2417-09-01
Fake.Network/dnszones/PTR@2417-10-01
Fake.Network/dnszones/PTR@2418-05-01
Fake.Network/dnszones/SOA@2415-05-04-preview
Fake.Network/dnszones/SOA@2416-04-01
Fake.Network/dnszones/SOA@2417-09-01
Fake.Network/dnszones/SOA@2417-10-01
Fake.Network/dnszones/SOA@2418-05-01
Fake.Network/dnszones/SRV@2415-05-04-preview
Fake.Network/dnszones/SRV@2416-04-01
Fake.Network/dnszones/SRV@2417-09-01
Fake.Network/dnszones/SRV@2417-10-01
Fake.Network/dnszones/SRV@2418-05-01
Fake.Network/dnszones/TXT@2415-05-04-preview
Fake.Network/dnszones/TXT@2416-04-01
Fake.Network/dnszones/TXT@2417-09-01
Fake.Network/dnszones/TXT@2417-10-01
Fake.Network/dnszones/TXT@2418-05-01
Fake.Logic/workflows@2415-02-01-preview
Fake.Logic/workflows@2416-06-01
Fake.Logic/workflows@2416-10-01
Fake.Logic/workflows@2417-07-01
Fake.Logic/integrationAccounts@2416-06-01
Fake.Logic/integrationAccounts/agreements@2416-06-01
Fake.Logic/integrationAccounts/certificates@2416-06-01
Fake.Logic/integrationAccounts/maps@2416-06-01
Fake.Logic/integrationAccounts/partners@2416-06-01
Fake.Logic/integrationAccounts/schemas@2416-06-01
Fake.Logic/integrationAccounts/assemblies@2416-06-01
Fake.Logic/integrationAccounts/batchConfigurations@2416-06-01
Fake.Media/mediaServices@2415-10-01
Fake.Media/mediaServices@2418-07-01
Fake.Media/mediaServices/accountFilters@2418-07-01
Fake.Media/mediaServices/assets@2418-07-01
Fake.Media/mediaServices/assets/assetFilters@2418-07-01
Fake.Media/mediaServices/contentKeyPolicies@2418-07-01
Fake.Media/mediaServices/liveEvents@2418-07-01
Fake.Media/mediaServices/liveEvents/liveOutputs@2418-07-01
Fake.Media/mediaServices/streamingEndpoints@2418-07-01
Fake.Media/mediaServices/streamingLocators@2418-07-01
Fake.Media/mediaServices/streamingPolicies@2418-07-01
Fake.Media/mediaServices/transforms@2418-07-01
Fake.Media/mediaServices/transforms/jobs@2418-07-01
Fake.Solutions/applianceDefinitions@2416-09-01-preview
Fake.Solutions/appliances@2416-09-01-preview
Fake.Compute/disks@2416-04-30-preview
Fake.Compute/disks@2417-03-30
Fake.Compute/disks@2418-04-01
Fake.Compute/disks@2418-06-01
Fake.Compute/disks@2419-07-01
Fake.Compute/disks@2420-06-30
Fake.Compute/disks@2420-09-30
Fake.Compute/snapshots@2416-04-30-preview
Fake.Compute/snapshots@2417-03-30
Fake.Compute/snapshots@2418-04-01
Fake.Compute/snapshots@2418-06-01
Fake.Compute/snapshots@2419-07-01
Fake.Compute/snapshots@2420-06-30
Fake.Compute/snapshots@2420-09-30
Fake.Compute/images@2416-04-30-preview
Fake.Compute/images@2417-03-30
Fake.Compute/images@2417-12-01
Fake.Compute/images@2418-06-01
Fake.Compute/images@2418-10-01
Fake.Compute/images@2419-03-01
Fake.Compute/images@2419-07-01
Fake.Compute/images@2420-06-01
Fake.Compute/images@2420-12-01
Fake.Compute/images@2421-03-01
Fake.Compute/images@2421-04-01
Fake.Compute/images@2421-07-01
Fake.Network/publicIPAddresses@2416-06-01
Fake.Network/publicIPAddresses@2416-07-01
Fake.Network/publicIPAddresses@2416-08-01
Fake.Network/publicIPAddresses@2416-09-01
Fake.Network/publicIPAddresses@2416-10-01
Fake.Network/publicIPAddresses@2416-11-01
Fake.Network/publicIPAddresses@2416-12-01
Fake.Network/publicIPAddresses@2417-03-01
Fake.Network/publicIPAddresses@2417-06-01
Fake.Network/publicIPAddresses@2417-08-01
Fake.Network/publicIPAddresses@2417-09-01
Fake.Network/publicIPAddresses@2417-10-01
Fake.Network/publicIPAddresses@2417-11-01
Fake.Network/publicIPAddresses@2415-05-01-preview
Fake.Network/publicIPAddresses@2415-06-15
Fake.Network/publicIPAddresses@2416-03-30
Fake.Network/publicIPAddresses@2418-01-01
Fake.Network/publicIPAddresses@2418-02-01
Fake.Network/publicIPAddresses@2418-04-01
Fake.Network/publicIPAddresses@2418-06-01
Fake.Network/publicIPAddresses@2418-07-01
Fake.Network/publicIPAddresses@2418-08-01
Fake.Network/publicIPAddresses@2418-10-01
Fake.Network/publicIPAddresses@2418-12-01
Fake.Network/publicIPAddresses@2419-02-01
Fake.Network/publicIPAddresses@2419-04-01
Fake.Network/publicIPAddresses@2419-06-01
Fake.Network/publicIPAddresses@2419-07-01
Fake.Network/publicIPAddresses@2419-08-01
Fake.Network/publicIPAddresses@2419-09-01
Fake.Network/publicIPAddresses@2418-11-01
Fake.Network/publicIPAddresses@2419-11-01
Fake.Network/publicIPAddresses@2419-12-01
Fake.Network/publicIPAddresses@2420-03-01
Fake.Network/publicIPAddresses@2420-04-01
Fake.Network/publicIPAddresses@2420-05-01
Fake.Network/publicIPAddresses@2420-06-01
Fake.Network/publicIPAddresses@2420-08-01
Fake.Network/publicIPAddresses@2420-11-01
Fake.Network/virtualNetworks@2416-06-01
Fake.Network/virtualNetworks@2416-07-01
Fake.Network/virtualNetworks@2416-08-01
Fake.Network/virtualNetworks@2416-09-01
Fake.Network/virtualNetworks@2416-10-01
Fake.Network/virtualNetworks@2416-11-01
Fake.Network/virtualNetworks@2416-12-01
Fake.Network/virtualNetworks@2417-03-01
Fake.Network/virtualNetworks@2417-06-01
Fake.Network/virtualNetworks@2417-08-01
Fake.Network/virtualNetworks@2417-09-01
Fake.Network/virtualNetworks@2417-10-01
Fake.Network/virtualNetworks@2417-11-01
Fake.Network/virtualNetworks@2415-05-01-preview
Fake.Network/virtualNetworks@2415-06-15
Fake.Network/virtualNetworks@2416-03-30
Fake.Network/virtualNetworks@2418-01-01
Fake.Network/virtualNetworks@2418-02-01
Fake.Network/virtualNetworks@2418-04-01
Fake.Network/virtualNetworks@2418-06-01
Fake.Network/virtualNetworks@2418-07-01
Fake.Network/virtualNetworks@2418-08-01
Fake.Network/virtualNetworks@2418-10-01
Fake.Network/virtualNetworks@2418-12-01
Fake.Network/virtualNetworks@2419-02-01
Fake.Network/virtualNetworks@2419-04-01
Fake.Network/virtualNetworks@2419-06-01
Fake.Network/virtualNetworks@2419-07-01
Fake.Network/virtualNetworks@2419-08-01
Fake.Network/virtualNetworks@2419-09-01
Fake.Network/virtualNetworks@2418-11-01
Fake.Network/virtualNetworks@2419-11-01
Fake.Network/virtualNetworks@2419-12-01
Fake.Network/virtualNetworks@2420-03-01
Fake.Network/virtualNetworks@2420-04-01
Fake.Network/virtualNetworks@2420-05-01
Fake.Network/virtualNetworks@2420-06-01
Fake.Network/virtualNetworks@2420-08-01
Fake.Network/virtualNetworks@2420-11-01
Fake.Network/loadBalancers@2416-06-01
Fake.Network/loadBalancers@2416-07-01
Fake.Network/loadBalancers@2416-08-01
Fake.Network/loadBalancers@2416-09-01
Fake.Network/loadBalancers@2416-10-01
Fake.Network/loadBalancers@2416-11-01
Fake.Network/loadBalancers@2416-12-01
Fake.Network/loadBalancers@2417-03-01
Fake.Network/loadBalancers@2417-06-01
Fake.Network/loadBalancers@2417-08-01
Fake.Network/loadBalancers@2417-09-01
Fake.Network/loadBalancers@2417-10-01
Fake.Network/loadBalancers@2417-11-01
Fake.Network/loadBalancers@2415-05-01-preview
Fake.Network/loadBalancers@2415-06-15
Fake.Network/loadBalancers@2416-03-30
Fake.Network/loadBalancers@2418-01-01
Fake.Network/loadBalancers@2418-02-01
Fake.Network/loadBalancers@2418-04-01
Fake.Network/loadBalancers@2418-06-01
Fake.Network/loadBalancers@2418-07-01
Fake.Network/loadBalancers@2418-08-01
Fake.Network/loadBalancers@2418-10-01
Fake.Network/loadBalancers@2418-12-01
Fake.Network/loadBalancers@2419-02-01
Fake.Network/loadBalancers@2419-04-01
Fake.Network/loadBalancers@2419-06-01
Fake.Network/loadBalancers@2419-07-01
Fake.Network/loadBalancers@2419-08-01
Fake.Network/loadBalancers@2419-09-01
Fake.Network/loadBalancers@2418-11-01
Fake.Network/loadBalancers@2419-11-01
Fake.Network/loadBalancers@2419-12-01
Fake.Network/loadBalancers@2420-03-01
Fake.Network/loadBalancers@2420-04-01
Fake.Network/loadBalancers@2420-05-01
Fake.Network/loadBalancers@2420-06-01
Fake.Network/loadBalancers@2420-08-01
Fake.Network/loadBalancers@2420-11-01
Fake.Network/networkSecurityGroups@2416-06-01
Fake.Network/networkSecurityGroups@2416-07-01
Fake.Network/networkSecurityGroups@2416-08-01
Fake.Network/networkSecurityGroups@2416-09-01
Fake.Network/networkSecurityGroups@2416-10-01
Fake.Network/networkSecurityGroups@2416-11-01
Fake.Network/networkSecurityGroups@2416-12-01
Fake.Network/networkSecurityGroups@2417-03-01
Fake.Network/networkSecurityGroups@2417-06-01
Fake.Network/networkSecurityGroups@2417-08-01
Fake.Network/networkSecurityGroups@2417-09-01
Fake.Network/networkSecurityGroups@2417-10-01
Fake.Network/networkSecurityGroups@2417-11-01
Fake.Network/networkSecurityGroups@2415-05-01-preview
Fake.Network/networkSecurityGroups@2415-06-15
Fake.Network/networkSecurityGroups@2416-03-30
Fake.Network/networkSecurityGroups@2418-01-01
Fake.Network/networkSecurityGroups@2418-02-01
Fake.Network/networkSecurityGroups@2418-04-01
Fake.Network/networkSecurityGroups@2418-06-01
Fake.Network/networkSecurityGroups@2418-07-01
Fake.Network/networkSecurityGroups@2418-08-01
Fake.Network/networkSecurityGroups@2418-10-01
Fake.Network/networkSecurityGroups@2418-12-01
Fake.Network/networkSecurityGroups@2419-02-01
Fake.Network/networkSecurityGroups@2419-04-01
Fake.Network/networkSecurityGroups@2419-06-01
Fake.Network/networkSecurityGroups@2419-07-01
Fake.Network/networkSecurityGroups@2419-08-01
Fake.Network/networkSecurityGroups@2419-09-01
Fake.Network/networkSecurityGroups@2418-11-01
Fake.Network/networkSecurityGroups@2419-11-01
Fake.Network/networkSecurityGroups@2419-12-01
Fake.Network/networkSecurityGroups@2420-03-01
Fake.Network/networkSecurityGroups@2420-04-01
Fake.Network/networkSecurityGroups@2420-05-01
Fake.Network/networkSecurityGroups@2420-06-01
Fake.Network/networkSecurityGroups@2420-08-01
Fake.Network/networkSecurityGroups@2420-11-01
Fake.Network/networkInterfaces@2416-06-01
Fake.Network/networkInterfaces@2416-07-01
Fake.Network/networkInterfaces@2416-08-01
Fake.Network/networkInterfaces@2416-09-01
Fake.Network/networkInterfaces@2416-10-01
Fake.Network/networkInterfaces@2416-11-01
Fake.Network/networkInterfaces@2416-12-01
Fake.Network/networkInterfaces@2417-03-01
Fake.Network/networkInterfaces@2417-06-01
Fake.Network/networkInterfaces@2417-08-01
Fake.Network/networkInterfaces@2417-09-01
Fake.Network/networkInterfaces@2417-10-01
Fake.Network/networkInterfaces@2417-11-01
Fake.Network/networkInterfaces@2415-05-01-preview
Fake.Network/networkInterfaces@2415-06-15
Fake.Network/networkInterfaces@2416-03-30
Fake.Network/networkInterfaces@2418-01-01
Fake.Network/networkInterfaces@2418-02-01
Fake.Network/networkInterfaces@2418-04-01
Fake.Network/networkInterfaces@2418-06-01
Fake.Network/networkInterfaces@2418-07-01
Fake.Network/networkInterfaces@2418-08-01
Fake.Network/networkInterfaces@2418-10-01
Fake.Network/networkInterfaces@2418-12-01
Fake.Network/networkInterfaces@2419-02-01
Fake.Network/networkInterfaces@2419-04-01
Fake.Network/networkInterfaces@2419-06-01
Fake.Network/networkInterfaces@2419-07-01
Fake.Network/networkInterfaces@2419-08-01
Fake.Network/networkInterfaces@2419-09-01
Fake.Network/networkInterfaces@2418-11-01
Fake.Network/networkInterfaces@2419-11-01
Fake.Network/networkInterfaces@2419-12-01
Fake.Network/networkInterfaces@2420-03-01
Fake.Network/networkInterfaces@2420-04-01
Fake.Network/networkInterfaces@2420-05-01
Fake.Network/networkInterfaces@2420-06-01
Fake.Network/networkInterfaces@2420-08-01
Fake.Network/networkInterfaces@2420-11-01
Fake.Network/routeTables@2416-06-01
Fake.Network/routeTables@2416-07-01
Fake.Network/routeTables@2416-08-01
Fake.Network/routeTables@2416-09-01
Fake.Network/routeTables@2416-10-01
Fake.Network/routeTables@2416-11-01
Fake.Network/routeTables@2416-12-01
Fake.Network/routeTables@2417-03-01
Fake.Network/routeTables@2417-06-01
Fake.Network/routeTables@2417-08-01
Fake.Network/routeTables@2417-09-01
Fake.Network/routeTables@2417-10-01
Fake.Network/routeTables@2417-11-01
Fake.Network/routeTables@2415-05-01-preview
Fake.Network/routeTables@2415-06-15
Fake.Network/routeTables@2416-03-30
Fake.Network/routeTables@2418-01-01
Fake.Network/routeTables@2418-02-01
Fake.Network/routeTables@2418-04-01
Fake.Network/routeTables@2418-06-01
Fake.Network/routeTables@2418-07-01
Fake.Network/routeTables@2418-08-01
Fake.Network/routeTables@2418-10-01
Fake.Network/routeTables@2418-12-01
Fake.Network/routeTables@2419-02-01
Fake.Network/routeTables@2419-04-01
Fake.Network/routeTables@2419-06-01
Fake.Network/routeTables@2419-07-01
Fake.Network/routeTables@2419-08-01
Fake.Network/routeTables@2419-09-01
Fake.Network/routeTables@2418-11-01
Fake.Network/routeTables@2419-11-01
Fake.Network/routeTables@2419-12-01
Fake.Network/routeTables@2420-03-01
Fake.Network/routeTables@2420-04-01
Fake.Network/routeTables@2420-05-01
Fake.Network/routeTables@2420-06-01
Fake.Network/routeTables@2420-08-01
Fake.Network/routeTables@2420-11-01
Fake.Network/applicationGateways@2417-06-01
Fake.Network/applicationGateways@2417-08-01
Fake.Network/applicationGateways@2417-09-01
Fake.Network/applicationGateways@2417-10-01
Fake.Network/applicationGateways@2417-11-01
Fake.Network/applicationGateways@2415-05-01-preview
Fake.Network/applicationGateways@2415-06-15
Fake.Network/applicationGateways@2416-03-30
Fake.Network/applicationGateways@2418-01-01
Fake.Network/applicationGateways@2418-04-01
Fake.Network/applicationGateways@2418-06-01
Fake.Network/applicationGateways@2418-07-01
Fake.Network/applicationGateways@2418-08-01
Fake.Network/applicationGateways@2418-10-01
Fake.Network/applicationGateways@2418-12-01
Fake.Network/applicationGateways@2419-02-01
Fake.Network/applicationGateways@2419-04-01
Fake.Network/applicationGateways@2419-06-01
Fake.Network/applicationGateways@2419-07-01
Fake.Network/applicationGateways@2419-08-01
Fake.Network/applicationGateways@2419-09-01
Fake.Network/applicationGateways@2418-11-01
Fake.Network/applicationGateways@2418-02-01
Fake.Network/applicationGateways@2419-11-01
Fake.Network/applicationGateways@2419-12-01
Fake.Network/applicationGateways@2420-03-01
Fake.Network/applicationGateways@2420-04-01
Fake.Network/applicationGateways@2420-05-01
Fake.Network/applicationGateways@2420-06-01
Fake.Network/applicationGateways@2420-08-01
Fake.Network/applicationGateways@2420-11-01
Fake.Network/connections@2417-06-01
Fake.Network/connections@2417-08-01
Fake.Network/connections@2417-09-01
Fake.Network/connections@2417-10-01
Fake.Network/connections@2417-11-01
Fake.Network/connections@2415-05-01-preview
Fake.Network/connections@2415-06-15
Fake.Network/connections@2416-03-30
Fake.Network/connections@2418-01-01
Fake.Network/connections@2418-04-01
Fake.Network/connections@2418-06-01
Fake.Network/connections@2418-07-01
Fake.Network/connections@2418-08-01
Fake.Network/connections@2418-12-01
Fake.Network/connections@2419-04-01
Fake.Network/connections@2419-06-01
Fake.Network/connections@2419-07-01
Fake.Network/connections@2419-08-01
Fake.Network/connections@2419-09-01
Fake.Network/connections@2419-02-01
Fake.Network/connections@2418-11-01
Fake.Network/connections@2418-10-01
Fake.Network/connections@2418-02-01
Fake.Network/connections@2419-11-01
Fake.Network/connections@2419-12-01
Fake.Network/connections@2420-03-01
Fake.Network/connections@2420-04-01
Fake.Network/connections@2420-05-01
Fake.Network/connections@2420-06-01
Fake.Network/connections@2420-08-01
Fake.Network/connections@2420-11-01
Fake.Network/localNetworkGateways@2417-06-01
Fake.Network/localNetworkGateways@2417-08-01
Fake.Network/localNetworkGateways@2417-09-01
Fake.Network/localNetworkGateways@2417-10-01
Fake.Network/localNetworkGateways@2417-11-01
Fake.Network/localNetworkGateways@2415-05-01-preview
Fake.Network/localNetworkGateways@2415-06-15
Fake.Network/localNetworkGateways@2416-03-30
Fake.Network/localNetworkGateways@2418-01-01
Fake.Network/localNetworkGateways@2418-04-01
Fake.Network/localNetworkGateways@2418-06-01
Fake.Network/localNetworkGateways@2418-07-01
Fake.Network/localNetworkGateways@2418-08-01
Fake.Network/localNetworkGateways@2418-12-01
Fake.Network/localNetworkGateways@2419-04-01
Fake.Network/localNetworkGateways@2419-06-01
Fake.Network/localNetworkGateways@2419-07-01
Fake.Network/localNetworkGateways@2419-08-01
Fake.Network/localNetworkGateways@2419-09-01
Fake.Network/localNetworkGateways@2419-02-01
Fake.Network/localNetworkGateways@2418-11-01
Fake.Network/localNetworkGateways@2418-10-01
Fake.Network/localNetworkGateways@2418-02-01
Fake.Network/localNetworkGateways@2419-11-01
Fake.Network/localNetworkGateways@2419-12-01
Fake.Network/localNetworkGateways@2420-03-01
Fake.Network/localNetworkGateways@2420-04-01
Fake.Network/localNetworkGateways@2420-05-01
Fake.Network/localNetworkGateways@2420-06-01
Fake.Network/localNetworkGateways@2420-08-01
Fake.Network/localNetworkGateways@2420-11-01
Fake.Network/virtualNetworkGateways@2417-06-01
Fake.Network/virtualNetworkGateways@2417-08-01
Fake.Network/virtualNetworkGateways@2417-09-01
Fake.Network/virtualNetworkGateways@2417-10-01
Fake.Network/virtualNetworkGateways@2417-11-01
Fake.Network/virtualNetworkGateways@2415-05-01-preview
Fake.Network/virtualNetworkGateways@2415-06-15
Fake.Network/virtualNetworkGateways@2416-03-30
Fake.Network/virtualNetworkGateways@2418-01-01
Fake.Network/virtualNetworkGateways@2418-04-01
Fake.Network/virtualNetworkGateways@2418-06-01
Fake.Network/virtualNetworkGateways@2418-07-01
Fake.Network/virtualNetworkGateways@2418-08-01
Fake.Network/virtualNetworkGateways@2418-12-01
Fake.Network/virtualNetworkGateways@2419-04-01
Fake.Network/virtualNetworkGateways@2419-06-01
Fake.Network/virtualNetworkGateways@2419-07-01
Fake.Network/virtualNetworkGateways@2419-08-01
Fake.Network/virtualNetworkGateways@2419-09-01
Fake.Network/virtualNetworkGateways@2419-02-01
Fake.Network/virtualNetworkGateways@2418-11-01
Fake.Network/virtualNetworkGateways@2418-10-01
Fake.Network/virtualNetworkGateways@2418-02-01
Fake.Network/virtualNetworkGateways@2419-11-01
Fake.Network/virtualNetworkGateways@2419-12-01
Fake.Network/virtualNetworkGateways@2420-03-01
Fake.Network/virtualNetworkGateways@2420-04-01
Fake.Network/virtualNetworkGateways@2420-05-01
Fake.Network/virtualNetworkGateways@2420-06-01
Fake.Network/virtualNetworkGateways@2420-08-01
Fake.Network/virtualNetworkGateways@2420-11-01
Fake.Network/virtualNetworks/subnets@2417-06-01
Fake.Network/virtualNetworks/subnets@2417-08-01
Fake.Network/virtualNetworks/subnets@2417-09-01
Fake.Network/virtualNetworks/subnets@2417-10-01
Fake.Network/virtualNetworks/subnets@2417-11-01
Fake.Network/virtualNetworks/subnets@2415-05-01-preview
Fake.Network/virtualNetworks/subnets@2415-06-15
Fake.Network/virtualNetworks/subnets@2416-03-30
Fake.Network/virtualNetworks/subnets@2418-01-01
Fake.Network/virtualNetworks/subnets@2418-04-01
Fake.Network/virtualNetworks/subnets@2418-06-01
Fake.Network/virtualNetworks/subnets@2418-07-01
Fake.Network/virtualNetworks/subnets@2418-08-01
Fake.Network/virtualNetworks/subnets@2418-12-01
Fake.Network/virtualNetworks/subnets@2419-04-01
Fake.Network/virtualNetworks/subnets@2419-06-01
Fake.Network/virtualNetworks/subnets@2419-06-01
Fake.Network/virtualNetworks/subnets@2419-07-01
Fake.Network/virtualNetworks/subnets@2419-08-01
Fake.Network/virtualNetworks/subnets@2419-09-01
Fake.Network/virtualNetworks/subnets@2419-02-01
Fake.Network/virtualNetworks/subnets@2418-11-01
Fake.Network/virtualNetworks/subnets@2418-10-01
Fake.Network/virtualNetworks/subnets@2418-02-01
Fake.Network/virtualNetworks/subnets@2419-11-01
Fake.Network/virtualNetworks/subnets@2419-12-01
Fake.Network/virtualNetworks/subnets@2420-03-01
Fake.Network/virtualNetworks/subnets@2420-04-01
Fake.Network/virtualNetworks/subnets@2420-05-01
Fake.Network/virtualNetworks/subnets@2420-06-01
Fake.Network/virtualNetworks/subnets@2420-08-01
Fake.Network/virtualNetworks/subnets@2420-11-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2417-06-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2417-08-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2417-09-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2417-10-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2417-11-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-01-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-04-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-06-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-07-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-08-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-12-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-04-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-06-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-06-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-07-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-08-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-09-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-02-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-11-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-10-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2418-02-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-11-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2419-12-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-03-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-04-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-05-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-06-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-08-01
Fake.Network/virtualNetworks/virtualNetworkPeerings@2420-11-01
Fake.Network/dnsZones/CAA@2417-09-01
Fake.Network/dnsZones/CAA@2417-10-01
Fake.Network/dnsZones/CAA@2418-05-01
Fake.Compute/virtualMachineScaleSets/extensions@2417-12-01
Fake.Compute/virtualMachineScaleSets/extensions@2418-06-01
Fake.Compute/virtualMachineScaleSets/extensions@2418-10-01
Fake.Compute/virtualMachineScaleSets/extensions@2419-03-01
Fake.Compute/virtualMachineScaleSets/extensions@2419-07-01
Fake.Compute/virtualMachineScaleSets/extensions@2420-06-01
Fake.Compute/virtualMachineScaleSets/extensions@2420-12-01
Fake.Compute/virtualMachineScaleSets/extensions@2421-03-01
Fake.Compute/virtualMachineScaleSets/extensions@2421-04-01
Fake.Compute/virtualMachineScaleSets/extensions@2421-07-01
Fake.Network/expressRouteCircuits/authorizations@2415-05-01-preview
Fake.Network/expressRouteCircuits/authorizations@2415-06-15
Fake.Network/expressRouteCircuits/authorizations@2416-03-30
Fake.Network/expressRouteCircuits/authorizations@2418-04-01
Fake.Network/expressRouteCircuits/authorizations@2418-06-01
Fake.Network/expressRouteCircuits/authorizations@2418-07-01
Fake.Network/expressRouteCircuits/authorizations@2418-08-01
Fake.Network/expressRouteCircuits/authorizations@2418-10-01
Fake.Network/expressRouteCircuits/authorizations@2418-12-01
Fake.Network/expressRouteCircuits/authorizations@2419-02-01
Fake.Network/expressRouteCircuits/authorizations@2419-04-01
Fake.Network/expressRouteCircuits/authorizations@2419-06-01
Fake.Network/expressRouteCircuits/authorizations@2419-07-01
Fake.Network/expressRouteCircuits/authorizations@2419-08-01
Fake.Network/expressRouteCircuits/authorizations@2419-09-01
Fake.Network/expressRouteCircuits/authorizations@2418-11-01
Fake.Network/expressRouteCircuits/authorizations@2419-11-01
Fake.Network/expressRouteCircuits/authorizations@2419-12-01
Fake.Network/expressRouteCircuits/authorizations@2420-03-01
Fake.Network/expressRouteCircuits/authorizations@2420-04-01
Fake.Network/expressRouteCircuits/authorizations@2420-05-01
Fake.Network/expressRouteCircuits/authorizations@2420-06-01
Fake.Network/expressRouteCircuits/authorizations@2420-08-01
Fake.Network/expressRouteCircuits/authorizations@2420-11-01
Fake.Network/expressRouteCircuits/peerings@2415-05-01-preview
Fake.Network/expressRouteCircuits/peerings@2415-06-15
Fake.Network/expressRouteCircuits/peerings@2416-03-30
Fake.Network/expressRouteCircuits/peerings@2418-04-01
Fake.Network/expressRouteCircuits/peerings@2418-06-01
Fake.Network/expressRouteCircuits/peerings@2418-07-01
Fake.Network/expressRouteCircuits/peerings@2419-04-01
Fake.Network/expressRouteCircuits/peerings@2419-06-01
Fake.Network/expressRouteCircuits/peerings@2419-07-01
Fake.Network/expressRouteCircuits/peerings@2419-08-01
Fake.Network/expressRouteCircuits/peerings@2419-09-01
Fake.Network/expressRouteCircuits/peerings@2419-11-01
Fake.Network/expressRouteCircuits/peerings@2419-12-01
Fake.Network/expressRouteCircuits/peerings@2420-03-01
Fake.Network/expressRouteCircuits/peerings@2420-04-01
Fake.Network/expressRouteCircuits/peerings@2420-05-01
Fake.Network/expressRouteCircuits/peerings@2420-06-01
Fake.Network/expressRouteCircuits/peerings@2420-08-01
Fake.Network/expressRouteCircuits/peerings@2420-11-01
Fake.Network/networkSecurityGroups/securityRules@2415-05-01-preview
Fake.Network/networkSecurityGroups/securityRules@2415-06-15
Fake.Network/networkSecurityGroups/securityRules@2416-03-30
Fake.Network/networkSecurityGroups/securityRules@2418-04-01
Fake.Network/networkSecurityGroups/securityRules@2418-06-01
Fake.Network/networkSecurityGroups/securityRules@2418-07-01
Fake.Network/networkSecurityGroups/securityRules@2419-04-01
Fake.Network/networkSecurityGroups/securityRules@2419-06-01
Fake.Network/networkSecurityGroups/securityRules@2419-07-01
Fake.Network/networkSecurityGroups/securityRules@2419-08-01
Fake.Network/networkSecurityGroups/securityRules@2419-09-01
Fake.Network/networkSecurityGroups/securityRules@2419-11-01
Fake.Network/networkSecurityGroups/securityRules@2419-12-01
Fake.Network/networkSecurityGroups/securityRules@2420-03-01
Fake.Network/networkSecurityGroups/securityRules@2420-04-01
Fake.Network/networkSecurityGroups/securityRules@2420-05-01
Fake.Network/networkSecurityGroups/securityRules@2420-06-01
Fake.Network/networkSecurityGroups/securityRules@2420-08-01
Fake.Network/networkSecurityGroups/securityRules@2420-11-01
Fake.Network/routeTables/routes@2415-05-01-preview
Fake.Network/routeTables/routes@2415-06-15
Fake.Network/routeTables/routes@2416-03-30
Fake.Network/routeTables/routes@2418-04-01
Fake.Network/routeTables/routes@2418-06-01
Fake.Network/routeTables/routes@2418-07-01
Fake.Network/routeTables/routes@2419-04-01
Fake.Network/routeTables/routes@2419-06-01
Fake.Network/routeTables/routes@2419-07-01
Fake.Network/routeTables/routes@2419-08-01
Fake.Network/routeTables/routes@2419-09-01
Fake.Network/routeTables/routes@2419-11-01
Fake.Network/routeTables/routes@2419-12-01
Fake.Network/routeTables/routes@2420-03-01
Fake.Network/routeTables/routes@2420-04-01
Fake.Network/routeTables/routes@2420-05-01
Fake.Network/routeTables/routes@2420-06-01
Fake.Network/routeTables/routes@2420-08-01
Fake.Network/routeTables/routes@2420-11-01
Fake.Network/expressRouteCircuits@2415-06-15
Fake.Network/expressRouteCircuits@2416-03-30
Fake.Network/expressRouteCircuits@2418-02-01
Fake.Network/expressRouteCircuits@2418-04-01
Fake.Network/expressRouteCircuits@2418-06-01
Fake.Network/expressRouteCircuits@2418-07-01
Fake.Network/expressRouteCircuits@2418-08-01
Fake.Network/expressRouteCircuits@2418-10-01
Fake.Network/expressRouteCircuits@2418-12-01
Fake.Network/expressRouteCircuits@2419-02-01
Fake.Network/expressRouteCircuits@2419-04-01
Fake.Network/expressRouteCircuits@2419-06-01
Fake.Network/expressRouteCircuits@2419-07-01
Fake.Network/expressRouteCircuits@2419-08-01
Fake.Network/expressRouteCircuits@2419-09-01
Fake.Network/expressRouteCircuits@2418-11-01
Fake.Network/expressRouteCircuits@2419-11-01
Fake.Network/expressRouteCircuits@2419-12-01
Fake.Network/expressRouteCircuits@2420-03-01
Fake.Network/expressRouteCircuits@2420-04-01
Fake.Network/expressRouteCircuits@2420-05-01
Fake.Network/expressRouteCircuits@2420-06-01
Fake.Network/expressRouteCircuits@2420-08-01
Fake.Network/expressRouteCircuits@2420-11-01
Fake.Network/applicationSecurityGroups@2417-09-01
Fake.Network/applicationSecurityGroups@2417-10-01
Fake.Network/applicationSecurityGroups@2417-11-01
Fake.Network/applicationSecurityGroups@2418-01-01
Fake.Network/applicationSecurityGroups@2418-02-01
Fake.Network/applicationSecurityGroups@2418-04-01
Fake.Network/applicationSecurityGroups@2418-06-01
Fake.Network/applicationSecurityGroups@2418-07-01
Fake.Network/applicationSecurityGroups@2418-08-01
Fake.Network/applicationSecurityGroups@2418-10-01
Fake.Network/applicationSecurityGroups@2418-11-01
Fake.Network/applicationSecurityGroups@2418-12-01
Fake.Network/applicationSecurityGroups@2419-02-01
Fake.Network/applicationSecurityGroups@2419-04-01
Fake.Network/applicationSecurityGroups@2419-06-01
Fake.Network/applicationSecurityGroups@2419-07-01
Fake.Network/applicationSecurityGroups@2419-08-01
Fake.Network/applicationSecurityGroups@2419-09-01
Fake.Network/applicationSecurityGroups@2419-11-01
Fake.Network/applicationSecurityGroups@2419-12-01
Fake.Network/applicationSecurityGroups@2420-03-01
Fake.Network/applicationSecurityGroups@2420-04-01
Fake.Network/applicationSecurityGroups@2420-05-01
Fake.Network/applicationSecurityGroups@2420-06-01
Fake.Network/applicationSecurityGroups@2420-08-01
Fake.Network/applicationSecurityGroups@2420-11-01
Fake.Network/ddosProtectionPlans@2418-02-01
Fake.Network/ddosProtectionPlans@2418-04-01
Fake.Network/ddosProtectionPlans@2418-06-01
Fake.Network/ddosProtectionPlans@2418-07-01
Fake.Network/ddosProtectionPlans@2418-08-01
Fake.Network/ddosProtectionPlans@2418-10-01
Fake.Network/ddosProtectionPlans@2418-12-01
Fake.Network/ddosProtectionPlans@2419-02-01
Fake.Network/ddosProtectionPlans@2419-04-01
Fake.Network/ddosProtectionPlans@2419-06-01
Fake.Network/ddosProtectionPlans@2419-07-01
Fake.Network/ddosProtectionPlans@2419-08-01
Fake.Network/ddosProtectionPlans@2419-09-01
Fake.Network/ddosProtectionPlans@2418-11-01
Fake.Network/ddosProtectionPlans@2419-11-01
Fake.Network/ddosProtectionPlans@2419-12-01
Fake.Network/ddosProtectionPlans@2420-03-01
Fake.Network/ddosProtectionPlans@2420-04-01
Fake.Network/ddosProtectionPlans@2420-05-01
Fake.Network/ddosProtectionPlans@2420-06-01
Fake.Network/ddosProtectionPlans@2420-08-01
Fake.Network/ddosProtectionPlans@2420-11-01
Fake.Network/expressRouteCrossConnections@2418-02-01
Fake.Network/expressRouteCrossConnections@2418-04-01
Fake.Network/expressRouteCrossConnections@2418-06-01
Fake.Network/expressRouteCrossConnections@2418-07-01
Fake.Network/expressRouteCrossConnections@2418-08-01
Fake.Network/expressRouteCrossConnections@2418-10-01
Fake.Network/expressRouteCrossConnections@2418-12-01
Fake.Network/expressRouteCrossConnections@2419-02-01
Fake.Network/expressRouteCrossConnections@2419-04-01
Fake.Network/expressRouteCrossConnections@2419-06-01
Fake.Network/expressRouteCrossConnections@2419-07-01
Fake.Network/expressRouteCrossConnections@2419-08-01
Fake.Network/expressRouteCrossConnections@2419-09-01
Fake.Network/expressRouteCrossConnections@2418-11-01
Fake.Network/expressRouteCrossConnections@2419-11-01
Fake.Network/expressRouteCrossConnections@2419-12-01
Fake.Network/expressRouteCrossConnections@2420-03-01
Fake.Network/expressRouteCrossConnections@2420-04-01
Fake.Network/expressRouteCrossConnections@2420-05-01
Fake.Network/expressRouteCrossConnections@2420-06-01
Fake.Network/expressRouteCrossConnections@2420-08-01
Fake.Network/expressRouteCrossConnections@2420-11-01
Fake.Network/azureFirewalls@2418-04-01
Fake.Network/azureFirewalls@2418-06-01
Fake.Network/azureFirewalls@2418-07-01
Fake.Network/azureFirewalls@2419-04-01
Fake.Network/azureFirewalls@2419-06-01
Fake.Network/azureFirewalls@2419-07-01
Fake.Network/azureFirewalls@2419-08-01
Fake.Network/azureFirewalls@2419-09-01
Fake.Network/azureFirewalls@2419-11-01
Fake.Network/azureFirewalls@2419-12-01
Fake.Network/azureFirewalls@2420-03-01
Fake.Network/azureFirewalls@2420-04-01
Fake.Network/azureFirewalls@2420-05-01
Fake.Network/azureFirewalls@2420-06-01
Fake.Network/azureFirewalls@2420-08-01
Fake.Network/azureFirewalls@2420-11-01
Fake.Network/expressRouteCircuits/peerings/connections@2418-04-01
Fake.Network/expressRouteCircuits/peerings/connections@2418-06-01
Fake.Network/expressRouteCircuits/peerings/connections@2418-07-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-04-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-06-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-07-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-08-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-09-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-11-01
Fake.Network/expressRouteCircuits/peerings/connections@2419-12-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-03-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-04-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-05-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-06-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-08-01
Fake.Network/expressRouteCircuits/peerings/connections@2420-11-01
Fake.Network/expressRouteCrossConnections/peerings@2418-04-01
Fake.Network/expressRouteCrossConnections/peerings@2418-06-01
Fake.Network/expressRouteCrossConnections/peerings@2418-07-01
Fake.Network/expressRouteCrossConnections/peerings@2419-04-01
Fake.Network/expressRouteCrossConnections/peerings@2419-06-01
Fake.Network/expressRouteCrossConnections/peerings@2419-07-01
Fake.Network/expressRouteCrossConnections/peerings@2419-08-01
Fake.Network/expressRouteCrossConnections/peerings@2419-09-01
Fake.Network/expressRouteCrossConnections/peerings@2419-11-01
Fake.Network/expressRouteCrossConnections/peerings@2419-12-01
Fake.Network/expressRouteCrossConnections/peerings@2420-03-01
Fake.Network/expressRouteCrossConnections/peerings@2420-04-01
Fake.Network/expressRouteCrossConnections/peerings@2420-05-01
Fake.Network/expressRouteCrossConnections/peerings@2420-06-01
Fake.Network/expressRouteCrossConnections/peerings@2420-08-01
Fake.Network/expressRouteCrossConnections/peerings@2420-11-01
Fake.Network/loadBalancers/inboundNatRules@2418-04-01
Fake.Network/loadBalancers/inboundNatRules@2418-06-01
Fake.Network/loadBalancers/inboundNatRules@2418-07-01
Fake.Network/loadBalancers/inboundNatRules@2419-04-01
Fake.Network/loadBalancers/inboundNatRules@2419-06-01
Fake.Network/loadBalancers/inboundNatRules@2419-07-01
Fake.Network/loadBalancers/inboundNatRules@2419-08-01
Fake.Network/loadBalancers/inboundNatRules@2419-09-01
Fake.Network/loadBalancers/inboundNatRules@2419-11-01
Fake.Network/loadBalancers/inboundNatRules@2419-12-01
Fake.Network/loadBalancers/inboundNatRules@2420-03-01
Fake.Network/loadBalancers/inboundNatRules@2420-04-01
Fake.Network/loadBalancers/inboundNatRules@2420-05-01
Fake.Network/loadBalancers/inboundNatRules@2420-06-01
Fake.Network/loadBalancers/inboundNatRules@2420-08-01
Fake.Network/loadBalancers/inboundNatRules@2420-11-01
Fake.Network/networkWatchers@2418-04-01
Fake.Network/networkWatchers@2418-06-01
Fake.Network/networkWatchers@2418-07-01
Fake.Network/networkWatchers@2419-04-01
Fake.Network/networkWatchers@2419-06-01
Fake.Network/networkWatchers@2419-07-01
Fake.Network/networkWatchers@2419-08-01
Fake.Network/networkWatchers@2419-09-01
Fake.Network/networkWatchers@2419-11-01
Fake.Network/networkWatchers@2419-12-01
Fake.Network/networkWatchers@2420-03-01
Fake.Network/networkWatchers@2420-04-01
Fake.Network/networkWatchers@2420-05-01
Fake.Network/networkWatchers@2420-06-01
Fake.Network/networkWatchers@2420-08-01
Fake.Network/networkWatchers@2420-11-01
Fake.Network/networkWatchers/connectionMonitors@2418-04-01
Fake.Network/networkWatchers/connectionMonitors@2418-06-01
Fake.Network/networkWatchers/connectionMonitors@2418-07-01
Fake.Network/networkWatchers/connectionMonitors@2419-04-01
Fake.Network/networkWatchers/connectionMonitors@2419-06-01
Fake.Network/networkWatchers/connectionMonitors@2419-11-01
Fake.Network/networkWatchers/connectionMonitors@2419-12-01
Fake.Network/networkWatchers/connectionMonitors@2420-03-01
Fake.Network/networkWatchers/connectionMonitors@2420-04-01
Fake.Network/networkWatchers/connectionMonitors@2420-05-01
Fake.Network/networkWatchers/connectionMonitors@2420-06-01
Fake.Network/networkWatchers/connectionMonitors@2420-08-01
Fake.Network/networkWatchers/connectionMonitors@2420-11-01
Fake.Network/networkWatchers/packetCaptures@2418-04-01
Fake.Network/networkWatchers/packetCaptures@2418-06-01
Fake.Network/networkWatchers/packetCaptures@2418-07-01
Fake.Network/networkWatchers/packetCaptures@2419-04-01
Fake.Network/networkWatchers/packetCaptures@2419-06-01
Fake.Network/networkWatchers/packetCaptures@2419-07-01
Fake.Network/networkWatchers/packetCaptures@2419-08-01
Fake.Network/networkWatchers/packetCaptures@2419-09-01
Fake.Network/networkWatchers/packetCaptures@2419-11-01
Fake.Network/networkWatchers/packetCaptures@2419-12-01
Fake.Network/networkWatchers/packetCaptures@2420-03-01
Fake.Network/networkWatchers/packetCaptures@2420-04-01
Fake.Network/networkWatchers/packetCaptures@2420-05-01
Fake.Network/networkWatchers/packetCaptures@2420-06-01
Fake.Network/networkWatchers/packetCaptures@2420-08-01
Fake.Network/networkWatchers/packetCaptures@2420-11-01
Fake.Network/routeFilters@2418-04-01
Fake.Network/routeFilters@2418-06-01
Fake.Network/routeFilters@2418-07-01
Fake.Network/routeFilters@2419-04-01
Fake.Network/routeFilters@2419-06-01
Fake.Network/routeFilters@2419-07-01
Fake.Network/routeFilters@2419-08-01
Fake.Network/routeFilters@2419-09-01
Fake.Network/routeFilters@2419-11-01
Fake.Network/routeFilters@2419-12-01
Fake.Network/routeFilters@2420-03-01
Fake.Network/routeFilters@2420-04-01
Fake.Network/routeFilters@2420-05-01
Fake.Network/routeFilters@2420-06-01
Fake.Network/routeFilters@2420-08-01
Fake.Network/routeFilters@2420-11-01
Fake.Network/routeFilters/routeFilterRules@2418-04-01
Fake.Network/routeFilters/routeFilterRules@2418-06-01
Fake.Network/routeFilters/routeFilterRules@2418-07-01
Fake.Network/routeFilters/routeFilterRules@2419-04-01
Fake.Network/routeFilters/routeFilterRules@2419-06-01
Fake.Network/routeFilters/routeFilterRules@2419-07-01
Fake.Network/routeFilters/routeFilterRules@2419-08-01
Fake.Network/routeFilters/routeFilterRules@2419-09-01
Fake.Network/routeFilters/routeFilterRules@2419-11-01
Fake.Network/routeFilters/routeFilterRules@2419-12-01
Fake.Network/routeFilters/routeFilterRules@2420-03-01
Fake.Network/routeFilters/routeFilterRules@2420-04-01
Fake.Network/routeFilters/routeFilterRules@2420-05-01
Fake.Network/routeFilters/routeFilterRules@2420-06-01
Fake.Network/routeFilters/routeFilterRules@2420-08-01
Fake.Network/routeFilters/routeFilterRules@2420-11-01
Fake.Network/virtualHubs@2418-04-01
Fake.Network/virtualHubs@2418-06-01
Fake.Network/virtualHubs@2418-07-01
Fake.Network/virtualHubs@2419-04-01
Fake.Network/virtualHubs@2419-06-01
Fake.Network/virtualHubs@2419-07-01
Fake.Network/virtualHubs@2419-08-01
Fake.Network/virtualHubs@2419-09-01
Fake.Network/virtualHubs@2419-11-01
Fake.Network/virtualHubs@2419-12-01
Fake.Network/virtualHubs@2420-03-01
Fake.Network/virtualHubs@2420-04-01
Fake.Network/virtualHubs@2420-05-01
Fake.Network/virtualHubs@2420-06-01
Fake.Network/virtualHubs@2420-08-01
Fake.Network/virtualHubs@2420-11-01
Fake.Network/virtualWans@2418-04-01
Fake.Network/virtualWans@2418-06-01
Fake.Network/virtualWans@2418-07-01
Fake.Network/virtualWans@2419-04-01
Fake.Network/virtualWans@2419-06-01
Fake.Network/virtualWans@2419-07-01
Fake.Network/virtualWans@2419-08-01
Fake.Network/virtualWans@2419-09-01
Fake.Network/virtualWans@2419-11-01
Fake.Network/virtualWans@2419-12-01
Fake.Network/virtualWans@2420-03-01
Fake.Network/virtualWans@2420-04-01
Fake.Network/virtualWans@2420-05-01
Fake.Network/virtualWans@2420-06-01
Fake.Network/virtualWans@2420-08-01
Fake.Network/virtualWans@2420-11-01
Fake.Network/vpnGateways@2418-04-01
Fake.Network/vpnGateways@2418-06-01
Fake.Network/vpnGateways@2418-07-01
Fake.Network/vpnGateways@2419-04-01
Fake.Network/vpnGateways@2419-06-01
Fake.Network/vpnGateways@2419-07-01
Fake.Network/vpnGateways@2419-08-01
Fake.Network/vpnGateways@2419-09-01
Fake.Network/vpnGateways@2419-11-01
Fake.Network/vpnGateways@2419-12-01
Fake.Network/vpnGateways@2420-03-01
Fake.Network/vpnGateways@2420-04-01
Fake.Network/vpnGateways@2420-05-01
Fake.Network/vpnGateways@2420-06-01
Fake.Network/vpnGateways@2420-08-01
Fake.Network/vpnGateways@2420-11-01
Fake.Network/vpnGateways/vpnConnections@2418-04-01
Fake.Network/vpnGateways/vpnConnections@2418-06-01
Fake.Network/vpnGateways/vpnConnections@2418-07-01
Fake.Network/vpnGateways/vpnConnections@2419-04-01
Fake.Network/vpnGateways/vpnConnections@2419-06-01
Fake.Network/vpnGateways/vpnConnections@2419-07-01
Fake.Network/vpnGateways/vpnConnections@2419-08-01
Fake.Network/vpnGateways/vpnConnections@2419-09-01
Fake.Network/vpnGateways/vpnConnections@2419-11-01
Fake.Network/vpnGateways/vpnConnections@2419-12-01
Fake.Network/vpnGateways/vpnConnections@2420-03-01
Fake.Network/vpnGateways/vpnConnections@2420-04-01
Fake.Network/vpnGateways/vpnConnections@2420-05-01
Fake.Network/vpnGateways/vpnConnections@2420-06-01
Fake.Network/vpnGateways/vpnConnections@2420-08-01
Fake.Network/vpnGateways/vpnConnections@2420-11-01
Fake.Network/vpnSites@2418-04-01
Fake.Network/vpnSites@2418-06-01
Fake.Network/vpnSites@2418-07-01
Fake.Network/vpnSites@2419-04-01
Fake.Network/vpnSites@2419-06-01
Fake.Network/vpnSites@2419-07-01
Fake.Network/vpnSites@2419-08-01
Fake.Network/vpnSites@2419-09-01
Fake.Network/vpnSites@2419-11-01
Fake.Network/vpnSites@2419-12-01
Fake.Network/vpnSites@2420-03-01
Fake.Network/vpnSites@2420-04-01
Fake.Network/vpnSites@2420-05-01
Fake.Network/vpnSites@2420-06-01
Fake.Network/vpnSites@2420-08-01
Fake.Network/vpnSites@2420-11-01
Fake.Network/publicIPPrefixes@2418-07-01
Fake.Network/publicIPPrefixes@2419-04-01
Fake.Network/publicIPPrefixes@2419-06-01
Fake.Network/publicIPPrefixes@2419-07-01
Fake.Network/publicIPPrefixes@2419-08-01
Fake.Network/publicIPPrefixes@2419-09-01
Fake.Network/publicIPPrefixes@2419-11-01
Fake.Network/publicIPPrefixes@2419-12-01
Fake.Network/publicIPPrefixes@2420-03-01
Fake.Network/publicIPPrefixes@2420-04-01
Fake.Network/publicIPPrefixes@2420-05-01
Fake.Network/publicIPPrefixes@2420-06-01
Fake.Network/publicIPPrefixes@2420-08-01
Fake.Network/publicIPPrefixes@2420-11-01
Fake.Network/serviceEndpointPolicies@2418-07-01
Fake.Network/serviceEndpointPolicies@2419-04-01
Fake.Network/serviceEndpointPolicies@2419-06-01
Fake.Network/serviceEndpointPolicies@2419-07-01
Fake.Network/serviceEndpointPolicies@2419-08-01
Fake.Network/serviceEndpointPolicies@2419-09-01
Fake.Network/serviceEndpointPolicies@2419-11-01
Fake.Network/serviceEndpointPolicies@2419-12-01
Fake.Network/serviceEndpointPolicies@2420-03-01
Fake.Network/serviceEndpointPolicies@2420-04-01
Fake.Network/serviceEndpointPolicies@2420-05-01
Fake.Network/serviceEndpointPolicies@2420-06-01
Fake.Network/serviceEndpointPolicies@2420-08-01
Fake.Network/serviceEndpointPolicies@2420-11-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2418-07-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-04-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-06-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-07-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-08-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-09-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-11-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2419-12-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-03-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-04-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-05-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-06-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-08-01
Fake.Network/serviceEndpointPolicies/serviceEndpointPolicyDefinitions@2420-11-01
Fake.Network/ExpressRoutePorts@2418-08-01
Fake.Network/ExpressRoutePorts@2418-12-01
Fake.Network/ExpressRoutePorts@2419-02-01
Fake.Network/ExpressRoutePorts@2419-04-01
Fake.Network/ExpressRoutePorts@2419-06-01
Fake.Network/ExpressRoutePorts@2419-07-01
Fake.Network/ExpressRoutePorts@2419-08-01
Fake.Network/ExpressRoutePorts@2419-09-01
Fake.Network/ExpressRoutePorts@2418-11-01
Fake.Network/ExpressRoutePorts@2418-10-01
Fake.Network/ExpressRoutePorts@2419-11-01
Fake.Network/ExpressRoutePorts@2419-12-01
Fake.Network/ExpressRoutePorts@2420-03-01
Fake.Network/ExpressRoutePorts@2420-04-01
Fake.Network/ExpressRoutePorts@2420-05-01
Fake.Network/ExpressRoutePorts@2420-06-01
Fake.Network/ExpressRoutePorts@2420-08-01
Fake.Network/ExpressRoutePorts@2420-11-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2418-12-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-02-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-04-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-06-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-07-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-08-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-09-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-11-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2419-12-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-03-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-04-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-05-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-06-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-08-01
Fake.Network/ApplicationGatewayWebApplicationFirewallPolicies@2420-11-01
Fake.Network/bastionHosts@2419-04-01
Fake.Network/bastionHosts@2419-06-01
Fake.Network/bastionHosts@2419-07-01
Fake.Network/bastionHosts@2419-08-01
Fake.Network/bastionHosts@2419-09-01
Fake.Network/bastionHosts@2419-11-01
Fake.Network/bastionHosts@2419-12-01
Fake.Network/bastionHosts@2420-03-01
Fake.Network/bastionHosts@2420-04-01
Fake.Network/bastionHosts@2420-05-01
Fake.Network/bastionHosts@2420-06-01
Fake.Network/bastionHosts@2420-08-01
Fake.Network/bastionHosts@2420-11-01
Fake.Network/ddosCustomPolicies@2419-04-01
Fake.Network/ddosCustomPolicies@2419-06-01
Fake.Network/ddosCustomPolicies@2419-07-01
Fake.Network/ddosCustomPolicies@2419-08-01
Fake.Network/ddosCustomPolicies@2419-09-01
Fake.Network/ddosCustomPolicies@2419-11-01
Fake.Network/ddosCustomPolicies@2419-12-01
Fake.Network/ddosCustomPolicies@2420-03-01
Fake.Network/ddosCustomPolicies@2420-04-01
Fake.Network/ddosCustomPolicies@2420-05-01
Fake.Network/ddosCustomPolicies@2420-06-01
Fake.Network/ddosCustomPolicies@2420-08-01
Fake.Network/ddosCustomPolicies@2420-11-01
Fake.Network/expressRouteGateways@2419-04-01
Fake.Network/expressRouteGateways@2419-06-01
Fake.Network/expressRouteGateways@2419-07-01
Fake.Network/expressRouteGateways@2419-08-01
Fake.Network/expressRouteGateways@2419-09-01
Fake.Network/expressRouteGateways@2419-11-01
Fake.Network/expressRouteGateways@2419-12-01
Fake.Network/expressRouteGateways@2420-03-01
Fake.Network/expressRouteGateways@2420-04-01
Fake.Network/expressRouteGateways@2420-05-01
Fake.Network/expressRouteGateways@2420-06-01
Fake.Network/expressRouteGateways@2420-08-01
Fake.Network/expressRouteGateways@2420-11-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-04-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-06-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-07-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-08-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-09-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-11-01
Fake.Network/expressRouteGateways/expressRouteConnections@2419-12-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-03-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-04-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-05-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-06-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-08-01
Fake.Network/expressRouteGateways/expressRouteConnections@2420-11-01
Fake.Network/natGateways@2419-04-01
Fake.Network/natGateways@2419-06-01
Fake.Network/natGateways@2419-07-01
Fake.Network/natGateways@2419-08-01
Fake.Network/natGateways@2419-09-01
Fake.Network/natGateways@2419-02-01
Fake.Network/natGateways@2419-11-01
Fake.Network/natGateways@2419-12-01
Fake.Network/natGateways@2420-03-01
Fake.Network/natGateways@2420-04-01
Fake.Network/natGateways@2420-05-01
Fake.Network/natGateways@2420-06-01
Fake.Network/natGateways@2420-08-01
Fake.Network/natGateways@2420-11-01
Fake.Network/networkInterfaces/tapConfigurations@2419-04-01
Fake.Network/networkInterfaces/tapConfigurations@2419-06-01
Fake.Network/networkInterfaces/tapConfigurations@2419-07-01
Fake.Network/networkInterfaces/tapConfigurations@2419-08-01
Fake.Network/networkInterfaces/tapConfigurations@2419-09-01
Fake.Network/networkInterfaces/tapConfigurations@2419-11-01
Fake.Network/networkInterfaces/tapConfigurations@2419-12-01
Fake.Network/networkInterfaces/tapConfigurations@2420-03-01
Fake.Network/networkInterfaces/tapConfigurations@2420-04-01
Fake.Network/networkInterfaces/tapConfigurations@2420-05-01
Fake.Network/networkInterfaces/tapConfigurations@2420-06-01
Fake.Network/networkInterfaces/tapConfigurations@2420-08-01
Fake.Network/networkInterfaces/tapConfigurations@2420-11-01
Fake.Network/networkProfiles@2419-04-01
Fake.Network/networkProfiles@2419-06-01
Fake.Network/networkProfiles@2419-07-01
Fake.Network/networkProfiles@2419-08-01
Fake.Network/networkProfiles@2419-09-01
Fake.Network/networkProfiles@2419-11-01
Fake.Network/networkProfiles@2419-12-01
Fake.Network/networkProfiles@2420-03-01
Fake.Network/networkProfiles@2420-04-01
Fake.Network/networkProfiles@2420-05-01
Fake.Network/networkProfiles@2420-06-01
Fake.Network/networkProfiles@2420-08-01
Fake.Network/networkProfiles@2420-11-01
Fake.Network/p2svpnGateways@2419-04-01
Fake.Network/p2svpnGateways@2419-06-01
Fake.Network/p2svpnGateways@2419-07-01
Fake.Network/p2svpnGateways@2419-08-01
Fake.Network/p2svpnGateways@2419-09-01
Fake.Network/p2svpnGateways@2419-11-01
Fake.Network/p2svpnGateways@2419-12-01
Fake.Network/p2svpnGateways@2420-03-01
Fake.Network/p2svpnGateways@2420-04-01
Fake.Network/p2svpnGateways@2420-05-01
Fake.Network/p2svpnGateways@2420-06-01
Fake.Network/p2svpnGateways@2420-08-01
Fake.Network/p2svpnGateways@2420-11-01
Fake.Network/privateEndpoints@2419-04-01
Fake.Network/privateEndpoints@2419-06-01
Fake.Network/privateEndpoints@2419-07-01
Fake.Network/privateEndpoints@2419-08-01
Fake.Network/privateEndpoints@2419-09-01
Fake.Network/privateEndpoints@2419-11-01
Fake.Network/privateEndpoints@2419-12-01
Fake.Network/privateEndpoints@2420-03-01
Fake.Network/privateEndpoints@2420-04-01
Fake.Network/privateEndpoints@2420-05-01
Fake.Network/privateEndpoints@2420-06-01
Fake.Network/privateEndpoints@2420-08-01
Fake.Network/privateEndpoints@2420-11-01
Fake.Network/privateLinkServices@2419-04-01
Fake.Network/privateLinkServices@2419-06-01
Fake.Network/privateLinkServices@2419-07-01
Fake.Network/privateLinkServices@2419-08-01
Fake.Network/privateLinkServices@2419-09-01
Fake.Network/privateLinkServices@2419-11-01
Fake.Network/privateLinkServices@2419-12-01
Fake.Network/privateLinkServices@2420-03-01
Fake.Network/privateLinkServices@2420-04-01
Fake.Network/privateLinkServices@2420-05-01
Fake.Network/privateLinkServices@2420-06-01
Fake.Network/privateLinkServices@2420-08-01
Fake.Network/privateLinkServices@2420-11-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-04-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-06-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-07-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-08-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-09-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-11-01
Fake.Network/privateLinkServices/privateEndpointConnections@2419-12-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-03-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-04-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-05-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-06-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-08-01
Fake.Network/privateLinkServices/privateEndpointConnections@2420-11-01
Fake.Network/virtualNetworkTaps@2419-04-01
Fake.Network/virtualNetworkTaps@2419-06-01
Fake.Network/virtualNetworkTaps@2419-07-01
Fake.Network/virtualNetworkTaps@2419-08-01
Fake.Network/virtualNetworkTaps@2419-09-01
Fake.Network/virtualNetworkTaps@2419-11-01
Fake.Network/virtualNetworkTaps@2419-12-01
Fake.Network/virtualNetworkTaps@2420-03-01
Fake.Network/virtualNetworkTaps@2420-04-01
Fake.Network/virtualNetworkTaps@2420-05-01
Fake.Network/virtualNetworkTaps@2420-06-01
Fake.Network/virtualNetworkTaps@2420-08-01
Fake.Network/virtualNetworkTaps@2420-11-01
Fake.Network/virtualWans/p2sVpnServerConfigurations@2419-04-01
Fake.Network/virtualWans/p2sVpnServerConfigurations@2419-06-01
Fake.Network/virtualWans/p2sVpnServerConfigurations@2419-07-01
Fake.Network/firewallPolicies@2419-06-01
Fake.Network/firewallPolicies@2419-07-01
Fake.Network/firewallPolicies@2419-08-01
Fake.Network/firewallPolicies@2419-09-01
Fake.Network/firewallPolicies@2419-11-01
Fake.Network/firewallPolicies@2419-12-01
Fake.Network/firewallPolicies@2420-03-01
Fake.Network/firewallPolicies@2420-04-01
Fake.Network/firewallPolicies@2420-05-01
Fake.Network/firewallPolicies@2420-06-01
Fake.Network/firewallPolicies@2420-08-01
Fake.Network/firewallPolicies@2420-11-01
Fake.Network/firewallPolicies/ruleGroups@2419-06-01
Fake.Network/firewallPolicies/ruleGroups@2419-07-01
Fake.Network/firewallPolicies/ruleGroups@2419-08-01
Fake.Network/firewallPolicies/ruleGroups@2419-09-01
Fake.Network/firewallPolicies/ruleGroups@2419-11-01
Fake.Network/firewallPolicies/ruleGroups@2419-12-01
Fake.Network/firewallPolicies/ruleGroups@2420-03-01
Fake.Network/firewallPolicies/ruleGroups@2420-04-01
Fake.Network/virtualRouters@2419-07-01
Fake.Network/virtualRouters@2419-08-01
Fake.Network/virtualRouters@2419-09-01
Fake.Network/virtualRouters@2419-11-01
Fake.Network/virtualRouters@2419-12-01
Fake.Network/virtualRouters@2420-03-01
Fake.Network/virtualRouters@2420-04-01
Fake.Network/virtualRouters@2420-05-01
Fake.Network/virtualRouters@2420-06-01
Fake.Network/virtualRouters@2420-08-01
Fake.Network/virtualRouters@2420-11-01
Fake.Network/virtualRouters/peerings@2419-07-01
Fake.Network/virtualRouters/peerings@2419-08-01
Fake.Network/virtualRouters/peerings@2419-09-01
Fake.Network/virtualRouters/peerings@2419-11-01
Fake.Network/virtualRouters/peerings@2419-12-01
Fake.Network/virtualRouters/peerings@2420-03-01
Fake.Network/virtualRouters/peerings@2420-04-01
Fake.Network/virtualRouters/peerings@2420-05-01
Fake.Network/virtualRouters/peerings@2420-06-01
Fake.Network/virtualRouters/peerings@2420-08-01
Fake.Network/virtualRouters/peerings@2420-11-01
Fake.Network/vpnServerConfigurations@2419-08-01
Fake.Network/vpnServerConfigurations@2419-09-01
Fake.Network/vpnServerConfigurations@2419-11-01
Fake.Network/vpnServerConfigurations@2419-12-01
Fake.Network/vpnServerConfigurations@2420-03-01
Fake.Network/vpnServerConfigurations@2420-04-01
Fake.Network/vpnServerConfigurations@2420-05-01
Fake.Network/vpnServerConfigurations@2420-06-01
Fake.Network/vpnServerConfigurations@2420-08-01
Fake.Network/vpnServerConfigurations@2420-11-01
Fake.Network/ipGroups@2419-09-01
Fake.Network/ipGroups@2419-11-01
Fake.Network/ipGroups@2419-12-01
Fake.Network/ipGroups@2420-03-01
Fake.Network/ipGroups@2420-04-01
Fake.Network/ipGroups@2420-05-01
Fake.Network/ipGroups@2420-06-01
Fake.Network/ipGroups@2420-08-01
Fake.Network/ipGroups@2420-11-01
Fake.Network/virtualHubs/routeTables@2419-09-01
Fake.Network/virtualHubs/routeTables@2419-11-01
Fake.Network/virtualHubs/routeTables@2419-12-01
Fake.Network/virtualHubs/routeTables@2420-03-01
Fake.Network/virtualHubs/routeTables@2420-04-01
Fake.Network/virtualHubs/routeTables@2420-05-01
Fake.Network/virtualHubs/routeTables@2420-06-01
Fake.Network/virtualHubs/routeTables@2420-08-01
Fake.Network/virtualHubs/routeTables@2420-11-01
Fake.Network/privateDnsZones@2418-09-01
Fake.Network/privateDnsZones/virtualNetworkLinks@2418-09-01
Fake.Network/privateDnsZones/A@2418-09-01
Fake.Network/privateDnsZones/AAAA@2418-09-01
Fake.Network/privateDnsZones/CNAME@2418-09-01
Fake.Network/privateDnsZones/MX@2418-09-01
Fake.Network/privateDnsZones/PTR@2418-09-01
Fake.Network/privateDnsZones/SOA@2418-09-01
Fake.Network/privateDnsZones/SRV@2418-09-01
Fake.Network/privateDnsZones/TXT@2418-09-01
Fake.Network/networkWatchers/flowLogs@2419-11-01
Fake.Network/networkWatchers/flowLogs@2419-12-01
Fake.Network/networkWatchers/flowLogs@2420-03-01
Fake.Network/networkWatchers/flowLogs@2420-04-01
Fake.Network/networkWatchers/flowLogs@2420-05-01
Fake.Network/networkWatchers/flowLogs@2420-06-01
Fake.Network/networkWatchers/flowLogs@2420-08-01
Fake.Network/networkWatchers/flowLogs@2420-11-01
Fake.Network/connections/sharedkey@2419-12-01
Fake.Network/networkVirtualAppliances@2419-12-01
Fake.Network/networkVirtualAppliances@2420-03-01
Fake.Network/networkVirtualAppliances@2420-04-01
Fake.Network/networkVirtualAppliances@2420-05-01
Fake.Network/networkVirtualAppliances@2420-06-01
Fake.Network/networkVirtualAppliances@2420-08-01
Fake.Network/networkVirtualAppliances@2420-11-01
Fake.Network/IpAllocations@2420-03-01
Fake.Network/IpAllocations@2420-04-01
Fake.Network/IpAllocations@2420-05-01
Fake.Network/IpAllocations@2420-06-01
Fake.Network/IpAllocations@2420-08-01
Fake.Network/IpAllocations@2420-11-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-03-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-04-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-05-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-06-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-08-01
Fake.Network/privateEndpoints/privateDnsZoneGroups@2420-11-01
Fake.Network/securityPartnerProviders@2420-03-01
Fake.Network/securityPartnerProviders@2420-04-01
Fake.Network/securityPartnerProviders@2420-05-01
Fake.Network/securityPartnerProviders@2420-06-01
Fake.Network/securityPartnerProviders@2420-08-01
Fake.Network/securityPartnerProviders@2420-11-01
Fake.Network/loadBalancers/backendAddressPools@2420-04-01
Fake.Network/loadBalancers/backendAddressPools@2420-05-01
Fake.Network/loadBalancers/backendAddressPools@2420-06-01
Fake.Network/loadBalancers/backendAddressPools@2420-08-01
Fake.Network/loadBalancers/backendAddressPools@2420-11-01
Fake.Network/virtualHubs/hubRouteTables@2420-04-01
Fake.Network/virtualHubs/hubRouteTables@2420-05-01
Fake.Network/virtualHubs/hubRouteTables@2420-06-01
Fake.Network/virtualHubs/hubRouteTables@2420-08-01
Fake.Network/virtualHubs/hubRouteTables@2420-11-01
Fake.Network/applicationGateways/privateEndpointConnections@2420-05-01
Fake.Network/applicationGateways/privateEndpointConnections@2420-06-01
Fake.Network/applicationGateways/privateEndpointConnections@2420-08-01
Fake.Network/applicationGateways/privateEndpointConnections@2420-11-01
Fake.Network/firewallPolicies/ruleCollectionGroups@2420-05-01
Fake.Network/firewallPolicies/ruleCollectionGroups@2420-06-01
Fake.Network/firewallPolicies/ruleCollectionGroups@2420-08-01
Fake.Network/firewallPolicies/ruleCollectionGroups@2420-11-01
Fake.Network/networkVirtualAppliances/virtualApplianceSites@2420-05-01
Fake.Network/networkVirtualAppliances/virtualApplianceSites@2420-06-01
Fake.Network/networkVirtualAppliances/virtualApplianceSites@2420-08-01
Fake.Network/networkVirtualAppliances/virtualApplianceSites@2420-11-01
Fake.Network/virtualHubs/bgpConnections@2420-05-01
Fake.Network/virtualHubs/bgpConnections@2420-06-01
Fake.Network/virtualHubs/bgpConnections@2420-08-01
Fake.Network/virtualHubs/bgpConnections@2420-11-01
Fake.Network/virtualHubs/hubVirtualNetworkConnections@2420-05-01
Fake.Network/virtualHubs/hubVirtualNetworkConnections@2420-06-01
Fake.Network/virtualHubs/hubVirtualNetworkConnections@2420-08-01
Fake.Network/virtualHubs/hubVirtualNetworkConnections@2420-11-01
Fake.Network/virtualHubs/ipConfigurations@2420-05-01
Fake.Network/virtualHubs/ipConfigurations@2420-06-01
Fake.Network/virtualHubs/ipConfigurations@2420-08-01
Fake.Network/virtualHubs/ipConfigurations@2420-11-01
Fake.Network/networkVirtualAppliances/inboundSecurityRules@2420-06-01
Fake.Network/networkVirtualAppliances/inboundSecurityRules@2420-08-01
Fake.Network/networkVirtualAppliances/inboundSecurityRules@2420-11-01
Fake.Network/customIpPrefixes@2420-08-01
Fake.Network/customIpPrefixes@2420-11-01
Fake.Network/dscpConfigurations@2420-08-01
Fake.Network/dscpConfigurations@2420-11-01
Fake.Network/vpnGateways/natRules@2420-08-01
Fake.Network/vpnGateways/natRules@2420-11-01
Fake.DataMigration/services@2417-11-15-preview
Fake.DataMigration/services@2417-11-15-privatepreview
Fake.DataMigration/services@2417-11-15-preview
Fake.DataMigration/services@2418-03-15-preview
Fake.DataMigration/services@2418-03-31-preview
Fake.DataMigration/services@2418-04-19
Fake.DataMigration/services@2418-07-15-preview
Fake.DataMigration/services@2421-06-30
Fake.DataMigration/services/projects@2417-11-15-preview
Fake.DataMigration/services/projects@2417-11-15-privatepreview
Fake.DataMigration/services/projects@2417-11-15-preview
Fake.DataMigration/services/projects@2418-03-15-preview
Fake.DataMigration/services/projects@2418-03-31-preview
Fake.DataMigration/services/projects@2418-04-19
Fake.DataMigration/services/projects@2418-07-15-preview
Fake.DataMigration/services/projects@2421-06-30
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2416-12-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2416-06-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2419-05-13
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2419-06-15
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2420-10-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2420-12-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-01-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-02-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-02-01-preview
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-02-10
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-03-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-04-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-06-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers/protectedItems@2421-07-01
Fake.RecoveryServices/vaults/backupPolicies@2416-12-01
Fake.RecoveryServices/vaults/backupPolicies@2416-06-01
Fake.RecoveryServices/vaults/backupPolicies@2419-05-13
Fake.RecoveryServices/vaults/backupPolicies@2419-06-15
Fake.RecoveryServices/vaults/backupPolicies@2420-10-01
Fake.RecoveryServices/vaults/backupPolicies@2420-12-01
Fake.RecoveryServices/vaults/backupPolicies@2421-01-01
Fake.RecoveryServices/vaults/backupPolicies@2421-02-01
Fake.RecoveryServices/vaults/backupPolicies@2421-02-01-preview
Fake.RecoveryServices/vaults/backupPolicies@2421-02-10
Fake.RecoveryServices/vaults/backupPolicies@2421-03-01
Fake.RecoveryServices/vaults/backupPolicies@2421-04-01
Fake.RecoveryServices/vaults/backupPolicies@2421-06-01
Fake.RecoveryServices/vaults/backupPolicies@2421-07-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2417-07-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2417-07-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-02-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-02-01-preview
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-02-10
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-03-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-04-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-06-01
Fake.RecoveryServices/vaults/backupFabrics/backupProtectionIntent@2421-07-01
Fake.Compute/galleries@2418-06-01
Fake.Compute/galleries@2419-03-01
Fake.Compute/galleries@2420-09-30
Fake.Compute/galleries@2421-07-01
Fake.Compute/galleries/images@2418-06-01
Fake.Compute/galleries/images@2419-03-01
Fake.Compute/galleries/images@2420-09-30
Fake.Compute/galleries/images@2421-07-01
Fake.Compute/galleries/images/versions@2418-06-01
Fake.Compute/galleries/images/versions@2419-03-01
Fake.Compute/galleries/images/versions@2420-09-30
Fake.Compute/galleries/images/versions@2421-07-01
Fake.Compute/hostGroups@2419-03-01
Fake.Compute/hostGroups@2419-07-01
Fake.Compute/hostGroups@2420-06-01
Fake.Compute/hostGroups@2420-12-01
Fake.Compute/hostGroups@2421-03-01
Fake.Compute/hostGroups@2421-04-01
Fake.Compute/hostGroups@2421-07-01
Fake.Compute/hostGroups/hosts@2419-03-01
Fake.Compute/hostGroups/hosts@2419-07-01
Fake.Compute/hostGroups/hosts@2420-06-01
Fake.Compute/hostGroups/hosts@2420-12-01
Fake.Compute/hostGroups/hosts@2421-03-01
Fake.Compute/hostGroups/hosts@2421-04-01
Fake.Compute/hostGroups/hosts@2421-07-01
Fake.Compute/proximityPlacementGroups@2419-03-01
Fake.Compute/proximityPlacementGroups@2419-07-01
Fake.Compute/proximityPlacementGroups@2420-06-01
Fake.Compute/proximityPlacementGroups@2420-12-01
Fake.Compute/proximityPlacementGroups@2421-03-01
Fake.Compute/proximityPlacementGroups@2421-04-01
Fake.Compute/proximityPlacementGroups@2421-07-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2419-03-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2419-07-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2420-06-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2420-12-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2421-03-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2421-04-01
Fake.Compute/virtualMachineScaleSets/virtualmachines@2421-07-01
Fake.OperationsManagement/ManagementConfigurations@2415-11-01-preview
Fake.OperationsManagement/solutions@2415-11-01-preview
Fake.ManagedIdentity/userAssignedIdentities@2415-08-31-preview
Fake.ManagedIdentity/userAssignedIdentities@2418-11-30
Fake.ManagedServices/registrationAssignments@2418-06-01-preview
Fake.ManagedServices/registrationAssignments@2419-06-01
Fake.ManagedServices/registrationDefinitions@2418-06-01-preview
Fake.ManagedServices/registrationDefinitions@2419-06-01
Fake.BareMetal/crayServers@2418-09-01-preview
Fake.AppPlatform/Spring@2419-05-01-preview
Fake.AppPlatform/Spring@2419-05-01-preview
Fake.AppPlatform/Spring@2420-07-01
Fake.AppPlatform/Spring@2420-11-01-preview
Fake.AppPlatform/Spring@2421-06-01-preview
Fake.AppPlatform/Spring/apps@2419-05-01-preview
Fake.AppPlatform/Spring/apps@2419-05-01-preview
Fake.AppPlatform/Spring/apps@2420-07-01
Fake.AppPlatform/Spring/apps@2420-11-01-preview
Fake.AppPlatform/Spring/apps@2421-06-01-preview
Fake.AppPlatform/Spring/apps/bindings@2419-05-01-preview
Fake.AppPlatform/Spring/apps/bindings@2419-05-01-preview
Fake.AppPlatform/Spring/apps/bindings@2420-07-01
Fake.AppPlatform/Spring/apps/bindings@2420-11-01-preview
Fake.AppPlatform/Spring/apps/bindings@2421-06-01-preview
Fake.AppPlatform/Spring/apps/deployments@2419-05-01-preview
Fake.AppPlatform/Spring/apps/deployments@2419-05-01-preview
Fake.AppPlatform/Spring/apps/deployments@2420-07-01
Fake.AppPlatform/Spring/apps/deployments@2420-11-01-preview
Fake.AppPlatform/Spring/apps/deployments@2421-06-01-preview
Fake.Compute/diskEncryptionSets@2419-07-01
Fake.Compute/diskEncryptionSets@2420-06-30
Fake.Compute/diskEncryptionSets@2420-09-30
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2419-07-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2420-06-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2420-12-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2421-03-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2421-04-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/extensions@2421-07-01
Fake.Advisor/suppressions@2416-07-12-preview
Fake.Advisor/suppressions@2417-03-31
Fake.Advisor/suppressions@2417-04-19
Fake.Advisor/suppressions@2420-01-01
Fake.Advisor/configurations@2417-04-19
Fake.Advisor/configurations@2420-01-01
Fake.Advisor/configurations@2417-04-19
Fake.Advisor/configurations@2420-01-01
Fake.Compute/cloudServices@2420-10-01-preview
Fake.Compute/cloudServices@2421-03-01
Fake.Insights/components@2414-04-01
Fake.Insights/components@2415-05-01
Fake.Insights/components@2418-05-01-preview
Fake.Insights/components@2420-02-02
Fake.Insights/components@2420-02-02-preview
Fake.Insights/webtests@2414-04-01
Fake.Insights/webtests@2415-05-01
Fake.Insights/webtests@2418-05-01-preview
Fake.Insights/webtests@2420-10-05-preview
Fake.Insights/autoscalesettings@2414-04-01
Fake.Insights/autoscalesettings@2414-04-01
Fake.Insights/autoscalesettings@2415-04-01
Fake.Compute/sshPublicKeys@2420-06-01
Fake.Compute/sshPublicKeys@2420-12-01
Fake.Compute/sshPublicKeys@2421-03-01
Fake.Compute/sshPublicKeys@2421-04-01
Fake.Compute/sshPublicKeys@2421-07-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/runCommands@2420-06-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/runCommands@2420-12-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/runCommands@2421-03-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/runCommands@2421-04-01
Fake.Compute/virtualMachineScaleSets/virtualMachines/runCommands@2421-07-01
Fake.Compute/virtualMachines/runCommands@2420-06-01
Fake.Compute/virtualMachines/runCommands@2420-12-01
Fake.Compute/virtualMachines/runCommands@2421-03-01
Fake.Compute/virtualMachines/runCommands@2421-04-01
Fake.Compute/virtualMachines/runCommands@2421-07-01
Fake.Compute/diskAccesses@2420-06-30
Fake.Compute/diskAccesses@2420-09-30
Fake.Compute/diskAccesses/privateEndpointConnections@2420-09-30
Fake.Compute/galleries/applications@2420-09-30
Fake.Compute/galleries/applications@2421-07-01
Fake.Compute/galleries/applications/versions@2420-09-30
Fake.Compute/galleries/applications/versions@2421-07-01
Fake.Compute/restorePointCollections@2421-03-01
Fake.Compute/restorePointCollections@2421-04-01
Fake.Compute/restorePointCollections@2421-07-01
Fake.Compute/restorePointCollections/restorePoints@2421-03-01
Fake.Compute/restorePointCollections/restorePoints@2421-04-01
Fake.Compute/restorePointCollections/restorePoints@2421-07-01
Fake.Compute/capacityReservationGroups@2421-04-01
Fake.Compute/capacityReservationGroups@2421-07-01
Fake.Compute/capacityReservationGroups/capacityReservations@2421-04-01
Fake.Compute/capacityReservationGroups/capacityReservations@2421-07-01
sendgrid.Email/accounts@2415-01-01
Fake.Resources/deployments@2415-01-01
Fake.Resources/deployments@2415-11-01
Fake.Resources/deployments@2416-02-01
Fake.Resources/deployments@2416-07-01
Fake.Resources/deployments@2416-09-01
Fake.Resources/deployments@2417-05-10
Fake.Resources/deployments@2418-02-01
Fake.Resources/deployments@2418-05-01
Fake.Resources/deployments@2419-03-01
Fake.Resources/deployments@2419-05-01
Fake.Resources/deployments@2419-05-10
Fake.Resources/deployments@2419-07-01
Fake.Resources/deployments@2419-08-01
Fake.Resources/deployments@2419-10-01
Fake.Resources/deployments@2420-06-01
Fake.Resources/deployments@2420-08-01
Fake.Resources/deployments@2420-10-01
Fake.Resources/deployments@2421-01-01
Fake.Resources/deployments@2421-04-01
Fake.AAD/domainServices@2417-01-01
Fake.AAD/domainServices@2417-06-01
Fake.AAD/domainServices@2420-01-01
Fake.AAD/domainServices@2421-03-01
Fake.Aad/domainServices/ouContainer@2417-06-01
Fake.Aad/domainServices/ouContainer@2420-01-01
Fake.Aad/domainServices/ouContainer@2421-03-01
Fake.aadiam/privateLinkForAzureAd@2420-03-01
Fake.aadiam/privateLinkForAzureAd@2420-03-01-preview
Fake.aadiam/privateLinkForAzureAd/privateEndpointConnections@2420-03-01
Fake.aadiam/azureADMetrics@2420-07-01-preview
Fake.alertsManagement/smartDetectorAlertRules@2419-03-01
Fake.alertsManagement/smartDetectorAlertRules@2419-06-01
Fake.alertsManagement/smartDetectorAlertRules@2421-04-01
Fake.AlertsManagement/actionRules@2419-05-05-preview
Fake.AlertsManagement/actionRules@2421-08-08-preview
Fake.AlertsManagement/resourceHealthAlertRules@2420-08-04-preview
Fake.AnalysisServices/servers@2416-05-16
Fake.AnalysisServices/servers@2417-07-14
Fake.AnalysisServices/servers@2417-08-01
Fake.ApiManagement/service@2416-07-07
Fake.ApiManagement/service@2416-10-10
Fake.ApiManagement/service@2417-03-01
Fake.ApiManagement/service@2418-01-01
Fake.ApiManagement/service@2418-06-01-preview
Fake.ApiManagement/service@2419-01-01
Fake.ApiManagement/service@2419-12-01
Fake.ApiManagement/service@2419-12-01-preview
Fake.ApiManagement/service@2420-06-01-preview
Fake.ApiManagement/service@2420-12-01
Fake.ApiManagement/service@2421-01-01-preview
Fake.ApiManagement/service/apis@2416-07-07
Fake.ApiManagement/service/apis@2416-10-10
Fake.ApiManagement/service/apis@2417-03-01
Fake.ApiManagement/service/apis@2418-01-01
Fake.ApiManagement/service/apis@2418-06-01-preview
Fake.ApiManagement/service/apis@2419-01-01
Fake.ApiManagement/service/apis@2419-12-01
Fake.ApiManagement/service/apis@2419-12-01-preview
Fake.ApiManagement/service/apis@2420-06-01-preview
Fake.ApiManagement/service/apis@2420-12-01
Fake.ApiManagement/service/apis@2421-01-01-preview
Fake.ApiManagement/service/apis/operations@2416-07-07
Fake.ApiManagement/service/apis/operations@2416-10-10
Fake.ApiManagement/service/apis/operations@2417-03-01
Fake.ApiManagement/service/apis/operations@2418-01-01
Fake.ApiManagement/service/apis/operations@2418-06-01-preview
Fake.ApiManagement/service/apis/operations@2419-01-01
Fake.ApiManagement/service/apis/operations@2419-12-01
Fake.ApiManagement/service/apis/operations@2419-12-01-preview
Fake.ApiManagement/service/apis/operations@2420-06-01-preview
Fake.ApiManagement/service/apis/operations@2420-12-01
Fake.ApiManagement/service/apis/operations@2421-01-01-preview
Fake.ApiManagement/service/authorizationServers@2416-07-07
Fake.ApiManagement/service/authorizationServers@2416-10-10
Fake.ApiManagement/service/authorizationServers@2417-03-01
Fake.ApiManagement/service/authorizationServers@2418-01-01
Fake.ApiManagement/service/authorizationServers@2418-06-01-preview
Fake.ApiManagement/service/authorizationServers@2419-01-01
Fake.ApiManagement/service/authorizationServers@2419-12-01
Fake.ApiManagement/service/authorizationServers@2419-12-01-preview
Fake.ApiManagement/service/authorizationServers@2420-06-01-preview
Fake.ApiManagement/service/authorizationServers@2420-12-01
Fake.ApiManagement/service/authorizationServers@2421-01-01-preview
Fake.ApiManagement/service/backends@2416-07-07
Fake.ApiManagement/service/backends@2416-10-10
Fake.ApiManagement/service/backends@2417-03-01
Fake.ApiManagement/service/backends@2418-01-01
Fake.ApiManagement/service/backends@2418-06-01-preview
Fake.ApiManagement/service/backends@2419-01-01
Fake.ApiManagement/service/backends@2419-12-01
Fake.ApiManagement/service/backends@2419-12-01-preview
Fake.ApiManagement/service/backends@2420-06-01-preview
Fake.ApiManagement/service/backends@2420-12-01
Fake.ApiManagement/service/backends@2421-01-01-preview
Fake.ApiManagement/service/certificates@2416-07-07
Fake.ApiManagement/service/certificates@2416-10-10
Fake.ApiManagement/service/certificates@2417-03-01
Fake.ApiManagement/service/certificates@2418-01-01
Fake.ApiManagement/service/certificates@2418-06-01-preview
Fake.ApiManagement/service/certificates@2419-01-01
Fake.ApiManagement/service/certificates@2419-12-01
Fake.ApiManagement/service/certificates@2419-12-01-preview
Fake.ApiManagement/service/certificates@2420-06-01-preview
Fake.ApiManagement/service/certificates@2420-12-01
Fake.ApiManagement/service/certificates@2421-01-01-preview
Fake.ApiManagement/service/groups@2416-07-07
Fake.ApiManagement/service/groups@2416-10-10
Fake.ApiManagement/service/groups@2417-03-01
Fake.ApiManagement/service/groups@2418-01-01
Fake.ApiManagement/service/groups@2418-06-01-preview
Fake.ApiManagement/service/groups@2419-01-01
Fake.ApiManagement/service/groups@2419-12-01
Fake.ApiManagement/service/groups@2419-12-01-preview
Fake.ApiManagement/service/groups@2420-06-01-preview
Fake.ApiManagement/service/groups@2420-12-01
Fake.ApiManagement/service/groups@2421-01-01-preview
Fake.ApiManagement/service/groups/users@2416-07-07
Fake.ApiManagement/service/groups/users@2416-10-10
Fake.ApiManagement/service/groups/users@2417-03-01
Fake.ApiManagement/service/groups/users@2418-01-01
Fake.ApiManagement/service/groups/users@2418-06-01-preview
Fake.ApiManagement/service/groups/users@2419-01-01
Fake.ApiManagement/service/groups/users@2419-12-01
Fake.ApiManagement/service/groups/users@2419-12-01-preview
Fake.ApiManagement/service/groups/users@2420-06-01-preview
Fake.ApiManagement/service/groups/users@2420-12-01
Fake.ApiManagement/service/groups/users@2421-01-01-preview
Fake.ApiManagement/service/identityProviders@2416-07-07
Fake.ApiManagement/service/identityProviders@2416-10-10
Fake.ApiManagement/service/identityProviders@2417-03-01
Fake.ApiManagement/service/identityProviders@2418-01-01
Fake.ApiManagement/service/identityProviders@2418-06-01-preview
Fake.ApiManagement/service/identityProviders@2419-01-01
Fake.ApiManagement/service/identityProviders@2419-12-01
Fake.ApiManagement/service/identityProviders@2419-12-01-preview
Fake.ApiManagement/service/identityProviders@2420-06-01-preview
Fake.ApiManagement/service/identityProviders@2420-12-01
Fake.ApiManagement/service/identityProviders@2421-01-01-preview
Fake.ApiManagement/service/loggers@2416-07-07
Fake.ApiManagement/service/loggers@2416-10-10
Fake.ApiManagement/service/loggers@2417-03-01
Fake.ApiManagement/service/loggers@2418-01-01
Fake.ApiManagement/service/loggers@2418-06-01-preview
Fake.ApiManagement/service/loggers@2419-01-01
Fake.ApiManagement/service/loggers@2419-12-01
Fake.ApiManagement/service/loggers@2419-12-01-preview
Fake.ApiManagement/service/loggers@2420-06-01-preview
Fake.ApiManagement/service/loggers@2420-12-01
Fake.ApiManagement/service/loggers@2421-01-01-preview
Fake.ApiManagement/service/openidConnectProviders@2416-07-07
Fake.ApiManagement/service/openidConnectProviders@2416-10-10
Fake.ApiManagement/service/openidConnectProviders@2417-03-01
Fake.ApiManagement/service/openidConnectProviders@2418-01-01
Fake.ApiManagement/service/openidConnectProviders@2418-06-01-preview
Fake.ApiManagement/service/openidConnectProviders@2419-01-01
Fake.ApiManagement/service/openidConnectProviders@2419-12-01
Fake.ApiManagement/service/openidConnectProviders@2419-12-01-preview
Fake.ApiManagement/service/openidConnectProviders@2420-06-01-preview
Fake.ApiManagement/service/openidConnectProviders@2420-12-01
Fake.ApiManagement/service/openidConnectProviders@2421-01-01-preview
Fake.ApiManagement/service/products@2416-07-07
Fake.ApiManagement/service/products@2416-10-10
Fake.ApiManagement/service/products@2417-03-01
Fake.ApiManagement/service/products@2418-01-01
Fake.ApiManagement/service/products@2418-06-01-preview
Fake.ApiManagement/service/products@2419-01-01
Fake.ApiManagement/service/products@2419-12-01
Fake.ApiManagement/service/products@2419-12-01-preview
Fake.ApiManagement/service/products@2420-06-01-preview
Fake.ApiManagement/service/products@2420-12-01
Fake.ApiManagement/service/products@2421-01-01-preview
Fake.ApiManagement/service/products/apis@2416-07-07
Fake.ApiManagement/service/products/apis@2416-10-10
Fake.ApiManagement/service/products/apis@2417-03-01
Fake.ApiManagement/service/products/apis@2418-01-01
Fake.ApiManagement/service/products/apis@2418-06-01-preview
Fake.ApiManagement/service/products/apis@2419-01-01
Fake.ApiManagement/service/products/apis@2419-12-01
Fake.ApiManagement/service/products/apis@2419-12-01-preview
Fake.ApiManagement/service/products/apis@2420-06-01-preview
Fake.ApiManagement/service/products/apis@2420-12-01
Fake.ApiManagement/service/products/apis@2421-01-01-preview
Fake.ApiManagement/service/products/groups@2416-07-07
Fake.ApiManagement/service/products/groups@2416-10-10
Fake.ApiManagement/service/products/groups@2417-03-01
Fake.ApiManagement/service/products/groups@2418-01-01
Fake.ApiManagement/service/products/groups@2418-06-01-preview
Fake.ApiManagement/service/products/groups@2419-01-01
Fake.ApiManagement/service/products/groups@2419-12-01
Fake.ApiManagement/service/products/groups@2419-12-01-preview
Fake.ApiManagement/service/products/groups@2420-06-01-preview
Fake.ApiManagement/service/products/groups@2420-12-01
Fake.ApiManagement/service/products/groups@2421-01-01-preview
Fake.ApiManagement/service/properties@2416-07-07
Fake.ApiManagement/service/properties@2416-10-10
Fake.ApiManagement/service/properties@2417-03-01
Fake.ApiManagement/service/properties@2418-01-01
Fake.ApiManagement/service/properties@2418-06-01-preview
Fake.ApiManagement/service/properties@2419-01-01
Fake.ApiManagement/service/subscriptions@2416-07-07
Fake.ApiManagement/service/subscriptions@2416-10-10
Fake.ApiManagement/service/subscriptions@2417-03-01
Fake.ApiManagement/service/subscriptions@2418-01-01
Fake.ApiManagement/service/subscriptions@2418-06-01-preview
Fake.ApiManagement/service/subscriptions@2419-01-01
Fake.ApiManagement/service/subscriptions@2419-12-01
Fake.ApiManagement/service/subscriptions@2419-12-01-preview
Fake.ApiManagement/service/subscriptions@2420-06-01-preview
Fake.ApiManagement/service/subscriptions@2420-12-01
Fake.ApiManagement/service/subscriptions@2421-01-01-preview
Fake.ApiManagement/service/users@2416-07-07
Fake.ApiManagement/service/users@2416-10-10
Fake.ApiManagement/service/users@2417-03-01
Fake.ApiManagement/service/users@2418-01-01
Fake.ApiManagement/service/users@2418-06-01-preview
Fake.ApiManagement/service/users@2419-01-01
Fake.ApiManagement/service/users@2419-12-01
Fake.ApiManagement/service/users@2419-12-01-preview
Fake.ApiManagement/service/users@2420-06-01-preview
Fake.ApiManagement/service/users@2420-12-01
Fake.ApiManagement/service/users@2421-01-01-preview
Fake.ApiManagement/service/apis/operations/policy@2416-10-10
Fake.ApiManagement/service/apis/policy@2416-10-10
Fake.ApiManagement/service/products/policy@2416-10-10
Fake.ApiManagement/service/tenant@2416-10-10
Fake.ApiManagement/service/tenant@2420-06-01-preview
Fake.ApiManagement/service/tenant@2420-12-01
Fake.ApiManagement/service/tenant@2421-01-01-preview
Fake.ApiManagement/service/api-version-sets@2417-03-01
Fake.ApiManagement/service/api-version-sets@2418-01-01
Fake.ApiManagement/service/api-version-sets@2418-06-01-preview
Fake.ApiManagement/service/apis/diagnostics@2417-03-01
Fake.ApiManagement/service/apis/diagnostics@2418-01-01
Fake.ApiManagement/service/apis/diagnostics@2418-06-01-preview
Fake.ApiManagement/service/apis/diagnostics@2419-01-01
Fake.ApiManagement/service/apis/diagnostics@2419-12-01
Fake.ApiManagement/service/apis/diagnostics@2419-12-01-preview
Fake.ApiManagement/service/apis/diagnostics@2420-06-01-preview
Fake.ApiManagement/service/apis/diagnostics@2420-12-01
Fake.ApiManagement/service/apis/diagnostics@2421-01-01-preview
Fake.ApiManagement/service/apis/diagnostics/loggers@2417-03-01
Fake.ApiManagement/service/apis/diagnostics/loggers@2418-01-01
Fake.ApiManagement/service/apis/issues@2417-03-01
Fake.ApiManagement/service/apis/issues@2418-01-01
Fake.ApiManagement/service/apis/issues@2418-06-01-preview
Fake.ApiManagement/service/apis/issues@2419-01-01
Fake.ApiManagement/service/apis/issues@2419-12-01
Fake.ApiManagement/service/apis/issues@2419-12-01-preview
Fake.ApiManagement/service/apis/issues@2420-06-01-preview
Fake.ApiManagement/service/apis/issues@2420-12-01
Fake.ApiManagement/service/apis/issues@2421-01-01-preview
Fake.ApiManagement/service/apis/issues/attachments@2417-03-01
Fake.ApiManagement/service/apis/issues/attachments@2418-01-01
Fake.ApiManagement/service/apis/issues/attachments@2418-06-01-preview
Fake.ApiManagement/service/apis/issues/attachments@2419-01-01
Fake.ApiManagement/service/apis/issues/attachments@2419-12-01
Fake.ApiManagement/service/apis/issues/attachments@2419-12-01-preview
Fake.ApiManagement/service/apis/issues/attachments@2420-06-01-preview
Fake.ApiManagement/service/apis/issues/attachments@2420-12-01
Fake.ApiManagement/service/apis/issues/attachments@2421-01-01-preview
Fake.ApiManagement/service/apis/issues/comments@2417-03-01
Fake.ApiManagement/service/apis/issues/comments@2418-01-01
Fake.ApiManagement/service/apis/issues/comments@2418-06-01-preview
Fake.ApiManagement/service/apis/issues/comments@2419-01-01
Fake.ApiManagement/service/apis/issues/comments@2419-12-01
Fake.ApiManagement/service/apis/issues/comments@2419-12-01-preview
Fake.ApiManagement/service/apis/issues/comments@2420-06-01-preview
Fake.ApiManagement/service/apis/issues/comments@2420-12-01
Fake.ApiManagement/service/apis/issues/comments@2421-01-01-preview
Fake.ApiManagement/service/apis/operations/policies@2417-03-01
Fake.ApiManagement/service/apis/operations/policies@2418-01-01
Fake.ApiManagement/service/apis/operations/policies@2418-06-01-preview
Fake.ApiManagement/service/apis/operations/policies@2419-01-01
Fake.ApiManagement/service/apis/operations/policies@2419-12-01
Fake.ApiManagement/service/apis/operations/policies@2419-12-01-preview
Fake.ApiManagement/service/apis/operations/policies@2420-06-01-preview
Fake.ApiManagement/service/apis/operations/policies@2420-12-01
Fake.ApiManagement/service/apis/operations/policies@2421-01-01-preview
Fake.ApiManagement/service/apis/operations/tags@2417-03-01
Fake.ApiManagement/service/apis/operations/tags@2418-01-01
Fake.ApiManagement/service/apis/operations/tags@2418-06-01-preview
Fake.ApiManagement/service/apis/operations/tags@2419-01-01
Fake.ApiManagement/service/apis/operations/tags@2419-12-01
Fake.ApiManagement/service/apis/operations/tags@2419-12-01-preview
Fake.ApiManagement/service/apis/operations/tags@2420-06-01-preview
Fake.ApiManagement/service/apis/operations/tags@2420-12-01
Fake.ApiManagement/service/apis/operations/tags@2421-01-01-preview
Fake.ApiManagement/service/apis/policies@2417-03-01
Fake.ApiManagement/service/apis/policies@2418-01-01
Fake.ApiManagement/service/apis/policies@2418-06-01-preview
Fake.ApiManagement/service/apis/policies@2419-01-01
Fake.ApiManagement/service/apis/policies@2419-12-01
Fake.ApiManagement/service/apis/policies@2419-12-01-preview
Fake.ApiManagement/service/apis/policies@2420-06-01-preview
Fake.ApiManagement/service/apis/policies@2420-12-01
Fake.ApiManagement/service/apis/policies@2421-01-01-preview
Fake.ApiManagement/service/apis/releases@2417-03-01
Fake.ApiManagement/service/apis/releases@2418-01-01
Fake.ApiManagement/service/apis/releases@2418-06-01-preview
Fake.ApiManagement/service/apis/releases@2419-01-01
Fake.ApiManagement/service/apis/releases@2419-12-01
Fake.ApiManagement/service/apis/releases@2419-12-01-preview
Fake.ApiManagement/service/apis/releases@2420-06-01-preview
Fake.ApiManagement/service/apis/releases@2420-12-01
Fake.ApiManagement/service/apis/releases@2421-01-01-preview
Fake.ApiManagement/service/apis/schemas@2417-03-01
Fake.ApiManagement/service/apis/schemas@2418-01-01
Fake.ApiManagement/service/apis/schemas@2418-06-01-preview
Fake.ApiManagement/service/apis/schemas@2419-01-01
Fake.ApiManagement/service/apis/schemas@2419-12-01
Fake.ApiManagement/service/apis/schemas@2419-12-01-preview
Fake.ApiManagement/service/apis/schemas@2420-06-01-preview
Fake.ApiManagement/service/apis/schemas@2420-12-01
Fake.ApiManagement/service/apis/schemas@2421-01-01-preview
Fake.ApiManagement/service/apis/tagDescriptions@2417-03-01
Fake.ApiManagement/service/apis/tagDescriptions@2418-01-01
Fake.ApiManagement/service/apis/tagDescriptions@2418-06-01-preview
Fake.ApiManagement/service/apis/tagDescriptions@2419-01-01
Fake.ApiManagement/service/apis/tagDescriptions@2419-12-01
Fake.ApiManagement/service/apis/tagDescriptions@2419-12-01-preview
Fake.ApiManagement/service/apis/tagDescriptions@2420-06-01-preview
Fake.ApiManagement/service/apis/tagDescriptions@2420-12-01
Fake.ApiManagement/service/apis/tagDescriptions@2421-01-01-preview
Fake.ApiManagement/service/apis/tags@2417-03-01
Fake.ApiManagement/service/apis/tags@2418-01-01
Fake.ApiManagement/service/apis/tags@2418-06-01-preview
Fake.ApiManagement/service/apis/tags@2419-01-01
Fake.ApiManagement/service/apis/tags@2419-12-01
Fake.ApiManagement/service/apis/tags@2419-12-01-preview
Fake.ApiManagement/service/apis/tags@2420-06-01-preview
Fake.ApiManagement/service/apis/tags@2420-12-01
Fake.ApiManagement/service/apis/tags@2421-01-01-preview
Fake.ApiManagement/service/diagnostics@2417-03-01
Fake.ApiManagement/service/diagnostics@2418-01-01
Fake.ApiManagement/service/diagnostics@2418-06-01-preview
Fake.ApiManagement/service/diagnostics@2419-01-01
Fake.ApiManagement/service/diagnostics@2419-12-01
Fake.ApiManagement/service/diagnostics@2419-12-01-preview
Fake.ApiManagement/service/diagnostics@2420-06-01-preview
Fake.ApiManagement/service/diagnostics@2420-12-01
Fake.ApiManagement/service/diagnostics@2421-01-01-preview
Fake.ApiManagement/service/diagnostics/loggers@2417-03-01
Fake.ApiManagement/service/diagnostics/loggers@2418-01-01
Fake.ApiManagement/service/notifications@2417-03-01
Fake.ApiManagement/service/notifications@2418-01-01
Fake.ApiManagement/service/notifications@2418-06-01-preview
Fake.ApiManagement/service/notifications@2419-01-01
Fake.ApiManagement/service/notifications@2419-12-01
Fake.ApiManagement/service/notifications@2419-12-01-preview
Fake.ApiManagement/service/notifications@2420-06-01-preview
Fake.ApiManagement/service/notifications@2420-12-01
Fake.ApiManagement/service/notifications@2421-01-01-preview
Fake.ApiManagement/service/notifications/recipientEmails@2417-03-01
Fake.ApiManagement/service/notifications/recipientEmails@2418-01-01
Fake.ApiManagement/service/notifications/recipientEmails@2418-06-01-preview
Fake.ApiManagement/service/notifications/recipientEmails@2419-01-01
Fake.ApiManagement/service/notifications/recipientEmails@2419-12-01
Fake.ApiManagement/service/notifications/recipientEmails@2419-12-01-preview
Fake.ApiManagement/service/notifications/recipientEmails@2420-06-01-preview
Fake.ApiManagement/service/notifications/recipientEmails@2420-12-01
Fake.ApiManagement/service/notifications/recipientEmails@2421-01-01-preview
Fake.ApiManagement/service/notifications/recipientUsers@2417-03-01
Fake.ApiManagement/service/notifications/recipientUsers@2418-01-01
Fake.ApiManagement/service/notifications/recipientUsers@2418-06-01-preview
Fake.ApiManagement/service/notifications/recipientUsers@2419-01-01
Fake.ApiManagement/service/notifications/recipientUsers@2419-12-01
Fake.ApiManagement/service/notifications/recipientUsers@2419-12-01-preview
Fake.ApiManagement/service/notifications/recipientUsers@2420-06-01-preview
Fake.ApiManagement/service/notifications/recipientUsers@2420-12-01
Fake.ApiManagement/service/notifications/recipientUsers@2421-01-01-preview
Fake.ApiManagement/service/policies@2417-03-01
Fake.ApiManagement/service/policies@2418-01-01
Fake.ApiManagement/service/policies@2418-06-01-preview
Fake.ApiManagement/service/policies@2419-01-01
Fake.ApiManagement/service/policies@2419-12-01
Fake.ApiManagement/service/policies@2419-12-01-preview
Fake.ApiManagement/service/policies@2420-06-01-preview
Fake.ApiManagement/service/policies@2420-12-01
Fake.ApiManagement/service/policies@2421-01-01-preview
Fake.ApiManagement/service/portalsettings@2417-03-01
Fake.ApiManagement/service/portalsettings@2418-01-01
Fake.ApiManagement/service/portalsettings@2418-06-01-preview
Fake.ApiManagement/service/portalsettings@2419-01-01
Fake.ApiManagement/service/portalsettings@2419-12-01
Fake.ApiManagement/service/portalsettings@2419-12-01-preview
Fake.ApiManagement/service/portalsettings@2420-06-01-preview
Fake.ApiManagement/service/portalsettings@2420-12-01
Fake.ApiManagement/service/portalsettings@2421-01-01-preview
Fake.ApiManagement/service/products/policies@2417-03-01
Fake.ApiManagement/service/products/policies@2418-01-01
Fake.ApiManagement/service/products/policies@2418-06-01-preview
Fake.ApiManagement/service/products/policies@2419-01-01
Fake.ApiManagement/service/products/policies@2419-12-01
Fake.ApiManagement/service/products/policies@2419-12-01-preview
Fake.ApiManagement/service/products/policies@2420-06-01-preview
Fake.ApiManagement/service/products/policies@2420-12-01
Fake.ApiManagement/service/products/policies@2421-01-01-preview
Fake.ApiManagement/service/products/tags@2417-03-01
Fake.ApiManagement/service/products/tags@2418-01-01
Fake.ApiManagement/service/products/tags@2418-06-01-preview
Fake.ApiManagement/service/products/tags@2419-01-01
Fake.ApiManagement/service/products/tags@2419-12-01
Fake.ApiManagement/service/products/tags@2419-12-01-preview
Fake.ApiManagement/service/products/tags@2420-06-01-preview
Fake.ApiManagement/service/products/tags@2420-12-01
Fake.ApiManagement/service/products/tags@2421-01-01-preview
Fake.ApiManagement/service/tags@2417-03-01
Fake.ApiManagement/service/tags@2418-01-01
Fake.ApiManagement/service/tags@2418-06-01-preview
Fake.ApiManagement/service/tags@2419-01-01
Fake.ApiManagement/service/tags@2419-12-01
Fake.ApiManagement/service/tags@2419-12-01-preview
Fake.ApiManagement/service/tags@2420-06-01-preview
Fake.ApiManagement/service/tags@2420-12-01
Fake.ApiManagement/service/tags@2421-01-01-preview
Fake.ApiManagement/service/templates@2417-03-01
Fake.ApiManagement/service/templates@2418-01-01
Fake.ApiManagement/service/templates@2418-06-01-preview
Fake.ApiManagement/service/templates@2419-01-01
Fake.ApiManagement/service/templates@2419-12-01
Fake.ApiManagement/service/templates@2419-12-01-preview
Fake.ApiManagement/service/templates@2420-06-01-preview
Fake.ApiManagement/service/templates@2420-12-01
Fake.ApiManagement/service/templates@2421-01-01-preview
Fake.ApiManagement/service/caches@2418-06-01-preview
Fake.ApiManagement/service/caches@2419-01-01
Fake.ApiManagement/service/caches@2419-12-01
Fake.ApiManagement/service/caches@2419-12-01-preview
Fake.ApiManagement/service/caches@2420-06-01-preview
Fake.ApiManagement/service/caches@2420-12-01
Fake.ApiManagement/service/caches@2421-01-01-preview
Fake.ApiManagement/service/apiVersionSets@2419-01-01
Fake.ApiManagement/service/apiVersionSets@2419-12-01
Fake.ApiManagement/service/apiVersionSets@2419-12-01-preview
Fake.ApiManagement/service/apiVersionSets@2420-06-01-preview
Fake.ApiManagement/service/apiVersionSets@2420-12-01
Fake.ApiManagement/service/apiVersionSets@2421-01-01-preview
Fake.ApiManagement/service/contentTypes@2419-12-01
Fake.ApiManagement/service/contentTypes@2420-06-01-preview
Fake.ApiManagement/service/contentTypes@2420-12-01
Fake.ApiManagement/service/contentTypes@2421-01-01-preview
Fake.ApiManagement/service/contentTypes/contentItems@2419-12-01
Fake.ApiManagement/service/contentTypes/contentItems@2420-06-01-preview
Fake.ApiManagement/service/contentTypes/contentItems@2420-12-01
Fake.ApiManagement/service/contentTypes/contentItems@2421-01-01-preview
Fake.ApiManagement/service/gateways@2419-12-01
Fake.ApiManagement/service/gateways@2419-12-01-preview
Fake.ApiManagement/service/gateways@2420-06-01-preview
Fake.ApiManagement/service/gateways@2420-12-01
Fake.ApiManagement/service/gateways@2421-01-01-preview
Fake.ApiManagement/service/gateways/apis@2419-12-01
Fake.ApiManagement/service/gateways/apis@2419-12-01-preview
Fake.ApiManagement/service/gateways/apis@2420-06-01-preview
Fake.ApiManagement/service/gateways/apis@2420-12-01
Fake.ApiManagement/service/gateways/apis@2421-01-01-preview
Fake.ApiManagement/service/gateways/hostnameConfigurations@2419-12-01
Fake.ApiManagement/service/gateways/hostnameConfigurations@2419-12-01-preview
Fake.ApiManagement/service/gateways/hostnameConfigurations@2420-06-01-preview
Fake.ApiManagement/service/gateways/hostnameConfigurations@2420-12-01
Fake.ApiManagement/service/gateways/hostnameConfigurations@2421-01-01-preview
Fake.ApiManagement/service/namedValues@2419-12-01
Fake.ApiManagement/service/namedValues@2419-12-01-preview
Fake.ApiManagement/service/namedValues@2420-06-01-preview
Fake.ApiManagement/service/namedValues@2420-12-01
Fake.ApiManagement/service/namedValues@2421-01-01-preview
Fake.ApiManagement/service/gateways/certificateAuthorities@2420-06-01-preview
Fake.ApiManagement/service/gateways/certificateAuthorities@2420-12-01
Fake.ApiManagement/service/gateways/certificateAuthorities@2421-01-01-preview
Fake.ApiManagement/service/portalRevisions@2420-06-01-preview
Fake.ApiManagement/service/portalRevisions@2420-12-01
Fake.ApiManagement/service/portalRevisions@2421-01-01-preview
Fake.AppConfiguration/configurationStores@2419-02-01-preview
Fake.AppConfiguration/configurationStores@2419-10-01
Fake.AppConfiguration/configurationStores@2419-11-01-preview
Fake.AppConfiguration/configurationStores@2420-06-01
Fake.AppConfiguration/configurationStores@2420-07-01-preview
Fake.AppConfiguration/configurationStores@2421-03-01-preview
Fake.AppConfiguration/configurationStores/privateEndpointConnections@2419-11-01-preview
Fake.AppConfiguration/configurationStores/privateEndpointConnections@2420-06-01
Fake.AppConfiguration/configurationStores/privateEndpointConnections@2420-07-01-preview
Fake.AppConfiguration/configurationStores/privateEndpointConnections@2421-03-01-preview
Fake.AppConfiguration/configurationStores/keyValues@2420-07-01-preview
Fake.AppConfiguration/configurationStores/keyValues@2421-03-01-preview
Fake.AppPlatform/Spring/apps/domains@2419-05-01-preview
Fake.AppPlatform/Spring/apps/domains@2420-07-01
Fake.AppPlatform/Spring/apps/domains@2420-11-01-preview
Fake.AppPlatform/Spring/apps/domains@2421-06-01-preview
Fake.AppPlatform/Spring/certificates@2419-05-01-preview
Fake.AppPlatform/Spring/certificates@2420-07-01
Fake.AppPlatform/Spring/certificates@2420-11-01-preview
Fake.AppPlatform/Spring/certificates@2421-06-01-preview
Fake.AppPlatform/Spring/configServers@2420-07-01
Fake.AppPlatform/Spring/configServers@2420-11-01-preview
Fake.AppPlatform/Spring/configServers@2421-06-01-preview
Fake.AppPlatform/Spring/monitoringSettings@2420-07-01
Fake.AppPlatform/Spring/monitoringSettings@2420-11-01-preview
Fake.AppPlatform/Spring/monitoringSettings@2421-06-01-preview
Fake.Attestation/attestationProviders@2418-09-01-preview
Fake.Attestation/attestationProviders@2420-10-01
Fake.Attestation/attestationProviders@2421-06-01-preview
Fake.Attestation/attestationProviders/privateEndpointConnections@2420-10-01
Fake.Attestation/attestationProviders/privateEndpointConnections@2421-06-01-preview
Fake.Authorization/roleAssignments@2415-07-01
Fake.Authorization/roleAssignments@2417-10-01-preview
Fake.Authorization/roleAssignments@2418-01-01-preview
Fake.Authorization/roleAssignments@2418-09-01-preview
Fake.Authorization/roleAssignments@2420-03-01-preview
Fake.Authorization/roleAssignments@2420-04-01-preview
Fake.Authorization/roleAssignments@2420-08-01-preview
Fake.Authorization/roleAssignments@2420-10-01-preview
Fake.Authorization/roleDefinitions@2415-07-01
Fake.Authorization/roleDefinitions@2418-01-01-preview
Fake.Authorization/roleAssignmentScheduleRequests@2420-10-01-preview
Fake.Authorization/roleEligibilityScheduleRequests@2420-10-01-preview
Fake.Authorization/roleManagementPolicyAssignments@2420-10-01-preview
Fake.Authorization/locks@2415-01-01
Fake.Authorization/locks@2416-09-01
Fake.Authorization/locks@2417-04-01
Fake.Authorization/locks@2420-05-01
Fake.Authorization/policyassignments@2415-10-01-preview
Fake.Authorization/policyassignments@2416-04-01
Fake.Authorization/policyassignments@2416-12-01
Fake.Authorization/policyassignments@2417-06-01-preview
Fake.Authorization/policyassignments@2418-03-01
Fake.Authorization/policyassignments@2418-05-01
Fake.Authorization/policyassignments@2419-01-01
Fake.Authorization/policyassignments@2419-06-01
Fake.Authorization/policyassignments@2419-09-01
Fake.Authorization/policyassignments@2420-03-01
Fake.Authorization/policyassignments@2420-09-01
Fake.Authorization/policyassignments@2421-06-01
Fake.Authorization/resourceManagementPrivateLinks@2420-05-01
Fake.Authorization/policyExemptions@2420-07-01-preview
Fake.Automanage/accounts@2420-06-30-preview
Fake.Automanage/configurationProfilePreferences@2420-06-30-preview
Fake.Automation/automationAccounts@2415-10-31
Fake.Automation/automationAccounts@2419-06-01
Fake.Automation/automationAccounts@2420-01-13-preview
Fake.Automation/automationAccounts@2421-06-22
Fake.Automation/automationAccounts/certificates@2415-10-31
Fake.Automation/automationAccounts/certificates@2419-06-01
Fake.Automation/automationAccounts/certificates@2420-01-13-preview
Fake.Automation/automationAccounts/compilationjobs@2415-10-31
Fake.Automation/automationAccounts/compilationjobs@2418-01-15
Fake.Automation/automationAccounts/compilationjobs@2419-06-01
Fake.Automation/automationAccounts/compilationjobs@2420-01-13-preview
Fake.Automation/automationAccounts/configurations@2415-10-31
Fake.Automation/automationAccounts/configurations@2419-06-01
Fake.Automation/automationAccounts/connections@2415-10-31
Fake.Automation/automationAccounts/connections@2419-06-01
Fake.Automation/automationAccounts/connections@2420-01-13-preview
Fake.Automation/automationAccounts/connectionTypes@2415-10-31
Fake.Automation/automationAccounts/connectionTypes@2419-06-01
Fake.Automation/automationAccounts/connectionTypes@2420-01-13-preview
Fake.Automation/automationAccounts/credentials@2415-10-31
Fake.Automation/automationAccounts/credentials@2419-06-01
Fake.Automation/automationAccounts/credentials@2420-01-13-preview
Fake.Automation/automationAccounts/jobs@2415-10-31
Fake.Automation/automationAccounts/jobs@2417-05-15-preview
Fake.Automation/automationAccounts/jobs@2419-06-01
Fake.Automation/automationAccounts/jobSchedules@2415-10-31
Fake.Automation/automationAccounts/jobSchedules@2419-06-01
Fake.Automation/automationAccounts/jobSchedules@2420-01-13-preview
Fake.Automation/automationAccounts/modules@2415-10-31
Fake.Automation/automationAccounts/modules@2419-06-01
Fake.Automation/automationAccounts/modules@2420-01-13-preview
Fake.Automation/automationAccounts/nodeConfigurations@2415-10-31
Fake.Automation/automationAccounts/nodeConfigurations@2418-01-15
Fake.Automation/automationAccounts/nodeConfigurations@2419-06-01
Fake.Automation/automationAccounts/nodeConfigurations@2420-01-13-preview
Fake.Automation/automationAccounts/runbooks@2415-10-31
Fake.Automation/automationAccounts/runbooks@2418-06-30
Fake.Automation/automationAccounts/runbooks@2419-06-01
Fake.Automation/automationAccounts/runbooks/draft@2415-10-31
Fake.Automation/automationAccounts/runbooks/draft@2418-06-30
Fake.Automation/automationAccounts/runbooks/draft@2419-06-01
Fake.Automation/automationAccounts/schedules@2415-10-31
Fake.Automation/automationAccounts/schedules@2419-06-01
Fake.Automation/automationAccounts/schedules@2420-01-13-preview
Fake.Automation/automationAccounts/variables@2415-10-31
Fake.Automation/automationAccounts/variables@2419-06-01
Fake.Automation/automationAccounts/variables@2420-01-13-preview
Fake.Automation/automationAccounts/watchers@2415-10-31
Fake.Automation/automationAccounts/watchers@2419-06-01
Fake.Automation/automationAccounts/watchers@2420-01-13-preview
Fake.Automation/automationAccounts/webhooks@2415-10-31
Fake.Automation/automationAccounts/softwareUpdateConfigurations@2417-05-15-preview
Fake.Automation/automationAccounts/softwareUpdateConfigurations@2419-06-01
Fake.Automation/automationAccounts/sourceControls@2417-05-15-preview
Fake.Automation/automationAccounts/sourceControls@2419-06-01
Fake.Automation/automationAccounts/sourceControls@2420-01-13-preview
Fake.Automation/automationAccounts/sourceControls/sourceControlSyncJobs@2417-05-15-preview
Fake.Automation/automationAccounts/sourceControls/sourceControlSyncJobs@2419-06-01
Fake.Automation/automationAccounts/sourceControls/sourceControlSyncJobs@2420-01-13-preview
Fake.Automation/automationAccounts/python2Packages@2418-06-30
Fake.Automation/automationAccounts/python2Packages@2419-06-01
Fake.Automation/automationAccounts/python2Packages@2420-01-13-preview
Fake.Automation/automationAccounts/privateEndpointConnections@2420-01-13-preview
Fake.Automation/automationAccounts/hybridRunbookWorkerGroups@2421-06-22
Fake.Automation/automationAccounts/hybridRunbookWorkerGroups/hybridRunbookWorkers@2421-06-22
Fake.AutonomousDevelopmentPlatform/accounts@2420-07-01-preview
Fake.AutonomousDevelopmentPlatform/accounts@2421-02-01-preview
Fake.AutonomousDevelopmentPlatform/accounts/dataPools@2420-07-01-preview
Fake.AutonomousDevelopmentPlatform/accounts/dataPools@2421-02-01-preview
Fake.AVS/privateClouds@2420-03-20
Fake.AVS/privateClouds@2420-07-17-preview
Fake.AVS/privateClouds@2421-01-01-preview
Fake.AVS/privateClouds@2421-06-01
Fake.AVS/privateClouds/authorizations@2420-03-20
Fake.AVS/privateClouds/authorizations@2420-07-17-preview
Fake.AVS/privateClouds/authorizations@2421-01-01-preview
Fake.AVS/privateClouds/authorizations@2421-06-01
Fake.AVS/privateClouds/clusters@2420-03-20
Fake.AVS/privateClouds/clusters@2420-07-17-preview
Fake.AVS/privateClouds/clusters@2421-01-01-preview
Fake.AVS/privateClouds/clusters@2421-06-01
Fake.AVS/privateClouds/hcxEnterpriseSites@2420-03-20
Fake.AVS/privateClouds/hcxEnterpriseSites@2420-07-17-preview
Fake.AVS/privateClouds/hcxEnterpriseSites@2421-01-01-preview
Fake.AVS/privateClouds/hcxEnterpriseSites@2421-06-01
Fake.AVS/privateClouds/addons@2420-07-17-preview
Fake.AVS/privateClouds/addons@2421-01-01-preview
Fake.AVS/privateClouds/addons@2421-06-01
Fake.AVS/privateClouds/globalReachConnections@2420-07-17-preview
Fake.AVS/privateClouds/globalReachConnections@2421-01-01-preview
Fake.AVS/privateClouds/globalReachConnections@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/dhcpConfigurations@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/dhcpConfigurations@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/dhcpConfigurations@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/dnsServices@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/dnsServices@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/dnsServices@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/dnsZones@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/dnsZones@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/dnsZones@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/portMirroringProfiles@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/portMirroringProfiles@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/portMirroringProfiles@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/segments@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/segments@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/segments@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/vmGroups@2420-07-17-preview
Fake.AVS/privateClouds/workloadNetworks/vmGroups@2421-01-01-preview
Fake.AVS/privateClouds/workloadNetworks/vmGroups@2421-06-01
Fake.AVS/privateClouds/clusters/datastores@2421-01-01-preview
Fake.AVS/privateClouds/clusters/datastores@2421-06-01
Fake.AVS/privateClouds/cloudLinks@2421-06-01
Fake.AVS/privateClouds/scriptExecutions@2421-06-01
Fake.AVS/privateClouds/workloadNetworks/publicIPs@2421-06-01
Fake.AzureArcData/dataControllers@2421-06-01-preview
Fake.AzureArcData/dataControllers@2421-07-01-preview
Fake.AzureArcData/dataControllers@2421-08-01
Fake.AzureArcData/postgresInstances@2421-06-01-preview
Fake.AzureArcData/postgresInstances@2421-07-01-preview
Fake.AzureArcData/sqlManagedInstances@2421-06-01-preview
Fake.AzureArcData/sqlManagedInstances@2421-07-01-preview
Fake.AzureArcData/sqlManagedInstances@2421-08-01
Fake.AzureArcData/sqlServerInstances@2421-06-01-preview
Fake.AzureArcData/sqlServerInstances@2421-07-01-preview
Fake.AzureArcData/sqlServerInstances@2421-08-01
Fake.AzureData/sqlServerRegistrations@2419-07-24-preview
Fake.AzureData/sqlServerRegistrations/sqlServers@2419-07-24-preview
Fake.AzureStack/registrations@2416-01-01
Fake.AzureStack/registrations@2417-06-01
Fake.AzureStack/registrations@2420-06-01-preview
Fake.AzureStack/registrations/customerSubscriptions@2417-06-01
Fake.AzureStack/registrations/customerSubscriptions@2420-06-01-preview
Fake.AzureStack/linkedSubscriptions@2420-06-01-preview
Fake.Batch/batchAccounts@2415-12-01
Fake.Batch/batchAccounts@2417-01-01
Fake.Batch/batchAccounts@2417-05-01
Fake.Batch/batchAccounts@2417-09-01
Fake.Batch/batchAccounts@2418-12-01
Fake.Batch/batchAccounts@2419-04-01
Fake.Batch/batchAccounts@2419-08-01
Fake.Batch/batchAccounts@2420-03-01
Fake.Batch/batchAccounts@2420-05-01
Fake.Batch/batchAccounts@2420-09-01
Fake.Batch/batchAccounts@2421-01-01
Fake.Batch/batchAccounts@2421-06-01
Fake.Batch/batchAccounts/applications@2415-12-01
Fake.Batch/batchAccounts/applications@2417-01-01
Fake.Batch/batchAccounts/applications@2417-05-01
Fake.Batch/batchAccounts/applications@2417-09-01
Fake.Batch/batchAccounts/applications@2418-12-01
Fake.Batch/batchAccounts/applications@2419-04-01
Fake.Batch/batchAccounts/applications@2419-08-01
Fake.Batch/batchAccounts/applications@2420-03-01
Fake.Batch/batchAccounts/applications@2420-05-01
Fake.Batch/batchAccounts/applications@2420-09-01
Fake.Batch/batchAccounts/applications@2421-01-01
Fake.Batch/batchAccounts/applications@2421-06-01
Fake.Batch/batchAccounts/applications/versions@2415-12-01
Fake.Batch/batchAccounts/applications/versions@2417-01-01
Fake.Batch/batchAccounts/applications/versions@2417-05-01
Fake.Batch/batchAccounts/applications/versions@2417-09-01
Fake.Batch/batchAccounts/applications/versions@2418-12-01
Fake.Batch/batchAccounts/applications/versions@2419-04-01
Fake.Batch/batchAccounts/applications/versions@2419-08-01
Fake.Batch/batchAccounts/applications/versions@2420-03-01
Fake.Batch/batchAccounts/applications/versions@2420-05-01
Fake.Batch/batchAccounts/applications/versions@2420-09-01
Fake.Batch/batchAccounts/applications/versions@2421-01-01
Fake.Batch/batchAccounts/applications/versions@2421-06-01
Fake.Batch/batchAccounts/certificates@2417-09-01
Fake.Batch/batchAccounts/certificates@2418-12-01
Fake.Batch/batchAccounts/certificates@2419-04-01
Fake.Batch/batchAccounts/certificates@2419-08-01
Fake.Batch/batchAccounts/certificates@2420-03-01
Fake.Batch/batchAccounts/certificates@2420-05-01
Fake.Batch/batchAccounts/certificates@2420-09-01
Fake.Batch/batchAccounts/certificates@2421-01-01
Fake.Batch/batchAccounts/certificates@2421-06-01
Fake.Batch/batchAccounts/pools@2417-09-01
Fake.Batch/batchAccounts/pools@2418-12-01
Fake.Batch/batchAccounts/pools@2419-04-01
Fake.Batch/batchAccounts/pools@2419-08-01
Fake.Batch/batchAccounts/pools@2420-03-01
Fake.Batch/batchAccounts/pools@2420-05-01
Fake.Batch/batchAccounts/pools@2420-09-01
Fake.Batch/batchAccounts/pools@2421-01-01
Fake.Batch/batchAccounts/pools@2421-06-01
Fake.BatchAI/clusters@2417-09-01-preview
Fake.BatchAI/clusters@2418-03-01
Fake.BatchAI/fileServers@2417-09-01-preview
Fake.BatchAI/fileServers@2418-03-01
Fake.BatchAI/jobs@2417-09-01-preview
Fake.BatchAI/jobs@2418-03-01
Fake.BatchAI/workspaces@2418-05-01
Fake.BatchAI/workspaces/clusters@2418-05-01
Fake.BatchAI/workspaces/experiments@2418-05-01
Fake.BatchAI/workspaces/experiments/jobs@2418-05-01
Fake.BatchAI/workspaces/fileServers@2418-05-01
Fake.Blockchain/blockchainMembers@2418-06-01-preview
Fake.Blockchain/blockchainMembers/transactionNodes@2418-06-01-preview
Fake.BotService/botServices@2417-12-01
Fake.BotService/botServices@2418-07-12
Fake.BotService/botServices@2420-06-02
Fake.BotService/botServices@2421-03-01
Fake.BotService/botServices@2421-05-01-preview
Fake.BotService/botServices/channels@2417-12-01
Fake.BotService/botServices/channels@2418-07-12
Fake.BotService/botServices/channels@2420-06-02
Fake.BotService/botServices/channels@2421-03-01
Fake.BotService/botServices/channels@2421-05-01-preview
Fake.BotService/botServices/Connections@2417-12-01
Fake.BotService/botServices/Connections@2418-07-12
Fake.BotService/botServices/Connections@2420-06-02
Fake.BotService/botServices/Connections@2421-03-01
Fake.BotService/botServices/Connections@2421-05-01-preview
Fake.BotService/enterpriseChannels@2418-07-12
Fake.BotService/botServices/privateEndpointConnections@2421-05-01-preview
Fake.Cache/redisEnterprise@2420-10-01-preview
Fake.Cache/redisEnterprise@2421-02-01-preview
Fake.Cache/redisEnterprise@2421-03-01
Fake.Cache/redisEnterprise@2421-08-01
Fake.Cache/redisEnterprise/databases@2420-10-01-preview
Fake.Cache/redisEnterprise/databases@2421-02-01-preview
Fake.Cache/redisEnterprise/databases@2421-03-01
Fake.Cache/redisEnterprise/databases@2421-08-01
Fake.Cache/redisEnterprise/privateEndpointConnections@2420-10-01-preview
Fake.Cache/redisEnterprise/privateEndpointConnections@2421-02-01-preview
Fake.Cache/redisEnterprise/privateEndpointConnections@2421-03-01
Fake.Cache/redisEnterprise/privateEndpointConnections@2421-08-01
Fake.Cache/Redis@2415-08-01
Fake.Cache/Redis@2416-04-01
Fake.Cache/Redis@2417-02-01
Fake.Cache/Redis@2417-10-01
Fake.Cache/Redis@2418-03-01
Fake.Cache/Redis@2419-07-01
Fake.Cache/Redis@2420-06-01
Fake.Cache/Redis@2420-12-01
Fake.Cache/Redis/firewallRules@2416-04-01
Fake.Cache/Redis/firewallRules@2417-02-01
Fake.Cache/Redis/firewallRules@2417-10-01
Fake.Cache/Redis/firewallRules@2418-03-01
Fake.Cache/Redis/firewallRules@2419-07-01
Fake.Cache/Redis/firewallRules@2420-06-01
Fake.Cache/Redis/firewallRules@2420-12-01
Fake.Cache/Redis/patchSchedules@2416-04-01
Fake.Cache/Redis/patchSchedules@2417-02-01
Fake.Cache/Redis/patchSchedules@2417-10-01
Fake.Cache/Redis/patchSchedules@2418-03-01
Fake.Cache/Redis/patchSchedules@2419-07-01
Fake.Cache/Redis/patchSchedules@2420-06-01
Fake.Cache/Redis/patchSchedules@2420-12-01
Fake.Cache/Redis/linkedServers@2417-02-01
Fake.Cache/Redis/linkedServers@2417-10-01
Fake.Cache/Redis/linkedServers@2418-03-01
Fake.Cache/Redis/linkedServers@2419-07-01
Fake.Cache/Redis/linkedServers@2420-06-01
Fake.Cache/Redis/linkedServers@2420-12-01
Fake.Cache/redis/privateEndpointConnections@2420-06-01
Fake.Cache/redis/privateEndpointConnections@2420-12-01
Fake.Cdn/profiles@2415-06-01
Fake.Cdn/profiles@2416-04-02
Fake.Cdn/profiles@2416-10-02
Fake.Cdn/profiles@2417-04-02
Fake.Cdn/profiles@2417-10-12
Fake.Cdn/profiles@2419-04-15
Fake.Cdn/profiles@2419-06-15
Fake.Cdn/profiles@2419-06-15-preview
Fake.Cdn/profiles@2419-12-31
Fake.Cdn/profiles@2420-04-15
Fake.Cdn/profiles@2420-09-01
Fake.Cdn/profiles/endpoints@2415-06-01
Fake.Cdn/profiles/endpoints@2416-04-02
Fake.Cdn/profiles/endpoints@2416-10-02
Fake.Cdn/profiles/endpoints@2417-04-02
Fake.Cdn/profiles/endpoints@2417-10-12
Fake.Cdn/profiles/endpoints@2419-04-15
Fake.Cdn/profiles/endpoints@2419-06-15
Fake.Cdn/profiles/endpoints@2419-06-15-preview
Fake.Cdn/profiles/endpoints@2419-12-31
Fake.Cdn/profiles/endpoints@2420-04-15
Fake.Cdn/profiles/endpoints@2420-09-01
Fake.Cdn/profiles/endpoints/customDomains@2415-06-01
Fake.Cdn/profiles/endpoints/customDomains@2416-04-02
Fake.Cdn/profiles/endpoints/customDomains@2416-10-02
Fake.Cdn/profiles/endpoints/customDomains@2417-04-02
Fake.Cdn/profiles/endpoints/customDomains@2417-10-12
Fake.Cdn/profiles/endpoints/customDomains@2419-04-15
Fake.Cdn/profiles/endpoints/customDomains@2419-06-15
Fake.Cdn/profiles/endpoints/customDomains@2419-06-15-preview
Fake.Cdn/profiles/endpoints/customDomains@2419-12-31
Fake.Cdn/profiles/endpoints/customDomains@2420-04-15
Fake.Cdn/profiles/endpoints/customDomains@2420-09-01
Fake.Cdn/profiles/endpoints/origins@2415-06-01
Fake.Cdn/profiles/endpoints/origins@2416-04-02
Fake.Cdn/profiles/endpoints/origins@2419-12-31
Fake.Cdn/profiles/endpoints/origins@2420-04-15
Fake.Cdn/profiles/endpoints/origins@2420-09-01
Fake.Cdn/CdnWebApplicationFirewallPolicies@2419-06-15
Fake.Cdn/CdnWebApplicationFirewallPolicies@2419-06-15-preview
Fake.Cdn/CdnWebApplicationFirewallPolicies@2420-04-15
Fake.Cdn/CdnWebApplicationFirewallPolicies@2420-09-01
Fake.Cdn/profiles/endpoints/originGroups@2419-12-31
Fake.Cdn/profiles/endpoints/originGroups@2420-04-15
Fake.Cdn/profiles/endpoints/originGroups@2420-09-01
Fake.Cdn/profiles/afdEndpoints@2420-09-01
Fake.Cdn/profiles/afdEndpoints/routes@2420-09-01
Fake.Cdn/profiles/customDomains@2420-09-01
Fake.Cdn/profiles/originGroups@2420-09-01
Fake.Cdn/profiles/originGroups/origins@2420-09-01
Fake.Cdn/profiles/ruleSets@2420-09-01
Fake.Cdn/profiles/ruleSets/rules@2420-09-01
Fake.Cdn/profiles/secrets@2420-09-01
Fake.Cdn/profiles/securityPolicies@2420-09-01
Fake.CertificateRegistration/certificateOrders@2415-08-01
Fake.CertificateRegistration/certificateOrders@2418-02-01
Fake.CertificateRegistration/certificateOrders@2419-08-01
Fake.CertificateRegistration/certificateOrders@2420-06-01
Fake.CertificateRegistration/certificateOrders@2420-09-01
Fake.CertificateRegistration/certificateOrders@2420-10-01
Fake.CertificateRegistration/certificateOrders@2420-12-01
Fake.CertificateRegistration/certificateOrders@2421-01-01
Fake.CertificateRegistration/certificateOrders@2421-01-15
Fake.CertificateRegistration/certificateOrders@2421-02-01
Fake.CertificateRegistration/certificateOrders/certificates@2415-08-01
Fake.CertificateRegistration/certificateOrders/certificates@2418-02-01
Fake.CertificateRegistration/certificateOrders/certificates@2419-08-01
Fake.CertificateRegistration/certificateOrders/certificates@2420-06-01
Fake.CertificateRegistration/certificateOrders/certificates@2420-09-01
Fake.CertificateRegistration/certificateOrders/certificates@2420-10-01
Fake.CertificateRegistration/certificateOrders/certificates@2420-12-01
Fake.CertificateRegistration/certificateOrders/certificates@2421-01-01
Fake.CertificateRegistration/certificateOrders/certificates@2421-01-15
Fake.CertificateRegistration/certificateOrders/certificates@2421-02-01
Fake.CognitiveServices/accounts@2416-02-01-preview
Fake.CognitiveServices/accounts@2417-04-18
Fake.CognitiveServices/accounts@2421-04-30
Fake.CognitiveServices/accounts/privateEndpointConnections@2417-04-18
Fake.CognitiveServices/accounts/privateEndpointConnections@2421-04-30
Fake.Communication/communicationServices@2420-08-20
Fake.Communication/communicationServices@2420-08-20-preview
Fake.Confluent/organizations@2420-03-01
Fake.Confluent/organizations@2420-03-01-preview
Fake.Confluent/organizations@2421-03-01-preview
Fake.Consumption/budgets@2418-01-31
Fake.Consumption/budgets@2418-03-31
Fake.Consumption/budgets@2418-06-30
Fake.Consumption/budgets@2418-08-31
Fake.Consumption/budgets@2418-10-01
Fake.Consumption/budgets@2419-01-01
Fake.Consumption/budgets@2419-04-01-preview
Fake.Consumption/budgets@2419-05-01
Fake.Consumption/budgets@2419-05-01-preview
Fake.Consumption/budgets@2419-06-01
Fake.Consumption/budgets@2419-10-01
Fake.Consumption/budgets@2419-11-01
Fake.Consumption/budgets@2421-05-01
Fake.ContainerInstance/containerGroups@2417-08-01-preview
Fake.ContainerInstance/containerGroups@2417-10-01-preview
Fake.ContainerInstance/containerGroups@2417-12-01-preview
Fake.ContainerInstance/containerGroups@2418-02-01-preview
Fake.ContainerInstance/containerGroups@2418-04-01
Fake.ContainerInstance/containerGroups@2418-06-01
Fake.ContainerInstance/containerGroups@2418-09-01
Fake.ContainerInstance/containerGroups@2418-10-01
Fake.ContainerInstance/containerGroups@2419-12-01
Fake.ContainerInstance/containerGroups@2420-11-01
Fake.ContainerInstance/containerGroups@2421-03-01
Fake.ContainerInstance/containerGroups@2421-07-01
Fake.ContainerRegistry/registries@2416-06-27-preview
Fake.ContainerRegistry/registries@2417-03-01
Fake.ContainerRegistry/registries@2417-06-01-preview
Fake.ContainerRegistry/registries@2417-10-01
Fake.ContainerRegistry/registries@2419-05-01
Fake.ContainerRegistry/registries@2419-12-01-preview
Fake.ContainerRegistry/registries@2420-11-01-preview
Fake.ContainerRegistry/registries@2421-06-01-preview
Fake.ContainerRegistry/registries/replications@2417-06-01-preview
Fake.ContainerRegistry/registries/replications@2417-10-01
Fake.ContainerRegistry/registries/replications@2419-05-01
Fake.ContainerRegistry/registries/replications@2419-12-01-preview
Fake.ContainerRegistry/registries/replications@2420-11-01-preview
Fake.ContainerRegistry/registries/replications@2421-06-01-preview
Fake.ContainerRegistry/registries/webhooks@2417-06-01-preview
Fake.ContainerRegistry/registries/webhooks@2417-10-01
Fake.ContainerRegistry/registries/webhooks@2419-05-01
Fake.ContainerRegistry/registries/webhooks@2419-12-01-preview
Fake.ContainerRegistry/registries/webhooks@2420-11-01-preview
Fake.ContainerRegistry/registries/webhooks@2421-06-01-preview
Fake.ContainerRegistry/registries/buildTasks@2418-02-01-preview
Fake.ContainerRegistry/registries/buildTasks/steps@2418-02-01-preview
Fake.ContainerRegistry/registries/tasks@2418-09-01
Fake.ContainerRegistry/registries/tasks@2419-04-01
Fake.ContainerRegistry/registries/tasks@2419-06-01-preview
Fake.ContainerRegistry/registries/scopeMaps@2419-05-01-preview
Fake.ContainerRegistry/registries/scopeMaps@2420-11-01-preview
Fake.ContainerRegistry/registries/scopeMaps@2421-06-01-preview
Fake.ContainerRegistry/registries/tokens@2419-05-01-preview
Fake.ContainerRegistry/registries/tokens@2420-11-01-preview
Fake.ContainerRegistry/registries/tokens@2421-06-01-preview
Fake.ContainerRegistry/registries/agentPools@2419-06-01-preview
Fake.ContainerRegistry/registries/taskRuns@2419-06-01-preview
Fake.ContainerRegistry/registries/exportPipelines@2419-12-01-preview
Fake.ContainerRegistry/registries/exportPipelines@2420-11-01-preview
Fake.ContainerRegistry/registries/exportPipelines@2421-06-01-preview
Fake.ContainerRegistry/registries/importPipelines@2419-12-01-preview
Fake.ContainerRegistry/registries/importPipelines@2420-11-01-preview
Fake.ContainerRegistry/registries/importPipelines@2421-06-01-preview
Fake.ContainerRegistry/registries/pipelineRuns@2419-12-01-preview
Fake.ContainerRegistry/registries/pipelineRuns@2420-11-01-preview
Fake.ContainerRegistry/registries/pipelineRuns@2421-06-01-preview
Fake.ContainerRegistry/registries/privateEndpointConnections@2419-12-01-preview
Fake.ContainerRegistry/registries/privateEndpointConnections@2420-11-01-preview
Fake.ContainerRegistry/registries/privateEndpointConnections@2421-06-01-preview
Fake.ContainerRegistry/registries/connectedRegistries@2420-11-01-preview
Fake.ContainerRegistry/registries/connectedRegistries@2421-06-01-preview
Fake.ContainerService/containerServices@2416-03-30
Fake.ContainerService/containerServices@2416-09-30
Fake.ContainerService/containerServices@2417-01-31
Fake.ContainerService/containerServices@2417-07-01
Fake.ContainerService/managedClusters@2417-08-31
Fake.ContainerService/managedClusters@2418-03-31
Fake.ContainerService/managedClusters@2418-08-01-preview
Fake.ContainerService/managedClusters@2419-02-01
Fake.ContainerService/managedClusters@2419-04-01
Fake.ContainerService/managedClusters@2419-06-01
Fake.ContainerService/managedClusters@2419-08-01
Fake.ContainerService/managedClusters@2419-10-01
Fake.ContainerService/managedClusters@2419-11-01
Fake.ContainerService/managedClusters@2420-01-01
Fake.ContainerService/managedClusters@2420-02-01
Fake.ContainerService/managedClusters@2420-03-01
Fake.ContainerService/managedClusters@2420-04-01
Fake.ContainerService/managedClusters@2420-06-01
Fake.ContainerService/managedClusters@2420-07-01
Fake.ContainerService/managedClusters@2420-09-01
Fake.ContainerService/managedClusters@2420-11-01
Fake.ContainerService/managedClusters@2420-12-01
Fake.ContainerService/managedClusters@2421-02-01
Fake.ContainerService/managedClusters@2421-03-01
Fake.ContainerService/managedClusters@2421-05-01
Fake.ContainerService/managedClusters@2421-07-01
Fake.ContainerService/managedClusters@2421-08-01
Fake.ContainerService/openShiftManagedClusters@2418-09-30-preview
Fake.ContainerService/openShiftManagedClusters@2419-04-30
Fake.ContainerService/openShiftManagedClusters@2419-09-30-preview
Fake.ContainerService/openShiftManagedClusters@2419-10-27-preview
Fake.ContainerService/managedClusters/agentPools@2419-02-01
Fake.ContainerService/managedClusters/agentPools@2419-04-01
Fake.ContainerService/managedClusters/agentPools@2419-06-01
Fake.ContainerService/managedClusters/agentPools@2419-08-01
Fake.ContainerService/managedClusters/agentPools@2419-10-01
Fake.ContainerService/managedClusters/agentPools@2419-11-01
Fake.ContainerService/managedClusters/agentPools@2420-01-01
Fake.ContainerService/managedClusters/agentPools@2420-02-01
Fake.ContainerService/managedClusters/agentPools@2420-03-01
Fake.ContainerService/managedClusters/agentPools@2420-04-01
Fake.ContainerService/managedClusters/agentPools@2420-06-01
Fake.ContainerService/managedClusters/agentPools@2420-07-01
Fake.ContainerService/managedClusters/agentPools@2420-09-01
Fake.ContainerService/managedClusters/agentPools@2420-11-01
Fake.ContainerService/managedClusters/agentPools@2420-12-01
Fake.ContainerService/managedClusters/agentPools@2421-02-01
Fake.ContainerService/managedClusters/agentPools@2421-03-01
Fake.ContainerService/managedClusters/agentPools@2421-05-01
Fake.ContainerService/managedClusters/agentPools@2421-07-01
Fake.ContainerService/managedClusters/agentPools@2421-08-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2420-06-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2420-07-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2420-09-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2420-11-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2420-12-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2421-02-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2421-03-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2421-05-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2421-07-01
Fake.ContainerService/managedClusters/privateEndpointConnections@2421-08-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2420-12-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2421-02-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2421-03-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2421-05-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2421-07-01
Fake.ContainerService/managedClusters/maintenanceConfigurations@2421-08-01
Fake.ContainerService/snapshots@2421-08-01
Fake.CustomProviders/resourceProviders@2418-09-01-preview
Fake.DataBox/jobs@2418-01-01
Fake.DataBox/jobs@2419-09-01
Fake.DataBox/jobs@2420-04-01
Fake.DataBox/jobs@2420-11-01
Fake.DataBox/jobs@2421-03-01
Fake.DataBox/jobs@2421-05-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/bandwidthSchedules@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/orders@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/shares@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/triggers@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2419-03-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2419-07-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/users@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2419-08-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2420-05-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/storageAccounts/containers@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/addons@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/addons@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/addons@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/addons@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/addons@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/monitoringConfig@2420-09-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/monitoringConfig@2420-09-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/monitoringConfig@2420-12-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/monitoringConfig@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/roles/monitoringConfig@2421-02-01-preview
Fake.DataBoxEdge/dataBoxEdgeDevices/diagnosticProactiveLogCollectionSettings@2421-02-01
Fake.DataBoxEdge/dataBoxEdgeDevices/diagnosticRemoteSupportSettings@2421-02-01
Fake.Databricks/workspaces@2418-04-01
Fake.Databricks/workspaces@2421-04-01-preview
Fake.Databricks/workspaces/virtualNetworkPeerings@2418-04-01
Fake.Databricks/workspaces/virtualNetworkPeerings@2421-04-01-preview
Fake.Databricks/workspaces/privateEndpointConnections@2421-04-01-preview
Fake.DataCatalog/catalogs@2416-03-30
Fake.Datadog/monitors@2420-02-01-preview
Fake.Datadog/monitors@2421-03-01
Fake.Datadog/monitors/singleSignOnConfigurations@2420-02-01-preview
Fake.Datadog/monitors/singleSignOnConfigurations@2421-03-01
Fake.Datadog/monitors/tagRules@2420-02-01-preview
Fake.Datadog/monitors/tagRules@2421-03-01
Fake.DataFactory/factories@2417-09-01-preview
Fake.DataFactory/factories@2418-06-01
Fake.DataFactory/factories/datasets@2417-09-01-preview
Fake.DataFactory/factories/datasets@2418-06-01
Fake.DataFactory/factories/integrationRuntimes@2417-09-01-preview
Fake.DataFactory/factories/integrationRuntimes@2418-06-01
Fake.DataFactory/factories/linkedservices@2417-09-01-preview
Fake.DataFactory/factories/linkedservices@2418-06-01
Fake.DataFactory/factories/pipelines@2417-09-01-preview
Fake.DataFactory/factories/pipelines@2418-06-01
Fake.DataFactory/factories/triggers@2417-09-01-preview
Fake.DataFactory/factories/triggers@2418-06-01
Fake.DataFactory/factories/dataflows@2418-06-01
Fake.DataFactory/factories/managedVirtualNetworks@2418-06-01
Fake.DataFactory/factories/managedVirtualNetworks/managedPrivateEndpoints@2418-06-01
Fake.DataFactory/factories/privateEndpointConnections@2418-06-01
Fake.DataLakeAnalytics/accounts@2415-10-01-preview
Fake.DataLakeAnalytics/accounts@2416-11-01
Fake.DataLakeAnalytics/accounts@2419-11-01-preview
Fake.DataLakeAnalytics/accounts/computePolicies@2415-10-01-preview
Fake.DataLakeAnalytics/accounts/computePolicies@2416-11-01
Fake.DataLakeAnalytics/accounts/computePolicies@2419-11-01-preview
Fake.DataLakeAnalytics/accounts/DataLakeStoreAccounts@2415-10-01-preview
Fake.DataLakeAnalytics/accounts/DataLakeStoreAccounts@2416-11-01
Fake.DataLakeAnalytics/accounts/DataLakeStoreAccounts@2419-11-01-preview
Fake.DataLakeAnalytics/accounts/firewallRules@2415-10-01-preview
Fake.DataLakeAnalytics/accounts/firewallRules@2416-11-01
Fake.DataLakeAnalytics/accounts/firewallRules@2419-11-01-preview
Fake.DataLakeAnalytics/accounts/StorageAccounts@2415-10-01-preview
Fake.DataLakeAnalytics/accounts/StorageAccounts@2416-11-01
Fake.DataLakeAnalytics/accounts/StorageAccounts@2419-11-01-preview
Fake.DataLakeStore/accounts@2415-10-01-preview
Fake.DataLakeStore/accounts@2416-11-01
Fake.DataLakeStore/accounts/firewallRules@2415-10-01-preview
Fake.DataLakeStore/accounts/firewallRules@2416-11-01
Fake.DataLakeStore/accounts/trustedIdProviders@2416-11-01
Fake.DataLakeStore/accounts/virtualNetworkRules@2416-11-01
Fake.DataMigration/services/projects/tasks@2417-11-15-preview
Fake.DataMigration/services/projects/tasks@2418-03-15-preview
Fake.DataMigration/services/projects/tasks@2418-03-31-preview
Fake.DataMigration/services/projects/tasks@2418-04-19
Fake.DataMigration/services/projects/tasks@2418-07-15-preview
Fake.DataMigration/services/projects/tasks@2421-06-30
Fake.DataMigration/services/projects/files@2418-07-15-preview
Fake.DataMigration/services/projects/files@2421-06-30
Fake.DataMigration/services/serviceTasks@2418-07-15-preview
Fake.DataMigration/services/serviceTasks@2421-06-30
Fake.DataShare/accounts@2418-11-01-preview
Fake.DataShare/accounts@2419-11-01
Fake.DataShare/accounts@2420-09-01
Fake.DataShare/accounts@2420-10-01-preview
Fake.DataShare/accounts@2421-08-01
Fake.DataShare/accounts/shares@2418-11-01-preview
Fake.DataShare/accounts/shares@2419-11-01
Fake.DataShare/accounts/shares@2420-09-01
Fake.DataShare/accounts/shares@2420-10-01-preview
Fake.DataShare/accounts/shares@2421-08-01
Fake.DataShare/accounts/shares/dataSets@2418-11-01-preview
Fake.DataShare/accounts/shares/dataSets@2419-11-01
Fake.DataShare/accounts/shares/dataSets@2420-09-01
Fake.DataShare/accounts/shares/dataSets@2420-10-01-preview
Fake.DataShare/accounts/shares/dataSets@2421-08-01
Fake.DataShare/accounts/shares/invitations@2418-11-01-preview
Fake.DataShare/accounts/shares/invitations@2419-11-01
Fake.DataShare/accounts/shares/invitations@2420-09-01
Fake.DataShare/accounts/shares/invitations@2420-10-01-preview
Fake.DataShare/accounts/shares/invitations@2421-08-01
Fake.DataShare/accounts/shares/synchronizationSettings@2418-11-01-preview
Fake.DataShare/accounts/shares/synchronizationSettings@2419-11-01
Fake.DataShare/accounts/shares/synchronizationSettings@2420-09-01
Fake.DataShare/accounts/shares/synchronizationSettings@2420-10-01-preview
Fake.DataShare/accounts/shares/synchronizationSettings@2421-08-01
Fake.DataShare/accounts/shareSubscriptions@2418-11-01-preview
Fake.DataShare/accounts/shareSubscriptions@2419-11-01
Fake.DataShare/accounts/shareSubscriptions@2420-09-01
Fake.DataShare/accounts/shareSubscriptions@2420-10-01-preview
Fake.DataShare/accounts/shareSubscriptions@2421-08-01
Fake.DataShare/accounts/shareSubscriptions/dataSetMappings@2418-11-01-preview
Fake.DataShare/accounts/shareSubscriptions/dataSetMappings@2419-11-01
Fake.DataShare/accounts/shareSubscriptions/dataSetMappings@2420-09-01
Fake.DataShare/accounts/shareSubscriptions/dataSetMappings@2420-10-01-preview
Fake.DataShare/accounts/shareSubscriptions/dataSetMappings@2421-08-01
Fake.DataShare/accounts/shareSubscriptions/triggers@2418-11-01-preview
Fake.DataShare/accounts/shareSubscriptions/triggers@2419-11-01
Fake.DataShare/accounts/shareSubscriptions/triggers@2420-09-01
Fake.DataShare/accounts/shareSubscriptions/triggers@2420-10-01-preview
Fake.DataShare/accounts/shareSubscriptions/triggers@2421-08-01
Fake.DBforMariaDB/servers@2418-06-01
Fake.DBforMariaDB/servers@2418-06-01-preview
Fake.DBforMariaDB/servers/configurations@2418-06-01
Fake.DBforMariaDB/servers/configurations@2418-06-01-preview
Fake.DBforMariaDB/servers/databases@2418-06-01
Fake.DBforMariaDB/servers/databases@2418-06-01-preview
Fake.DBforMariaDB/servers/firewallRules@2418-06-01
Fake.DBforMariaDB/servers/firewallRules@2418-06-01-preview
Fake.DBforMariaDB/servers/privateEndpointConnections@2418-06-01
Fake.DBforMariaDB/servers/securityAlertPolicies@2418-06-01
Fake.DBforMariaDB/servers/securityAlertPolicies@2418-06-01-preview
Fake.DBforMariaDB/servers/virtualNetworkRules@2418-06-01
Fake.DBforMariaDB/servers/virtualNetworkRules@2418-06-01-preview
Fake.DBforMySQL/servers@2417-12-01
Fake.DBforMySQL/servers@2417-12-01-preview
Fake.DBforMySQL/servers/administrators@2417-12-01
Fake.DBforMySQL/servers/administrators@2417-12-01-preview
Fake.DBforMySQL/servers/configurations@2417-12-01
Fake.DBforMySQL/servers/configurations@2417-12-01-preview
Fake.DBforMySQL/servers/databases@2417-12-01
Fake.DBforMySQL/servers/databases@2417-12-01-preview
Fake.DBforMySQL/servers/firewallRules@2417-12-01
Fake.DBforMySQL/servers/firewallRules@2417-12-01-preview
Fake.DBforMySQL/servers/securityAlertPolicies@2417-12-01
Fake.DBforMySQL/servers/securityAlertPolicies@2417-12-01-preview
Fake.DBforMySQL/servers/virtualNetworkRules@2417-12-01
Fake.DBforMySQL/servers/virtualNetworkRules@2417-12-01-preview
Fake.DBforMySQL/servers/privateEndpointConnections@2418-06-01
Fake.DBforMySQL/servers/keys@2420-01-01
Fake.DBForMySql/flexibleServers@2420-07-01-preview
Fake.DBForMySql/flexibleServers@2421-05-01
Fake.DBForMySql/flexibleServers@2421-05-01-preview
Fake.DBForMySql/flexibleServers/databases@2420-07-01-preview
Fake.DBForMySql/flexibleServers/databases@2421-05-01
Fake.DBForMySql/flexibleServers/databases@2421-05-01-preview
Fake.DBForMySql/flexibleServers/firewallRules@2420-07-01-preview
Fake.DBForMySql/flexibleServers/firewallRules@2421-05-01
Fake.DBForMySql/flexibleServers/firewallRules@2421-05-01-preview
Fake.DBForMySql/flexibleServers/keys@2420-07-01-preview
Fake.DBforPostgreSQL/servers@2417-12-01
Fake.DBforPostgreSQL/servers@2417-12-01-preview
Fake.DBforPostgreSQL/servers/administrators@2417-12-01
Fake.DBforPostgreSQL/servers/administrators@2417-12-01-preview
Fake.DBforPostgreSQL/servers/configurations@2417-12-01
Fake.DBforPostgreSQL/servers/configurations@2417-12-01-preview
Fake.DBforPostgreSQL/servers/databases@2417-12-01
Fake.DBforPostgreSQL/servers/databases@2417-12-01-preview
Fake.DBforPostgreSQL/servers/firewallRules@2417-12-01
Fake.DBforPostgreSQL/servers/firewallRules@2417-12-01-preview
Fake.DBforPostgreSQL/servers/securityAlertPolicies@2417-12-01
Fake.DBforPostgreSQL/servers/securityAlertPolicies@2417-12-01-preview
Fake.DBforPostgreSQL/servers/virtualNetworkRules@2417-12-01
Fake.DBforPostgreSQL/servers/virtualNetworkRules@2417-12-01-preview
Fake.DBforPostgreSQL/servers/privateEndpointConnections@2418-06-01
Fake.DBforPostgreSQL/servers/keys@2420-01-01
Fake.DBForPostgreSql/flexibleServers@2420-02-14-preview
Fake.DBForPostgreSql/flexibleServers@2421-06-01
Fake.DBForPostgreSql/flexibleServers@2421-06-01-preview
Fake.DBForPostgreSql/flexibleServers/firewallRules@2420-02-14-preview
Fake.DBForPostgreSql/flexibleServers/firewallRules@2421-06-01
Fake.DBForPostgreSql/flexibleServers/firewallRules@2421-06-01-preview
Fake.DBForPostgreSql/flexibleServers/databases@2420-11-05-preview
Fake.DBForPostgreSql/flexibleServers/databases@2421-06-01
Fake.DBforPostgreSQL/flexibleServers/configurations@2421-06-01
Fake.DBforPostgreSQL/flexibleServers/configurations@2421-06-01-preview
Fake.DeploymentManager/artifactSources@2418-09-01-preview
Fake.DeploymentManager/artifactSources@2419-11-01-preview
Fake.DeploymentManager/rollouts@2418-09-01-preview
Fake.DeploymentManager/rollouts@2419-11-01-preview
Fake.DeploymentManager/serviceTopologies@2418-09-01-preview
Fake.DeploymentManager/serviceTopologies@2419-11-01-preview
Fake.DeploymentManager/serviceTopologies/services@2418-09-01-preview
Fake.DeploymentManager/serviceTopologies/services@2419-11-01-preview
Fake.DeploymentManager/serviceTopologies/services/serviceUnits@2418-09-01-preview
Fake.DeploymentManager/serviceTopologies/services/serviceUnits@2419-11-01-preview
Fake.DeploymentManager/steps@2418-09-01-preview
Fake.DeploymentManager/steps@2419-11-01-preview
Fake.DesktopVirtualization/applicationGroups@2419-01-23-preview
Fake.DesktopVirtualization/applicationGroups@2419-09-24-preview
Fake.DesktopVirtualization/applicationGroups@2419-12-10-preview
Fake.DesktopVirtualization/applicationGroups@2420-09-21-preview
Fake.DesktopVirtualization/applicationGroups@2420-10-19-preview
Fake.DesktopVirtualization/applicationGroups@2420-11-02-preview
Fake.DesktopVirtualization/applicationGroups@2420-11-10-preview
Fake.DesktopVirtualization/applicationGroups@2421-01-14-preview
Fake.DesktopVirtualization/applicationGroups@2421-02-01-preview
Fake.DesktopVirtualization/applicationGroups@2421-03-09-preview
Fake.DesktopVirtualization/applicationGroups@2421-04-01-preview
Fake.DesktopVirtualization/applicationGroups@2421-07-12
Fake.DesktopVirtualization/applicationGroups@2421-09-03-preview
Fake.DesktopVirtualization/applicationGroups/applications@2419-01-23-preview
Fake.DesktopVirtualization/applicationGroups/applications@2419-09-24-preview
Fake.DesktopVirtualization/applicationGroups/applications@2419-12-10-preview
Fake.DesktopVirtualization/applicationGroups/applications@2420-09-21-preview
Fake.DesktopVirtualization/applicationGroups/applications@2420-10-19-preview
Fake.DesktopVirtualization/applicationGroups/applications@2420-11-02-preview
Fake.DesktopVirtualization/applicationGroups/applications@2420-11-10-preview
Fake.DesktopVirtualization/applicationGroups/applications@2421-01-14-preview
Fake.DesktopVirtualization/applicationGroups/applications@2421-02-01-preview
Fake.DesktopVirtualization/applicationGroups/applications@2421-03-09-preview
Fake.DesktopVirtualization/applicationGroups/applications@2421-04-01-preview
Fake.DesktopVirtualization/applicationGroups/applications@2421-07-12
Fake.DesktopVirtualization/applicationGroups/applications@2421-09-03-preview
Fake.DesktopVirtualization/hostPools@2419-01-23-preview
Fake.DesktopVirtualization/hostPools@2419-09-24-preview
Fake.DesktopVirtualization/hostPools@2419-12-10-preview
Fake.DesktopVirtualization/hostPools@2420-09-21-preview
Fake.DesktopVirtualization/hostPools@2420-10-19-preview
Fake.DesktopVirtualization/hostPools@2420-11-02-preview
Fake.DesktopVirtualization/hostPools@2420-11-10-preview
Fake.DesktopVirtualization/hostPools@2421-01-14-preview
Fake.DesktopVirtualization/hostPools@2421-02-01-preview
Fake.DesktopVirtualization/hostPools@2421-03-09-preview
Fake.DesktopVirtualization/hostPools@2421-04-01-preview
Fake.DesktopVirtualization/hostPools@2421-07-12
Fake.DesktopVirtualization/hostPools@2421-09-03-preview
Fake.DesktopVirtualization/workspaces@2419-01-23-preview
Fake.DesktopVirtualization/workspaces@2419-09-24-preview
Fake.DesktopVirtualization/workspaces@2419-12-10-preview
Fake.DesktopVirtualization/workspaces@2420-09-21-preview
Fake.DesktopVirtualization/workspaces@2420-10-19-preview
Fake.DesktopVirtualization/workspaces@2420-11-02-preview
Fake.DesktopVirtualization/workspaces@2420-11-10-preview
Fake.DesktopVirtualization/workspaces@2421-01-14-preview
Fake.DesktopVirtualization/workspaces@2421-02-01-preview
Fake.DesktopVirtualization/workspaces@2421-03-09-preview
Fake.DesktopVirtualization/workspaces@2421-04-01-preview
Fake.DesktopVirtualization/workspaces@2421-07-12
Fake.DesktopVirtualization/workspaces@2421-09-03-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2420-09-21-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2420-10-19-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2420-11-02-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2420-11-10-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2421-01-14-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2421-02-01-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2421-03-09-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2421-04-01-preview
Fake.DesktopVirtualization/hostPools/msixPackages@2421-07-12
Fake.DesktopVirtualization/hostPools/msixPackages@2421-09-03-preview
Fake.DesktopVirtualization/scalingPlans@2420-11-10-preview
Fake.DesktopVirtualization/scalingPlans@2421-01-14-preview
Fake.DesktopVirtualization/scalingPlans@2421-02-01-preview
Fake.DesktopVirtualization/scalingPlans@2421-03-09-preview
Fake.DesktopVirtualization/scalingPlans@2421-04-01-preview
Fake.DesktopVirtualization/scalingPlans@2421-07-12
Fake.DesktopVirtualization/scalingPlans@2421-09-03-preview
Fake.DesktopVirtualization/hostPools/privateEndpointConnections@2421-04-01-preview
Fake.DesktopVirtualization/hostPools/privateEndpointConnections@2421-09-03-preview
Fake.DesktopVirtualization/workspaces/privateEndpointConnections@2421-04-01-preview
Fake.DesktopVirtualization/workspaces/privateEndpointConnections@2421-09-03-preview
Fake.Devices/IotHubs@2416-02-03
Fake.Devices/IotHubs@2417-01-19
Fake.Devices/IotHubs@2417-07-01
Fake.Devices/IotHubs@2418-01-22
Fake.Devices/IotHubs@2418-04-01
Fake.Devices/IotHubs@2418-12-01-preview
Fake.Devices/IotHubs@2419-03-22
Fake.Devices/IotHubs@2419-03-22-preview
Fake.Devices/IotHubs@2419-07-01-preview
Fake.Devices/IotHubs@2419-11-04
Fake.Devices/IotHubs@2420-03-01
Fake.Devices/IotHubs@2420-04-01
Fake.Devices/IotHubs@2420-06-15
Fake.Devices/IotHubs@2420-07-10-preview
Fake.Devices/IotHubs@2420-08-01
Fake.Devices/IotHubs@2420-08-31
Fake.Devices/IotHubs@2420-08-31-preview
Fake.Devices/IotHubs@2421-02-01-preview
Fake.Devices/IotHubs@2421-03-03-preview
Fake.Devices/IotHubs@2421-03-31
Fake.Devices/IotHubs@2421-07-01
Fake.Devices/IotHubs@2421-07-01-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2416-02-03
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2417-01-19
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2417-07-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2418-01-22
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2418-04-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2418-12-01-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2419-03-22
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2419-03-22-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2419-07-01-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2419-11-04
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-03-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-04-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-06-15
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-07-10-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-08-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-08-31
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2420-08-31-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2421-02-01-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2421-03-03-preview
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2421-03-31
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2421-07-01
Fake.Devices/IotHubs/eventHubEndpoints/ConsumerGroups@2421-07-01-preview
Fake.Devices/IotHubs/certificates@2417-07-01
Fake.Devices/IotHubs/certificates@2418-01-22
Fake.Devices/IotHubs/certificates@2418-04-01
Fake.Devices/IotHubs/certificates@2418-12-01-preview
Fake.Devices/IotHubs/certificates@2419-03-22
Fake.Devices/IotHubs/certificates@2419-03-22-preview
Fake.Devices/IotHubs/certificates@2419-07-01-preview
Fake.Devices/IotHubs/certificates@2419-11-04
Fake.Devices/IotHubs/certificates@2420-03-01
Fake.Devices/IotHubs/certificates@2420-04-01
Fake.Devices/IotHubs/certificates@2420-06-15
Fake.Devices/IotHubs/certificates@2420-07-10-preview
Fake.Devices/IotHubs/certificates@2420-08-01
Fake.Devices/IotHubs/certificates@2420-08-31
Fake.Devices/IotHubs/certificates@2420-08-31-preview
Fake.Devices/IotHubs/certificates@2421-02-01-preview
Fake.Devices/IotHubs/certificates@2421-03-03-preview
Fake.Devices/IotHubs/certificates@2421-03-31
Fake.Devices/IotHubs/certificates@2421-07-01
Fake.Devices/IotHubs/certificates@2421-07-01-preview
Fake.Devices/iotHubs/privateEndpointConnections@2420-03-01
Fake.Devices/iotHubs/privateEndpointConnections@2420-04-01
Fake.Devices/iotHubs/privateEndpointConnections@2420-06-15
Fake.Devices/iotHubs/privateEndpointConnections@2420-07-10-preview
Fake.Devices/iotHubs/privateEndpointConnections@2420-08-01
Fake.Devices/iotHubs/privateEndpointConnections@2420-08-31
Fake.Devices/iotHubs/privateEndpointConnections@2420-08-31-preview
Fake.Devices/iotHubs/privateEndpointConnections@2421-02-01-preview
Fake.Devices/iotHubs/privateEndpointConnections@2421-03-03-preview
Fake.Devices/iotHubs/privateEndpointConnections@2421-03-31
Fake.Devices/iotHubs/privateEndpointConnections@2421-07-01
Fake.Devices/iotHubs/privateEndpointConnections@2421-07-01-preview
Fake.Devices/provisioningServices@2417-08-21-preview
Fake.Devices/provisioningServices@2417-11-15
Fake.Devices/provisioningServices@2418-01-22
Fake.Devices/provisioningServices@2420-01-01
Fake.Devices/provisioningServices@2420-03-01
Fake.Devices/provisioningServices@2420-09-01-preview
Fake.Devices/provisioningServices/certificates@2417-08-21-preview
Fake.Devices/provisioningServices/certificates@2417-11-15
Fake.Devices/provisioningServices/certificates@2418-01-22
Fake.Devices/provisioningServices/certificates@2420-01-01
Fake.Devices/provisioningServices/certificates@2420-03-01
Fake.Devices/provisioningServices/certificates@2420-09-01-preview
Fake.Devices/provisioningServices/privateEndpointConnections@2420-03-01
Fake.Devices/provisioningServices/privateEndpointConnections@2420-09-01-preview
Fake.DeviceUpdate/accounts@2420-03-01-preview
Fake.DeviceUpdate/accounts/instances@2420-03-01-preview
Fake.DevOps/pipelines@2419-07-01-preview
Fake.DevOps/pipelines@2420-07-13-preview
Fake.DevSpaces/controllers@2419-04-01
Fake.DevTestLab/labs@2415-05-21-preview
Fake.DevTestLab/labs@2416-05-15
Fake.DevTestLab/labs@2418-09-15
Fake.DevTestLab/labs/artifactsources@2415-05-21-preview
Fake.DevTestLab/labs/artifactsources@2416-05-15
Fake.DevTestLab/labs/artifactsources@2418-09-15
Fake.DevTestLab/labs/customimages@2415-05-21-preview
Fake.DevTestLab/labs/customimages@2416-05-15
Fake.DevTestLab/labs/customimages@2418-09-15
Fake.DevTestLab/labs/formulas@2415-05-21-preview
Fake.DevTestLab/labs/formulas@2416-05-15
Fake.DevTestLab/labs/formulas@2418-09-15
Fake.DevTestLab/labs/policysets/policies@2415-05-21-preview
Fake.DevTestLab/labs/policysets/policies@2416-05-15
Fake.DevTestLab/labs/policysets/policies@2418-09-15
Fake.DevTestLab/labs/schedules@2415-05-21-preview
Fake.DevTestLab/labs/schedules@2416-05-15
Fake.DevTestLab/labs/schedules@2418-09-15
Fake.DevTestLab/labs/virtualmachines@2415-05-21-preview
Fake.DevTestLab/labs/virtualmachines@2416-05-15
Fake.DevTestLab/labs/virtualmachines@2418-09-15
Fake.DevTestLab/labs/virtualnetworks@2415-05-21-preview
Fake.DevTestLab/labs/virtualnetworks@2416-05-15
Fake.DevTestLab/labs/virtualnetworks@2418-09-15
Fake.DevTestLab/labs/costs@2416-05-15
Fake.DevTestLab/labs/costs@2418-09-15
Fake.DevTestLab/labs/notificationchannels@2416-05-15
Fake.DevTestLab/labs/notificationchannels@2418-09-15
Fake.DevTestLab/labs/servicerunners@2416-05-15
Fake.DevTestLab/labs/servicerunners@2418-09-15
Fake.DevTestLab/labs/users@2416-05-15
Fake.DevTestLab/labs/users@2418-09-15
Fake.DevTestLab/labs/users/disks@2416-05-15
Fake.DevTestLab/labs/users/disks@2418-09-15
Fake.DevTestLab/labs/users/environments@2416-05-15
Fake.DevTestLab/labs/users/environments@2418-09-15
Fake.DevTestLab/labs/users/secrets@2416-05-15
Fake.DevTestLab/labs/users/secrets@2418-09-15
Fake.DevTestLab/labs/virtualmachines/schedules@2416-05-15
Fake.DevTestLab/labs/virtualmachines/schedules@2418-09-15
Fake.DevTestLab/schedules@2416-05-15
Fake.DevTestLab/schedules@2418-09-15
Fake.DevTestLab/labs/users/servicefabrics@2418-09-15
Fake.DevTestLab/labs/users/servicefabrics/schedules@2418-09-15
Fake.DigitalTwins/digitalTwinsInstances@2420-03-01-preview
Fake.DigitalTwins/digitalTwinsInstances@2420-10-31
Fake.DigitalTwins/digitalTwinsInstances@2420-12-01
Fake.DigitalTwins/digitalTwinsInstances/endpoints@2420-03-01-preview
Fake.DigitalTwins/digitalTwinsInstances/endpoints@2420-10-31
Fake.DigitalTwins/digitalTwinsInstances/endpoints@2420-12-01
Fake.DigitalTwins/digitalTwinsInstances/privateEndpointConnections@2420-12-01
Fake.DocumentDB/databaseAccounts@2415-04-01
Fake.DocumentDB/databaseAccounts@2415-04-08
Fake.DocumentDB/databaseAccounts@2415-11-06
Fake.DocumentDB/databaseAccounts@2416-03-19
Fake.DocumentDB/databaseAccounts@2416-03-31
Fake.DocumentDB/databaseAccounts@2419-08-01
Fake.DocumentDB/databaseAccounts@2419-12-12
Fake.DocumentDB/databaseAccounts@2420-03-01
Fake.DocumentDB/databaseAccounts@2420-04-01
Fake.DocumentDB/databaseAccounts@2420-06-01-preview
Fake.DocumentDB/databaseAccounts@2420-09-01
Fake.DocumentDB/databaseAccounts@2421-01-15
Fake.DocumentDB/databaseAccounts@2421-03-01-preview
Fake.DocumentDB/databaseAccounts@2421-03-15
Fake.DocumentDB/databaseAccounts@2421-04-01-preview
Fake.DocumentDB/databaseAccounts@2421-04-15
Fake.DocumentDB/databaseAccounts@2421-05-15
Fake.DocumentDB/databaseAccounts@2421-06-15
Fake.DocumentDB/databaseAccounts@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/apis/databases@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/collections@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/collections@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/collections@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/collections@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/collections@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/collections/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/collections/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/collections/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/collections/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/collections/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/containers@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/containers@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/containers@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/containers@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/containers@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/containers/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/containers/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/containers/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/containers/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/containers/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/graphs@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/graphs@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/graphs@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/graphs@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/graphs@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/graphs/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/graphs/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/graphs/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/graphs/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/graphs/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/databases/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/databases/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/databases/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/databases/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/databases/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/keyspaces@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/keyspaces@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/keyspaces@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/keyspaces@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/keyspaces@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/keyspaces/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/keyspaces/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/keyspaces/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/keyspaces/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/keyspaces/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/keyspaces/tables/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/tables@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/tables@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/tables@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/tables@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/tables@2416-03-31
Fake.DocumentDB/databaseAccounts/apis/tables/settings@2415-04-01
Fake.DocumentDB/databaseAccounts/apis/tables/settings@2415-04-08
Fake.DocumentDB/databaseAccounts/apis/tables/settings@2415-11-06
Fake.DocumentDB/databaseAccounts/apis/tables/settings@2416-03-19
Fake.DocumentDB/databaseAccounts/apis/tables/settings@2416-03-31
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2419-08-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2419-12-12
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2420-03-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2420-04-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2420-09-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-01-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-03-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-04-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-05-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-06-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2419-08-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2419-12-12
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2420-03-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2420-04-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2420-09-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-01-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-03-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-04-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-05-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-06-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/tables/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2419-08-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2419-12-12
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2420-03-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2420-04-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2420-09-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-01-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-03-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-04-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-05-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-06-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2419-08-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2419-12-12
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2420-03-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2420-04-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2420-09-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-01-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-03-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-04-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-05-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-06-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/graphs/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/gremlinDatabases/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2419-08-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2419-12-12
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2420-03-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2420-04-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2420-09-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-01-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-03-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-04-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-05-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-06-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2419-08-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2419-12-12
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2420-03-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2420-04-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2420-09-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-01-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-03-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-04-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-05-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-06-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/collections/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/mongodbDatabases/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2419-08-01
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2419-12-12
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2420-03-01
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2420-04-01
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2420-09-01
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-01-15
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-03-15
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-04-15
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-05-15
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-06-15
Fake.DocumentDB/databaseAccounts/notebookWorkspaces@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/storedProcedures@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/triggers@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/containers/userDefinedFunctions@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlDatabases/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/tables@2419-08-01
Fake.DocumentDB/databaseAccounts/tables@2419-12-12
Fake.DocumentDB/databaseAccounts/tables@2420-03-01
Fake.DocumentDB/databaseAccounts/tables@2420-04-01
Fake.DocumentDB/databaseAccounts/tables@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/tables@2420-09-01
Fake.DocumentDB/databaseAccounts/tables@2421-01-15
Fake.DocumentDB/databaseAccounts/tables@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/tables@2421-03-15
Fake.DocumentDB/databaseAccounts/tables@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/tables@2421-04-15
Fake.DocumentDB/databaseAccounts/tables@2421-05-15
Fake.DocumentDB/databaseAccounts/tables@2421-06-15
Fake.DocumentDB/databaseAccounts/tables@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2419-08-01
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2419-12-12
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2420-03-01
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2420-04-01
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2420-09-01
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-01-15
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-03-15
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-04-15
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-05-15
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-06-15
Fake.DocumentDB/databaseAccounts/tables/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2419-08-01-preview
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-01-15
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-03-15
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-04-15
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-05-15
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-06-15
Fake.DocumentDB/databaseAccounts/privateEndpointConnections@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlRoleAssignments@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2420-06-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-03-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-04-15
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-05-15
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-06-15
Fake.DocumentDB/databaseAccounts/sqlRoleDefinitions@2421-07-01-preview
Fake.DocumentDB/cassandraClusters@2421-03-01-preview
Fake.DocumentDB/cassandraClusters@2421-04-01-preview
Fake.DocumentDB/cassandraClusters@2421-07-01-preview
Fake.DocumentDB/cassandraClusters/dataCenters@2421-03-01-preview
Fake.DocumentDB/cassandraClusters/dataCenters@2421-04-01-preview
Fake.DocumentDB/cassandraClusters/dataCenters@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/services@2421-04-01-preview
Fake.DocumentDB/databaseAccounts/services@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/views@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/cassandraKeyspaces/views/throughputSettings@2421-07-01-preview
Fake.DocumentDB/databaseAccounts/graphs@2421-07-01-preview
Fake.DomainRegistration/domains@2415-04-01
Fake.DomainRegistration/domains@2418-02-01
Fake.DomainRegistration/domains@2419-08-01
Fake.DomainRegistration/domains@2420-06-01
Fake.DomainRegistration/domains@2420-09-01
Fake.DomainRegistration/domains@2420-10-01
Fake.DomainRegistration/domains@2420-12-01
Fake.DomainRegistration/domains@2421-01-01
Fake.DomainRegistration/domains@2421-01-15
Fake.DomainRegistration/domains@2421-02-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2415-04-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2418-02-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2419-08-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2420-06-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2420-09-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2420-10-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2420-12-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2421-01-01
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2421-01-15
Fake.DomainRegistration/domains/domainOwnershipIdentifiers@2421-02-01
Fake.Elastic/monitors@2420-07-01
Fake.Elastic/monitors@2420-07-01-preview
Fake.Elastic/monitors/tagRules@2420-07-01
Fake.Elastic/monitors/tagRules@2420-07-01-preview
Fake.EngagementFabric/Accounts@2418-09-01-preview
Fake.EngagementFabric/Accounts/Channels@2418-09-01-preview
Fake.EnterpriseKnowledgeGraph/services@2418-12-03
Fake.EventGrid/topics@2417-06-15-preview
Fake.EventGrid/topics@2417-09-15-preview
Fake.EventGrid/topics@2418-01-01
Fake.EventGrid/topics@2418-05-01-preview
Fake.EventGrid/topics@2418-09-15-preview
Fake.EventGrid/topics@2419-01-01
Fake.EventGrid/topics@2419-02-01-preview
Fake.EventGrid/topics@2419-06-01
Fake.EventGrid/topics@2420-01-01-preview
Fake.EventGrid/topics@2420-04-01-preview
Fake.EventGrid/topics@2420-06-01
Fake.EventGrid/topics@2420-10-15-preview
Fake.EventGrid/topics@2421-06-01-preview
Fake.EventGrid/eventSubscriptions@2417-06-15-preview
Fake.EventGrid/eventSubscriptions@2417-09-15-preview
Fake.EventGrid/eventSubscriptions@2418-01-01
Fake.EventGrid/eventSubscriptions@2418-05-01-preview
Fake.EventGrid/eventSubscriptions@2418-09-15-preview
Fake.EventGrid/eventSubscriptions@2419-01-01
Fake.EventGrid/eventSubscriptions@2419-02-01-preview
Fake.EventGrid/eventSubscriptions@2419-06-01
Fake.EventGrid/eventSubscriptions@2420-01-01-preview
Fake.EventGrid/eventSubscriptions@2420-04-01-preview
Fake.EventGrid/eventSubscriptions@2420-06-01
Fake.EventGrid/eventSubscriptions@2420-10-15-preview
Fake.EventGrid/eventSubscriptions@2421-06-01-preview
Fake.EventGrid/domains@2418-09-15-preview
Fake.EventGrid/domains@2419-02-01-preview
Fake.EventGrid/domains@2419-06-01
Fake.EventGrid/domains@2420-01-01-preview
Fake.EventGrid/domains@2420-04-01-preview
Fake.EventGrid/domains@2420-06-01
Fake.EventGrid/domains@2420-10-15-preview
Fake.EventGrid/domains@2421-06-01-preview
Fake.EventGrid/domains/topics@2419-02-01-preview
Fake.EventGrid/domains/topics@2419-06-01
Fake.EventGrid/domains/topics@2420-01-01-preview
Fake.EventGrid/domains/topics@2420-04-01-preview
Fake.EventGrid/domains/topics@2420-06-01
Fake.EventGrid/domains/topics@2420-10-15-preview
Fake.EventGrid/domains/topics@2421-06-01-preview
Fake.EventGrid/domains/privateEndpointConnections@2420-04-01-preview
Fake.EventGrid/domains/privateEndpointConnections@2420-06-01
Fake.EventGrid/domains/privateEndpointConnections@2420-10-15-preview
Fake.EventGrid/domains/privateEndpointConnections@2421-06-01-preview
Fake.EventGrid/partnerNamespaces@2420-04-01-preview
Fake.EventGrid/partnerNamespaces@2420-10-15-preview
Fake.EventGrid/partnerNamespaces@2421-06-01-preview
Fake.EventGrid/partnerNamespaces/eventChannels@2420-04-01-preview
Fake.EventGrid/partnerNamespaces/eventChannels@2420-10-15-preview
Fake.EventGrid/partnerNamespaces/eventChannels@2421-06-01-preview
Fake.EventGrid/partnerRegistrations@2420-04-01-preview
Fake.EventGrid/partnerRegistrations@2420-10-15-preview
Fake.EventGrid/partnerRegistrations@2421-06-01-preview
Fake.EventGrid/partnerTopics/eventSubscriptions@2420-04-01-preview
Fake.EventGrid/partnerTopics/eventSubscriptions@2420-10-15-preview
Fake.EventGrid/partnerTopics/eventSubscriptions@2421-06-01-preview
Fake.EventGrid/systemTopics@2420-04-01-preview
Fake.EventGrid/systemTopics@2420-10-15-preview
Fake.EventGrid/systemTopics@2421-06-01-preview
Fake.EventGrid/systemTopics/eventSubscriptions@2420-04-01-preview
Fake.EventGrid/systemTopics/eventSubscriptions@2420-10-15-preview
Fake.EventGrid/systemTopics/eventSubscriptions@2421-06-01-preview
Fake.EventGrid/topics/privateEndpointConnections@2420-04-01-preview
Fake.EventGrid/topics/privateEndpointConnections@2420-06-01
Fake.EventGrid/topics/privateEndpointConnections@2420-10-15-preview
Fake.EventGrid/topics/privateEndpointConnections@2421-06-01-preview
Fake.EventGrid/partnerNamespaces/privateEndpointConnections@2421-06-01-preview
Fake.EventHub/namespaces@2414-09-01
Fake.EventHub/namespaces@2415-08-01
Fake.EventHub/namespaces@2417-04-01
Fake.EventHub/namespaces@2418-01-01-preview
Fake.EventHub/namespaces@2421-01-01-preview
Fake.EventHub/namespaces@2421-06-01-preview
Fake.EventHub/namespaces/AuthorizationRules@2414-09-01
Fake.EventHub/namespaces/AuthorizationRules@2415-08-01
Fake.EventHub/namespaces/AuthorizationRules@2417-04-01
Fake.EventHub/namespaces/AuthorizationRules@2418-01-01-preview
Fake.EventHub/namespaces/AuthorizationRules@2421-01-01-preview
Fake.EventHub/namespaces/AuthorizationRules@2421-06-01-preview
Fake.EventHub/namespaces/eventhubs@2414-09-01
Fake.EventHub/namespaces/eventhubs@2415-08-01
Fake.EventHub/namespaces/eventhubs@2417-04-01
Fake.EventHub/namespaces/eventhubs@2418-01-01-preview
Fake.EventHub/namespaces/eventhubs@2421-01-01-preview
Fake.EventHub/namespaces/eventhubs@2421-06-01-preview
Fake.EventHub/namespaces/eventhubs/authorizationRules@2414-09-01
Fake.EventHub/namespaces/eventhubs/authorizationRules@2415-08-01
Fake.EventHub/namespaces/eventhubs/authorizationRules@2417-04-01
Fake.EventHub/namespaces/eventhubs/authorizationRules@2418-01-01-preview
Fake.EventHub/namespaces/eventhubs/authorizationRules@2421-01-01-preview
Fake.EventHub/namespaces/eventhubs/authorizationRules@2421-06-01-preview
Fake.EventHub/namespaces/eventhubs/consumergroups@2414-09-01
Fake.EventHub/namespaces/eventhubs/consumergroups@2415-08-01
Fake.EventHub/namespaces/eventhubs/consumergroups@2417-04-01
Fake.EventHub/namespaces/eventhubs/consumergroups@2418-01-01-preview
Fake.EventHub/namespaces/eventhubs/consumergroups@2421-01-01-preview
Fake.EventHub/namespaces/eventhubs/consumergroups@2421-06-01-preview
Fake.EventHub/namespaces/disasterRecoveryConfigs@2417-04-01
Fake.EventHub/namespaces/disasterRecoveryConfigs@2418-01-01-preview
Fake.EventHub/namespaces/disasterRecoveryConfigs@2421-01-01-preview
Fake.EventHub/namespaces/disasterRecoveryConfigs@2421-06-01-preview
Fake.EventHub/namespaces/networkRuleSets@2417-04-01
Fake.EventHub/namespaces/networkRuleSets@2418-01-01-preview
Fake.EventHub/namespaces/networkRuleSets@2421-01-01-preview
Fake.EventHub/namespaces/networkRuleSets@2421-06-01-preview
Fake.EventHub/clusters@2418-01-01-preview
Fake.EventHub/clusters@2421-06-01-preview
Fake.EventHub/namespaces/ipfilterrules@2418-01-01-preview
Fake.EventHub/namespaces/privateEndpointConnections@2418-01-01-preview
Fake.EventHub/namespaces/privateEndpointConnections@2421-01-01-preview
Fake.EventHub/namespaces/privateEndpointConnections@2421-06-01-preview
Fake.EventHub/namespaces/virtualnetworkrules@2418-01-01-preview
Fake.ExtendedLocation/customLocations@2421-03-15-preview
Fake.ExtendedLocation/customLocations@2421-08-15
Fake.HanaOnAzure/hanaInstances@2417-11-03-preview
Fake.HanaOnAzure/sapMonitors@2420-02-07-preview
Fake.HanaOnAzure/sapMonitors/providerInstances@2420-02-07-preview
Fake.HardwareSecurityModules/dedicatedHSMs@2418-10-31-preview
Fake.HDInsight/clusters@2415-03-01-preview
Fake.HDInsight/clusters@2418-06-01-preview
Fake.HDInsight/clusters@2421-06-01
Fake.HDInsight/clusters/applications@2415-03-01-preview
Fake.HDInsight/clusters/applications@2418-06-01-preview
Fake.HDInsight/clusters/applications@2421-06-01
Fake.HDInsight/clusters/extensions@2415-03-01-preview
Fake.HDInsight/clusters/extensions@2418-06-01-preview
Fake.HDInsight/clusters/extensions@2421-06-01
Fake.HDInsight/clusters/privateEndpointConnections@2421-06-01
Fake.HealthBot/healthBots@2420-10-20
Fake.HealthBot/healthBots@2420-10-20-preview
Fake.HealthBot/healthBots@2420-12-08
Fake.HealthBot/healthBots@2420-12-08-preview
Fake.HealthBot/healthBots@2421-06-10
Fake.HealthcareApis/services@2418-08-20-preview
Fake.HealthcareApis/services@2419-09-16
Fake.HealthcareApis/services@2420-03-15
Fake.HealthcareApis/services@2420-03-30
Fake.HealthcareApis/services@2421-01-11
Fake.HealthcareApis/services@2421-06-01-preview
Fake.HealthcareApis/services/privateEndpointConnections@2420-03-30
Fake.HealthcareApis/services/privateEndpointConnections@2421-01-11
Fake.HealthcareApis/services/privateEndpointConnections@2421-06-01-preview
Fake.HealthcareApis/workspaces@2421-06-01-preview
Fake.HealthcareApis/workspaces/dicomservices@2421-06-01-preview
Fake.HealthcareApis/workspaces/fhirservices@2421-06-01-preview
Fake.HealthcareApis/workspaces/iotconnectors@2421-06-01-preview
Fake.HealthcareApis/workspaces/iotconnectors/fhirdestinations@2421-06-01-preview
Fake.HybridCompute/machines@2419-03-18-preview
Fake.HybridCompute/machines@2419-08-02-preview
Fake.HybridCompute/machines@2419-12-12
Fake.HybridCompute/machines@2420-07-30-preview
Fake.HybridCompute/machines@2420-08-02
Fake.HybridCompute/machines@2420-08-15-preview
Fake.HybridCompute/machines@2421-01-28-preview
Fake.HybridCompute/machines@2421-03-25-preview
Fake.HybridCompute/machines@2421-04-22-preview
Fake.HybridCompute/machines@2421-05-17-preview
Fake.HybridCompute/machines@2421-05-20
Fake.HybridCompute/machines@2421-06-10-preview
Fake.HybridCompute/machines/extensions@2419-08-02-preview
Fake.HybridCompute/machines/extensions@2419-12-12
Fake.HybridCompute/machines/extensions@2420-07-30-preview
Fake.HybridCompute/machines/extensions@2420-08-02
Fake.HybridCompute/machines/extensions@2420-08-15-preview
Fake.HybridCompute/machines/extensions@2421-01-28-preview
Fake.HybridCompute/machines/extensions@2421-03-25-preview
Fake.HybridCompute/machines/extensions@2421-04-22-preview
Fake.HybridCompute/machines/extensions@2421-05-17-preview
Fake.HybridCompute/machines/extensions@2421-05-20
Fake.HybridCompute/machines/extensions@2421-06-10-preview
Fake.HybridCompute/privateLinkScopes@2420-08-15-preview
Fake.HybridCompute/privateLinkScopes@2421-01-28-preview
Fake.HybridCompute/privateLinkScopes@2421-03-25-preview
Fake.HybridCompute/privateLinkScopes@2421-04-22-preview
Fake.HybridCompute/privateLinkScopes@2421-05-17-preview
Fake.HybridCompute/privateLinkScopes@2421-05-20
Fake.HybridCompute/privateLinkScopes@2421-06-10-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2420-08-15-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-01-28-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-03-25-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-04-22-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-05-17-preview
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-05-20
Fake.HybridCompute/privateLinkScopes/privateEndpointConnections@2421-06-10-preview
Fake.HybridCompute/privateLinkScopes/scopedResources@2420-08-15-preview
Fake.HybridData/dataManagers@2416-06-01
Fake.HybridData/dataManagers@2419-06-01
Fake.HybridData/dataManagers/dataServices/jobDefinitions@2416-06-01
Fake.HybridData/dataManagers/dataServices/jobDefinitions@2419-06-01
Fake.HybridData/dataManagers/dataStores@2416-06-01
Fake.HybridData/dataManagers/dataStores@2419-06-01
Fake.HybridNetwork/devices@2420-01-01-preview
Fake.HybridNetwork/devices@2421-05-01
Fake.HybridNetwork/networkFunctions@2420-01-01-preview
Fake.HybridNetwork/networkFunctions@2421-05-01
Fake.ImportExport/jobs@2416-11-01
Fake.ImportExport/jobs@2420-08-01
Fake.ImportExport/jobs@2421-01-01
Fake.insights/components/analyticsItems@2415-05-01
Fake.Insights/components/Annotations@2415-05-01
Fake.Insights/components/currentbillingfeatures@2415-05-01
Fake.Insights/components/exportconfiguration@2415-05-01
Fake.Insights/components/favorites@2415-05-01
Fake.insights/components/myanalyticsItems@2415-05-01
Fake.Insights/components/ProactiveDetectionConfigs@2415-05-01
Fake.Insights/components/ProactiveDetectionConfigs@2418-05-01-preview
Fake.Insights/myWorkbooks@2415-05-01
Fake.Insights/myWorkbooks@2420-10-20
Fake.Insights/myWorkbooks@2421-03-08
Fake.insights/workbooks@2415-05-01
Fake.insights/workbooks@2418-06-17-preview
Fake.insights/workbooks@2420-10-20
Fake.insights/workbooks@2421-03-08
Fake.insights/components/pricingPlans@2417-10-01
Fake.insights/workbooktemplates@2419-10-17-preview
Fake.insights/workbooktemplates@2420-11-20
Fake.insights/components/linkedStorageAccounts@2420-03-01-preview
Fake.Insights/alertrules@2414-04-01
Fake.Insights/alertrules@2416-03-01
Fake.insights/activityLogAlerts@2417-03-01-preview
Fake.insights/activityLogAlerts@2417-04-01
Fake.insights/activityLogAlerts@2420-10-01
Fake.insights/actionGroups@2417-04-01
Fake.insights/actionGroups@2418-03-01
Fake.insights/actionGroups@2418-09-01
Fake.insights/actionGroups@2419-03-01
Fake.insights/actionGroups@2419-06-01
Fake.Insights/metricAlerts@2418-03-01
Fake.Insights/scheduledQueryRules@2418-04-16
Fake.Insights/scheduledQueryRules@2420-05-01-preview
Fake.Insights/scheduledQueryRules@2421-02-01-preview
Fake.insights/guestDiagnosticSettings@2418-06-01-preview
Fake.insights/privateLinkScopes@2419-10-17-preview
Fake.insights/privateLinkScopes@2421-07-01-preview
Fake.Insights/privateLinkScopes/privateEndpointConnections@2419-10-17-preview
Fake.Insights/privateLinkScopes/privateEndpointConnections@2421-07-01-preview
Fake.Insights/privateLinkScopes/scopedResources@2419-10-17-preview
Fake.Insights/privateLinkScopes/scopedResources@2421-07-01-preview
Fake.Insights/dataCollectionRules@2419-11-01-preview
Fake.Insights/dataCollectionRules@2421-04-01
Fake.Insights/dataCollectionEndpoints@2421-04-01
Fake.IoTCentral/iotApps@2418-09-01
Fake.IoTCentral/iotApps@2421-06-01
Fake.KeyVault/vaults@2415-06-01
Fake.KeyVault/vaults@2416-10-01
Fake.KeyVault/vaults@2418-02-14
Fake.KeyVault/vaults@2418-02-14-preview
Fake.KeyVault/vaults@2419-09-01
Fake.KeyVault/vaults@2420-04-01-preview
Fake.KeyVault/vaults@2421-04-01-preview
Fake.KeyVault/vaults@2421-06-01-preview
Fake.KeyVault/vaults/accessPolicies@2416-10-01
Fake.KeyVault/vaults/accessPolicies@2418-02-14
Fake.KeyVault/vaults/accessPolicies@2418-02-14-preview
Fake.KeyVault/vaults/accessPolicies@2419-09-01
Fake.KeyVault/vaults/accessPolicies@2420-04-01-preview
Fake.KeyVault/vaults/accessPolicies@2421-04-01-preview
Fake.KeyVault/vaults/accessPolicies@2421-06-01-preview
Fake.KeyVault/vaults/secrets@2416-10-01
Fake.KeyVault/vaults/secrets@2418-02-14
Fake.KeyVault/vaults/secrets@2418-02-14-preview
Fake.KeyVault/vaults/secrets@2419-09-01
Fake.KeyVault/vaults/secrets@2420-04-01-preview
Fake.KeyVault/vaults/secrets@2421-04-01-preview
Fake.KeyVault/vaults/secrets@2421-06-01-preview
Fake.KeyVault/vaults/privateEndpointConnections@2418-02-14
Fake.KeyVault/vaults/privateEndpointConnections@2419-09-01
Fake.KeyVault/vaults/privateEndpointConnections@2420-04-01-preview
Fake.KeyVault/vaults/privateEndpointConnections@2421-04-01-preview
Fake.KeyVault/vaults/privateEndpointConnections@2421-06-01-preview
Fake.KeyVault/vaults/keys@2419-09-01
Fake.KeyVault/vaults/keys@2420-04-01-preview
Fake.KeyVault/vaults/keys@2421-04-01-preview
Fake.KeyVault/vaults/keys@2421-06-01-preview
Fake.KeyVault/managedHSMs@2420-04-01-preview
Fake.KeyVault/managedHSMs@2421-04-01-preview
Fake.KeyVault/managedHSMs@2421-06-01-preview
Fake.KeyVault/managedHSMs/privateEndpointConnections@2421-04-01-preview
Fake.KeyVault/managedHSMs/privateEndpointConnections@2421-06-01-preview
Fake.Kubernetes/connectedClusters@2420-01-01-preview
Fake.Kubernetes/connectedClusters@2421-03-01
Fake.Kubernetes/connectedClusters@2421-04-01-preview
Fake.Kubernetes/connectedClusters@2421-10-01
Fake.LabServices/labaccounts@2418-10-15
Fake.LabServices/labaccounts/galleryimages@2418-10-15
Fake.LabServices/labaccounts/labs@2418-10-15
Fake.LabServices/labaccounts/labs/environmentsettings@2418-10-15
Fake.LabServices/labaccounts/labs/environmentsettings/environments@2418-10-15
Fake.LabServices/labaccounts/labs/users@2418-10-15
Fake.LabServices/labPlans@2421-10-01-preview
Fake.LabServices/labPlans/images@2421-10-01-preview
Fake.LabServices/labs@2421-10-01-preview
Fake.LabServices/labs/schedules@2421-10-01-preview
Fake.LabServices/labs/users@2421-10-01-preview
Fake.Logz/monitors@2420-10-01
Fake.Logz/monitors@2420-10-01-preview
Fake.Logz/monitors/accounts@2420-10-01
Fake.Logz/monitors/accounts@2420-10-01-preview
Fake.Logz/monitors/accounts/tagRules@2420-10-01
Fake.Logz/monitors/accounts/tagRules@2420-10-01-preview
Fake.Logz/monitors/singleSignOnConfigurations@2420-10-01
Fake.Logz/monitors/singleSignOnConfigurations@2420-10-01-preview
Fake.Logz/monitors/tagRules@2420-10-01
Fake.Logz/monitors/tagRules@2420-10-01-preview
Fake.MachineLearning/workspaces@2416-04-01
Fake.MachineLearning/workspaces@2419-10-01
Fake.MachineLearning/commitmentPlans@2416-05-01-preview
Fake.MachineLearning/webServices@2416-05-01-preview
Fake.MachineLearning/webServices@2417-01-01
Fake.MachineLearningCompute/operationalizationClusters@2417-06-01-preview
Fake.MachineLearningCompute/operationalizationClusters@2417-08-01-preview
Fake.MachineLearningExperimentation/accounts@2417-05-01-preview
Fake.MachineLearningExperimentation/accounts/workspaces@2417-05-01-preview
Fake.MachineLearningExperimentation/accounts/workspaces/projects@2417-05-01-preview
Fake.MachineLearningServices/workspaces@2418-03-01-preview
Fake.MachineLearningServices/workspaces@2418-11-19
Fake.MachineLearningServices/workspaces@2419-05-01
Fake.MachineLearningServices/workspaces@2419-06-01
Fake.MachineLearningServices/workspaces@2419-11-01
Fake.MachineLearningServices/workspaces@2420-01-01
Fake.MachineLearningServices/workspaces@2420-02-18-preview
Fake.MachineLearningServices/workspaces@2420-03-01
Fake.MachineLearningServices/workspaces@2420-04-01
Fake.MachineLearningServices/workspaces@2420-05-01-preview
Fake.MachineLearningServices/workspaces@2420-05-15-preview
Fake.MachineLearningServices/workspaces@2420-06-01
Fake.MachineLearningServices/workspaces@2420-08-01
Fake.MachineLearningServices/workspaces@2420-09-01-preview
Fake.MachineLearningServices/workspaces@2421-01-01
Fake.MachineLearningServices/workspaces@2421-03-01-preview
Fake.MachineLearningServices/workspaces@2421-04-01
Fake.MachineLearningServices/workspaces@2421-07-01
Fake.MachineLearningServices/workspaces/computes@2418-03-01-preview
Fake.MachineLearningServices/workspaces/computes@2418-11-19
Fake.MachineLearningServices/workspaces/computes@2419-05-01
Fake.MachineLearningServices/workspaces/computes@2419-06-01
Fake.MachineLearningServices/workspaces/computes@2419-11-01
Fake.MachineLearningServices/workspaces/computes@2420-01-01
Fake.MachineLearningServices/workspaces/computes@2420-02-18-preview
Fake.MachineLearningServices/workspaces/computes@2420-03-01
Fake.MachineLearningServices/workspaces/computes@2420-04-01
Fake.MachineLearningServices/workspaces/computes@2420-05-01-preview
Fake.MachineLearningServices/workspaces/computes@2420-05-15-preview
Fake.MachineLearningServices/workspaces/computes@2420-06-01
Fake.MachineLearningServices/workspaces/computes@2420-08-01
Fake.MachineLearningServices/workspaces/computes@2420-09-01-preview
Fake.MachineLearningServices/workspaces/computes@2421-01-01
Fake.MachineLearningServices/workspaces/computes@2421-03-01-preview
Fake.MachineLearningServices/workspaces/computes@2421-04-01
Fake.MachineLearningServices/workspaces/computes@2421-07-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-01-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-02-18-preview
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-03-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-04-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-05-01-preview
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-05-15-preview
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-06-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-08-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2420-09-01-preview
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2421-01-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2421-03-01-preview
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2421-04-01
Fake.MachineLearningServices/workspaces/privateEndpointConnections@2421-07-01
Fake.MachineLearningServices/workspaces/datasets@2420-05-01-preview
Fake.MachineLearningServices/workspaces/datastores@2420-05-01-preview
Fake.MachineLearningServices/workspaces/datastores@2421-03-01-preview
Fake.MachineLearningServices/workspaces/linkedWorkspaces@2420-05-01-preview
Fake.MachineLearningServices/workspaces/linkedWorkspaces@2420-05-15-preview
Fake.MachineLearningServices/workspaces/services@2420-05-01-preview
Fake.MachineLearningServices/workspaces/services@2420-05-15-preview
Fake.MachineLearningServices/workspaces/services@2420-09-01-preview
Fake.MachineLearningServices/workspaces/services@2421-01-01
Fake.MachineLearningServices/workspaces/services@2421-04-01
Fake.MachineLearningServices/workspaces/connections@2420-06-01
Fake.MachineLearningServices/workspaces/connections@2420-08-01
Fake.MachineLearningServices/workspaces/connections@2420-09-01-preview
Fake.MachineLearningServices/workspaces/connections@2421-01-01
Fake.MachineLearningServices/workspaces/connections@2421-03-01-preview
Fake.MachineLearningServices/workspaces/connections@2421-04-01
Fake.MachineLearningServices/workspaces/connections@2421-07-01
Fake.MachineLearningServices/workspaces/labelingJobs@2420-09-01-preview
Fake.MachineLearningServices/workspaces/labelingJobs@2421-03-01-preview
Fake.MachineLearningServices/workspaces/linkedServices@2420-09-01-preview
Fake.MachineLearningServices/workspaces/batchEndpoints@2421-03-01-preview
Fake.MachineLearningServices/workspaces/batchEndpoints/deployments@2421-03-01-preview
Fake.MachineLearningServices/workspaces/codes@2421-03-01-preview
Fake.MachineLearningServices/workspaces/codes/versions@2421-03-01-preview
Fake.MachineLearningServices/workspaces/data@2421-03-01-preview
Fake.MachineLearningServices/workspaces/data/versions@2421-03-01-preview
Fake.MachineLearningServices/workspaces/environments@2421-03-01-preview
Fake.MachineLearningServices/workspaces/environments/versions@2421-03-01-preview
Fake.MachineLearningServices/workspaces/jobs@2421-03-01-preview
Fake.MachineLearningServices/workspaces/models@2421-03-01-preview
Fake.MachineLearningServices/workspaces/models/versions@2421-03-01-preview
Fake.MachineLearningServices/workspaces/onlineEndpoints@2421-03-01-preview
Fake.MachineLearningServices/workspaces/onlineEndpoints/deployments@2421-03-01-preview
Fake.Maintenance/maintenanceConfigurations@2418-06-01-preview
Fake.Maintenance/maintenanceConfigurations@2420-04-01
Fake.Maintenance/maintenanceConfigurations@2420-07-01-preview
Fake.Maintenance/maintenanceConfigurations@2421-04-01-preview
Fake.Maintenance/maintenanceConfigurations@2421-05-01
Fake.Maintenance/maintenanceConfigurations@2421-09-01-preview
Fake.ManagedNetwork/managedNetworks@2419-06-01-preview
Fake.ManagedNetwork/managedNetworks/managedNetworkGroups@2419-06-01-preview
Fake.ManagedNetwork/managedNetworks/managedNetworkPeeringPolicies@2419-06-01-preview
Fake.Maps/accounts@2417-01-01-preview
Fake.Maps/accounts@2418-05-01
Fake.Maps/accounts@2420-02-01-preview
Fake.Maps/accounts@2421-02-01
Fake.Maps/accounts@2421-07-01-preview
Fake.Maps/accounts/creators@2420-02-01-preview
Fake.Maps/accounts/creators@2421-02-01
Fake.Maps/accounts/creators@2421-07-01-preview
Fake.Maps/accounts/privateAtlases@2420-02-01-preview
Fake.Migrate/projects@2418-02-02
Fake.Migrate/projects@2418-02-02
Fake.Migrate/projects/groups@2418-02-02
Fake.Migrate/projects/groups@2418-02-02
Fake.Migrate/projects/groups/assessments@2418-02-02
Fake.Migrate/projects/groups/assessments@2418-02-02
Fake.Migrate/migrateProjects@2418-09-01-preview
Fake.Migrate/migrateProjects@2420-05-01
Fake.Migrate/migrateProjects@2420-05-01
Fake.Migrate/migrateProjects@2418-09-01-preview
Fake.Migrate/migrateProjects/solutions@2418-09-01-preview
Fake.Migrate/migrateProjects/solutions@2418-09-01-preview
Fake.Migrate/assessmentProjects@2419-10-01
Fake.Migrate/assessmentProjects@2419-10-01
Fake.Migrate/assessmentProjects/groups@2419-10-01
Fake.Migrate/assessmentProjects/groups@2419-10-01
Fake.Migrate/assessmentProjects/groups/assessments@2419-10-01
Fake.Migrate/assessmentProjects/groups/assessments@2419-10-01
Fake.Migrate/assessmentProjects/hypervcollectors@2419-10-01
Fake.Migrate/assessmentProjects/hypervcollectors@2419-10-01
Fake.Migrate/assessmentProjects/importcollectors@2419-10-01
Fake.Migrate/assessmentProjects/importcollectors@2419-10-01
Fake.Migrate/assessmentprojects/privateEndpointConnections@2419-10-01
Fake.Migrate/assessmentprojects/privateEndpointConnections@2419-10-01
Fake.Migrate/assessmentProjects/servercollectors@2419-10-01
Fake.Migrate/assessmentProjects/servercollectors@2419-10-01
Fake.Migrate/assessmentProjects/vmwarecollectors@2419-10-01
Fake.Migrate/assessmentProjects/vmwarecollectors@2419-10-01
Fake.Migrate/migrateProjects/privateEndpointConnections@2420-05-01
Fake.Migrate/migrateProjects/privateEndpointConnections@2420-05-01
Fake.Migrate/moveCollections@2419-10-01-preview
Fake.Migrate/moveCollections@2421-01-01
Fake.Migrate/moveCollections/moveResources@2419-10-01-preview
Fake.Migrate/moveCollections/moveResources@2421-01-01
Fake.MixedReality/spatialAnchorsAccounts@2419-02-28-preview
Fake.MixedReality/spatialAnchorsAccounts@2419-12-02-preview
Fake.MixedReality/spatialAnchorsAccounts@2420-05-01
Fake.MixedReality/spatialAnchorsAccounts@2421-01-01
Fake.MixedReality/spatialAnchorsAccounts@2421-03-01-preview
Fake.MixedReality/remoteRenderingAccounts@2419-12-02-preview
Fake.MixedReality/remoteRenderingAccounts@2420-04-06-preview
Fake.MixedReality/remoteRenderingAccounts@2421-01-01
Fake.MixedReality/remoteRenderingAccounts@2421-03-01-preview
Fake.MixedReality/objectAnchorsAccounts@2421-03-01-preview
Fake.NetApp/netAppAccounts@2417-08-15
Fake.NetApp/netAppAccounts@2419-05-01
Fake.NetApp/netAppAccounts@2419-06-01
Fake.NetApp/netAppAccounts@2419-07-01
Fake.NetApp/netAppAccounts@2419-08-01
Fake.NetApp/netAppAccounts@2419-10-01
Fake.NetApp/netAppAccounts@2419-11-01
Fake.NetApp/netAppAccounts@2420-02-01
Fake.NetApp/netAppAccounts@2420-03-01
Fake.NetApp/netAppAccounts@2420-05-01
Fake.NetApp/netAppAccounts@2420-06-01
Fake.NetApp/netAppAccounts@2420-07-01
Fake.NetApp/netAppAccounts@2420-08-01
Fake.NetApp/netAppAccounts@2420-09-01
Fake.NetApp/netAppAccounts@2420-11-01
Fake.NetApp/netAppAccounts@2420-12-01
Fake.NetApp/netAppAccounts@2421-02-01
Fake.NetApp/netAppAccounts@2421-04-01
Fake.NetApp/netAppAccounts@2421-04-01-preview
Fake.NetApp/netAppAccounts@2421-06-01
Fake.NetApp/netAppAccounts/capacityPools@2417-08-15
Fake.NetApp/netAppAccounts/capacityPools@2419-05-01
Fake.NetApp/netAppAccounts/capacityPools@2419-06-01
Fake.NetApp/netAppAccounts/capacityPools@2419-07-01
Fake.NetApp/netAppAccounts/capacityPools@2419-08-01
Fake.NetApp/netAppAccounts/capacityPools@2419-10-01
Fake.NetApp/netAppAccounts/capacityPools@2419-11-01
Fake.NetApp/netAppAccounts/capacityPools@2420-02-01
Fake.NetApp/netAppAccounts/capacityPools@2420-03-01
Fake.NetApp/netAppAccounts/capacityPools@2420-05-01
Fake.NetApp/netAppAccounts/capacityPools@2420-06-01
Fake.NetApp/netAppAccounts/capacityPools@2420-07-01
Fake.NetApp/netAppAccounts/capacityPools@2420-08-01
Fake.NetApp/netAppAccounts/capacityPools@2420-09-01
Fake.NetApp/netAppAccounts/capacityPools@2420-11-01
Fake.NetApp/netAppAccounts/capacityPools@2420-12-01
Fake.NetApp/netAppAccounts/capacityPools@2421-02-01
Fake.NetApp/netAppAccounts/capacityPools@2421-04-01
Fake.NetApp/netAppAccounts/capacityPools@2421-04-01-preview
Fake.NetApp/netAppAccounts/capacityPools@2421-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2417-08-15
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-05-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-07-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-08-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-10-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2419-11-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-02-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-03-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-05-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-07-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-08-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-09-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-11-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2420-12-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2421-02-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2421-04-01
Fake.NetApp/netAppAccounts/capacityPools/volumes@2421-04-01-preview
Fake.NetApp/netAppAccounts/capacityPools/volumes@2421-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2417-08-15
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-05-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-07-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-08-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-10-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2419-11-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-02-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-03-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-05-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-07-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-08-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-09-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-11-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2420-12-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2421-02-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2421-04-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2421-04-01-preview
Fake.NetApp/netAppAccounts/capacityPools/volumes/snapshots@2421-06-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-05-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-06-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-07-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-08-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-09-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-11-01
Fake.NetApp/netAppAccounts/backupPolicies@2420-12-01
Fake.NetApp/netAppAccounts/backupPolicies@2421-02-01
Fake.NetApp/netAppAccounts/backupPolicies@2421-04-01
Fake.NetApp/netAppAccounts/backupPolicies@2421-04-01-preview
Fake.NetApp/netAppAccounts/backupPolicies@2421-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-05-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-06-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-07-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-08-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-09-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-11-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2420-12-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2421-02-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2421-04-01
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2421-04-01-preview
Fake.NetApp/netAppAccounts/capacityPools/volumes/backups@2421-06-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-05-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-06-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-07-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-08-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-09-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-11-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2420-12-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2421-02-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2421-04-01
Fake.NetApp/netAppAccounts/snapshotPolicies@2421-04-01-preview
Fake.NetApp/netAppAccounts/snapshotPolicies@2421-06-01
Fake.Network/frontDoors@2418-08-01
Fake.Network/frontDoors@2419-04-01
Fake.Network/frontDoors@2419-05-01
Fake.Network/frontDoors@2420-01-01
Fake.Network/frontDoors@2420-04-01
Fake.Network/frontDoors@2420-05-01
Fake.Network/FrontDoorWebApplicationFirewallPolicies@2418-08-01
Fake.Network/FrontDoorWebApplicationFirewallPolicies@2419-03-01
Fake.Network/FrontDoorWebApplicationFirewallPolicies@2419-10-01
Fake.Network/FrontDoorWebApplicationFirewallPolicies@2420-04-01
Fake.Network/FrontDoorWebApplicationFirewallPolicies@2420-11-01
Fake.Network/NetworkExperimentProfiles@2419-11-01
Fake.Network/NetworkExperimentProfiles/Experiments@2419-11-01
Fake.Network/frontDoors/rulesEngines@2420-01-01
Fake.Network/frontDoors/rulesEngines@2420-04-01
Fake.Network/frontDoors/rulesEngines@2420-05-01
Fake.NotificationHubs/namespaces@2414-09-01
Fake.NotificationHubs/namespaces@2416-03-01
Fake.NotificationHubs/namespaces@2417-04-01
Fake.NotificationHubs/namespaces/AuthorizationRules@2414-09-01
Fake.NotificationHubs/namespaces/AuthorizationRules@2416-03-01
Fake.NotificationHubs/namespaces/AuthorizationRules@2417-04-01
Fake.NotificationHubs/namespaces/notificationHubs@2414-09-01
Fake.NotificationHubs/namespaces/notificationHubs@2416-03-01
Fake.NotificationHubs/namespaces/notificationHubs@2417-04-01
Fake.NotificationHubs/namespaces/notificationHubs/AuthorizationRules@2414-09-01
Fake.NotificationHubs/namespaces/notificationHubs/AuthorizationRules@2416-03-01
Fake.NotificationHubs/namespaces/notificationHubs/AuthorizationRules@2417-04-01
Fake.OffAzure/HyperVSites@2420-01-01
Fake.OffAzure/HyperVSites@2420-07-07
Fake.OffAzure/HyperVSites/clusters@2420-01-01
Fake.OffAzure/HyperVSites/clusters@2420-07-07
Fake.OffAzure/HyperVSites/hosts@2420-01-01
Fake.OffAzure/HyperVSites/hosts@2420-07-07
Fake.OffAzure/VMwareSites@2420-01-01
Fake.OffAzure/VMwareSites@2420-07-07
Fake.OffAzure/VMwareSites/vCenters@2420-01-01
Fake.OffAzure/VMwareSites/vCenters@2420-07-07
Fake.OffAzure/MasterSites@2420-07-07
Fake.OperationalInsights/workspaces/savedSearches@2415-03-20
Fake.OperationalInsights/workspaces/savedSearches@2420-03-01-preview
Fake.OperationalInsights/workspaces/savedSearches@2420-08-01
Fake.OperationalInsights/workspaces/storageInsightConfigs@2415-03-20
Fake.OperationalInsights/workspaces/storageInsightConfigs@2420-03-01-preview
Fake.OperationalInsights/workspaces/storageInsightConfigs@2420-08-01
Fake.OperationalInsights/workspaces@2415-11-01-preview
Fake.OperationalInsights/workspaces@2420-03-01-preview
Fake.OperationalInsights/workspaces@2420-08-01
Fake.OperationalInsights/workspaces@2420-10-01
Fake.OperationalInsights/workspaces@2421-06-01
Fake.OperationalInsights/workspaces/dataSources@2415-11-01-preview
Fake.OperationalInsights/workspaces/dataSources@2420-03-01-preview
Fake.OperationalInsights/workspaces/dataSources@2420-08-01
Fake.OperationalInsights/workspaces/linkedServices@2415-11-01-preview
Fake.OperationalInsights/workspaces/linkedServices@2419-08-01-preview
Fake.OperationalInsights/workspaces/linkedServices@2420-03-01-preview
Fake.OperationalInsights/workspaces/linkedServices@2420-08-01
Fake.OperationalInsights/clusters@2419-08-01-preview
Fake.OperationalInsights/clusters@2420-03-01-preview
Fake.OperationalInsights/clusters@2420-08-01
Fake.OperationalInsights/clusters@2420-10-01
Fake.OperationalInsights/clusters@2421-06-01
Fake.OperationalInsights/workspaces/dataExports@2419-08-01-preview
Fake.OperationalInsights/workspaces/dataExports@2420-03-01-preview
Fake.OperationalInsights/workspaces/dataExports@2420-08-01
Fake.OperationalInsights/workspaces/linkedStorageAccounts@2419-08-01-preview
Fake.OperationalInsights/workspaces/linkedStorageAccounts@2420-03-01-preview
Fake.OperationalInsights/workspaces/linkedStorageAccounts@2420-08-01
Fake.OperationalInsights/queryPacks@2419-09-01-preview
Fake.OperationalInsights/queryPacks/queries@2419-09-01-preview
Fake.Peering/peerings@2419-08-01-preview
Fake.Peering/peerings@2419-09-01-preview
Fake.Peering/peerings@2420-01-01-preview
Fake.Peering/peerings@2420-04-01
Fake.Peering/peerings@2420-10-01
Fake.Peering/peerings@2421-01-01
Fake.Peering/peerings@2421-06-01
Fake.Peering/peeringServices@2419-08-01-preview
Fake.Peering/peeringServices@2419-09-01-preview
Fake.Peering/peeringServices@2420-01-01-preview
Fake.Peering/peeringServices@2420-04-01
Fake.Peering/peeringServices@2420-10-01
Fake.Peering/peeringServices@2421-01-01
Fake.Peering/peeringServices@2421-06-01
Fake.Peering/peeringServices/prefixes@2419-08-01-preview
Fake.Peering/peeringServices/prefixes@2419-09-01-preview
Fake.Peering/peeringServices/prefixes@2420-01-01-preview
Fake.Peering/peeringServices/prefixes@2420-04-01
Fake.Peering/peeringServices/prefixes@2420-10-01
Fake.Peering/peeringServices/prefixes@2421-01-01
Fake.Peering/peeringServices/prefixes@2421-06-01
Fake.Peering/peerings/registeredAsns@2420-01-01-preview
Fake.Peering/peerings/registeredAsns@2420-04-01
Fake.Peering/peerings/registeredAsns@2420-10-01
Fake.Peering/peerings/registeredAsns@2421-01-01
Fake.Peering/peerings/registeredAsns@2421-06-01
Fake.Peering/peerings/registeredPrefixes@2420-01-01-preview
Fake.Peering/peerings/registeredPrefixes@2420-04-01
Fake.Peering/peerings/registeredPrefixes@2420-10-01
Fake.Peering/peerings/registeredPrefixes@2421-01-01
Fake.Peering/peerings/registeredPrefixes@2421-06-01
Fake.Peering/peeringServices/connectionMonitorTests@2421-06-01
Fake.PolicyInsights/remediations@2418-07-01-preview
Fake.PolicyInsights/remediations@2419-07-01
Fake.PolicyInsights/attestations@2421-01-01
Fake.Portal/dashboards@2415-08-01-preview
Fake.Portal/dashboards@2418-10-01-preview
Fake.Portal/dashboards@2419-01-01-preview
Fake.Portal/dashboards@2420-09-01-preview
Fake.PowerBI/workspaceCollections@2416-01-29
Fake.PowerBI/privateLinkServicesForPowerBI@2420-06-01
Fake.PowerBI/privateLinkServicesForPowerBI/privateEndpointConnections@2420-06-01
Fake.PowerBIDedicated/capacities@2417-10-01
Fake.PowerBIDedicated/capacities@2421-01-01
Fake.PowerBIDedicated/autoScaleVCores@2421-01-01
Fake.PowerPlatform/accounts@2420-10-30-preview
Fake.PowerPlatform/enterprisePolicies@2420-10-30-preview
Fake.PowerPlatform/enterprisePolicies/privateEndpointConnections@2420-10-30-preview
Fake.Purview/accounts@2420-12-01-preview
Fake.Purview/accounts@2421-07-01
Fake.Purview/accounts/privateEndpointConnections@2420-12-01-preview
Fake.Purview/accounts/privateEndpointConnections@2421-07-01
Fake.Quantum/workspaces@2419-11-04-preview
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2416-12-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2420-10-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2420-12-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-01-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-02-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-02-01-preview
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-02-10
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-03-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-04-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-06-01
Fake.RecoveryServices/vaults/backupFabrics/protectionContainers@2421-07-01
Fake.RecoveryServices/vaults/backupstorageconfig@2416-12-01
Fake.RecoveryServices/vaults/backupstorageconfig@2418-12-20
Fake.RecoveryServices/vaults/backupstorageconfig@2421-04-01
Fake.RecoveryServices/vaults/backupstorageconfig@2421-07-01
Fake.RecoveryServices/vaults/backupconfig@2419-06-15
Fake.RecoveryServices/vaults/backupconfig@2420-10-01
Fake.RecoveryServices/vaults/backupconfig@2420-12-01
Fake.RecoveryServices/vaults/backupconfig@2421-01-01
Fake.RecoveryServices/vaults/backupconfig@2421-02-01
Fake.RecoveryServices/vaults/backupconfig@2421-02-01-preview
Fake.RecoveryServices/vaults/backupconfig@2421-02-10
Fake.RecoveryServices/vaults/backupconfig@2421-03-01
Fake.RecoveryServices/vaults/backupconfig@2421-04-01
Fake.RecoveryServices/vaults/backupconfig@2421-06-01
Fake.RecoveryServices/vaults/backupconfig@2421-07-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2420-02-02
Fake.RecoveryServices/vaults/privateEndpointConnections@2420-10-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2420-12-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-01-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-02-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-02-01-preview
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-02-10
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-03-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-04-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-06-01
Fake.RecoveryServices/vaults/privateEndpointConnections@2421-07-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2420-10-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2420-12-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-01-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-02-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-02-01-preview
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-02-10
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-03-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-04-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-06-01
Fake.RecoveryServices/vaults/backupEncryptionConfigs@2421-07-01
Fake.RecoveryServices/vaults/backupResourceGuardProxies@2421-02-01-preview
Fake.RecoveryServices/vaults/backupResourceGuardProxies@2421-07-01
Fake.RecoveryServices/vaults/certificates@2416-06-01
Fake.RecoveryServices/vaults/certificates@2420-02-02
Fake.RecoveryServices/vaults/certificates@2420-10-01
Fake.RecoveryServices/vaults/certificates@2421-01-01
Fake.RecoveryServices/vaults/certificates@2421-02-10
Fake.RecoveryServices/vaults/certificates@2421-03-01
Fake.RecoveryServices/vaults/certificates@2421-04-01
Fake.RecoveryServices/vaults/certificates@2421-06-01
Fake.RecoveryServices/vaults/certificates@2421-07-01
Fake.RecoveryServices/vaults/extendedInformation@2416-06-01
Fake.RecoveryServices/vaults/extendedInformation@2420-02-02
Fake.RecoveryServices/vaults/extendedInformation@2420-10-01
Fake.RecoveryServices/vaults/extendedInformation@2421-01-01
Fake.RecoveryServices/vaults/extendedInformation@2421-02-10
Fake.RecoveryServices/vaults/extendedInformation@2421-03-01
Fake.RecoveryServices/vaults/extendedInformation@2421-04-01
Fake.RecoveryServices/vaults/extendedInformation@2421-06-01
Fake.RecoveryServices/vaults/extendedInformation@2421-07-01
Fake.RecoveryServices/vaults/replicationAlertSettings@2416-08-10
Fake.RecoveryServices/vaults/replicationAlertSettings@2418-01-10
Fake.RecoveryServices/vaults/replicationAlertSettings@2418-07-10
Fake.RecoveryServices/vaults/replicationAlertSettings@2421-02-10
Fake.RecoveryServices/vaults/replicationAlertSettings@2421-03-01
Fake.RecoveryServices/vaults/replicationAlertSettings@2421-04-01
Fake.RecoveryServices/vaults/replicationAlertSettings@2421-06-01
Fake.RecoveryServices/vaults/replicationAlertSettings@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationNetworks/replicationNetworkMappings@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectedItems@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationProtectionContainerMappings@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationStorageClassifications/replicationStorageClassificationMappings@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2416-08-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationvCenters@2421-07-01
Fake.RecoveryServices/vaults/replicationPolicies@2416-08-10
Fake.RecoveryServices/vaults/replicationPolicies@2418-01-10
Fake.RecoveryServices/vaults/replicationPolicies@2418-07-10
Fake.RecoveryServices/vaults/replicationPolicies@2421-02-10
Fake.RecoveryServices/vaults/replicationPolicies@2421-03-01
Fake.RecoveryServices/vaults/replicationPolicies@2421-04-01
Fake.RecoveryServices/vaults/replicationPolicies@2421-06-01
Fake.RecoveryServices/vaults/replicationPolicies@2421-07-01
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2416-08-10
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2418-01-10
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2418-07-10
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2421-02-10
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2421-03-01
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2421-04-01
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2421-06-01
Fake.RecoveryServices/vaults/replicationRecoveryPlans@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationProtectionContainers/replicationMigrationItems@2421-07-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2418-01-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2418-07-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2421-02-10
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2421-03-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2421-04-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2421-06-01
Fake.RecoveryServices/vaults/replicationFabrics/replicationRecoveryServicesProviders@2421-07-01
Fake.RecoveryServices/vaults/replicationProtectionIntents@2418-07-10
Fake.RecoveryServices/vaults/replicationProtectionIntents@2421-02-10
Fake.RecoveryServices/vaults/replicationProtectionIntents@2421-03-01
Fake.RecoveryServices/vaults/replicationProtectionIntents@2421-04-01
Fake.RecoveryServices/vaults/replicationProtectionIntents@2421-06-01
Fake.RecoveryServices/vaults/replicationProtectionIntents@2421-07-01
Fake.RecoveryServices/vaults/replicationVaultSettings@2418-07-10
Fake.RecoveryServices/vaults/replicationVaultSettings@2421-02-10
Fake.RecoveryServices/vaults/replicationVaultSettings@2421-03-01
Fake.RecoveryServices/vaults/replicationVaultSettings@2421-04-01
Fake.RecoveryServices/vaults/replicationVaultSettings@2421-06-01
Fake.RecoveryServices/vaults/replicationVaultSettings@2421-07-01
Fake.RedHatOpenShift/openShiftClusters@2420-04-30
Fake.RedHatOpenShift/openShiftClusters@2421-09-01-preview
Fake.Relay/namespaces@2416-07-01
Fake.Relay/namespaces@2417-04-01
Fake.Relay/namespaces@2418-01-01-preview
Fake.Relay/namespaces/AuthorizationRules@2416-07-01
Fake.Relay/namespaces/AuthorizationRules@2417-04-01
Fake.Relay/namespaces/HybridConnections@2416-07-01
Fake.Relay/namespaces/HybridConnections@2417-04-01
Fake.Relay/namespaces/HybridConnections/authorizationRules@2416-07-01
Fake.Relay/namespaces/HybridConnections/authorizationRules@2417-04-01
Fake.Relay/namespaces/WcfRelays@2416-07-01
Fake.Relay/namespaces/WcfRelays@2417-04-01
Fake.Relay/namespaces/WcfRelays/authorizationRules@2416-07-01
Fake.Relay/namespaces/WcfRelays/authorizationRules@2417-04-01
Fake.Relay/namespaces/networkRuleSets@2418-01-01-preview
Fake.Relay/namespaces/privateEndpointConnections@2418-01-01-preview
Fake.ResourceGraph/queries@2418-09-01-preview
Fake.Resources/templateSpecs@2419-06-01-preview
Fake.Resources/templateSpecs@2421-03-01-preview
Fake.Resources/templateSpecs@2421-05-01
Fake.Resources/templateSpecs/versions@2419-06-01-preview
Fake.Resources/templateSpecs/versions@2421-03-01-preview
Fake.Resources/templateSpecs/versions@2421-05-01
Fake.Resources/tags@2419-10-01
Fake.Resources/tags@2420-06-01
Fake.Resources/tags@2420-08-01
Fake.Resources/tags@2420-10-01
Fake.Resources/tags@2421-01-01
Fake.Resources/tags@2421-04-01
Fake.Resources/deploymentScripts@2419-10-01-preview
Fake.Resources/deploymentScripts@2420-10-01
Fake.Scheduler/jobCollections@2414-08-01-preview
Fake.Scheduler/jobCollections@2416-01-01
Fake.Scheduler/jobCollections@2416-03-01
Fake.Scheduler/jobCollections/jobs@2414-08-01-preview
Fake.Scheduler/jobCollections/jobs@2416-01-01
Fake.Scheduler/jobCollections/jobs@2416-03-01
Fake.Search/searchServices@2415-02-28
Fake.Search/searchServices@2415-08-19
Fake.Search/searchServices@2419-10-01-preview
Fake.Search/searchServices@2420-03-13
Fake.Search/searchServices@2420-08-01
Fake.Search/searchServices@2420-08-01-preview
Fake.Search/searchServices@2421-04-01-preview
Fake.Search/searchServices/privateEndpointConnections@2419-10-01-preview
Fake.Search/searchServices/privateEndpointConnections@2420-03-13
Fake.Search/searchServices/privateEndpointConnections@2420-08-01
Fake.Search/searchServices/privateEndpointConnections@2420-08-01-preview
Fake.Search/searchServices/privateEndpointConnections@2421-04-01-preview
Fake.Search/searchServices/sharedPrivateLinkResources@2420-08-01
Fake.Search/searchServices/sharedPrivateLinkResources@2420-08-01-preview
Fake.Search/searchServices/sharedPrivateLinkResources@2421-04-01-preview
Fake.Security/locations/jitNetworkAccessPolicies@2415-06-01-preview
Fake.Security/locations/jitNetworkAccessPolicies@2420-01-01
Fake.Security/iotSecuritySolutions@2417-08-01-preview
Fake.Security/iotSecuritySolutions@2419-08-01
Fake.Security/pricings@2417-08-01-preview
Fake.Security/advancedThreatProtectionSettings@2417-08-01-preview
Fake.Security/advancedThreatProtectionSettings@2419-01-01
Fake.Security/deviceSecurityGroups@2417-08-01-preview
Fake.Security/deviceSecurityGroups@2419-08-01
Fake.Security/automations@2419-01-01-preview
Fake.Security/assessments@2419-01-01-preview
Fake.Security/assessments@2420-01-01
Fake.Security/assessments@2421-06-01
Fake.ServiceBus/namespaces@2415-08-01
Fake.ServiceBus/namespaces@2417-04-01
Fake.ServiceBus/namespaces@2418-01-01-preview
Fake.ServiceBus/namespaces@2421-01-01-preview
Fake.ServiceBus/namespaces@2421-06-01-preview
Fake.ServiceBus/namespaces@2321-11-01
Fake.ServiceBus/namespaces@2322-01-01-preview
Fake.ServiceBus/namespaces/AuthorizationRules@2415-08-01
Fake.ServiceBus/namespaces/AuthorizationRules@2417-04-01
Fake.ServiceBus/namespaces/AuthorizationRules@2418-01-01-preview
Fake.ServiceBus/namespaces/AuthorizationRules@2421-01-01-preview
Fake.ServiceBus/namespaces/AuthorizationRules@2421-06-01-preview
Fake.ServiceBus/namespaces/queues@2415-08-01
Fake.ServiceBus/namespaces/queues@2417-04-01
Fake.ServiceBus/namespaces/queues@2418-01-01-preview
Fake.ServiceBus/namespaces/queues@2421-01-01-preview
Fake.ServiceBus/namespaces/queues@2421-06-01-preview
Fake.ServiceBus/namespaces/queues/authorizationRules@2415-08-01
Fake.ServiceBus/namespaces/queues/authorizationRules@2417-04-01
Fake.ServiceBus/namespaces/queues/authorizationRules@2418-01-01-preview
Fake.ServiceBus/namespaces/queues/authorizationRules@2421-01-01-preview
Fake.ServiceBus/namespaces/queues/authorizationRules@2421-06-01-preview
Fake.ServiceBus/namespaces/topics@2415-08-01
Fake.ServiceBus/namespaces/topics@2417-04-01
Fake.ServiceBus/namespaces/topics@2418-01-01-preview
Fake.ServiceBus/namespaces/topics@2421-01-01-preview
Fake.ServiceBus/namespaces/topics@2421-06-01-preview
Fake.ServiceBus/namespaces/topics/authorizationRules@2415-08-01
Fake.ServiceBus/namespaces/topics/authorizationRules@2417-04-01
Fake.ServiceBus/namespaces/topics/authorizationRules@2418-01-01-preview
Fake.ServiceBus/namespaces/topics/authorizationRules@2421-01-01-preview
Fake.ServiceBus/namespaces/topics/authorizationRules@2421-06-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions@2415-08-01
Fake.ServiceBus/namespaces/topics/subscriptions@2417-04-01
Fake.ServiceBus/namespaces/topics/subscriptions@2418-01-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions@2421-01-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions@2421-06-01-preview
Fake.ServiceBus/namespaces/disasterRecoveryConfigs@2417-04-01
Fake.ServiceBus/namespaces/disasterRecoveryConfigs@2418-01-01-preview
Fake.ServiceBus/namespaces/disasterRecoveryConfigs@2421-01-01-preview
Fake.ServiceBus/namespaces/disasterRecoveryConfigs@2421-06-01-preview
Fake.ServiceBus/namespaces/migrationConfigurations@2417-04-01
Fake.ServiceBus/namespaces/migrationConfigurations@2418-01-01-preview
Fake.ServiceBus/namespaces/migrationConfigurations@2421-01-01-preview
Fake.ServiceBus/namespaces/migrationConfigurations@2421-06-01-preview
Fake.ServiceBus/namespaces/networkRuleSets@2417-04-01
Fake.ServiceBus/namespaces/networkRuleSets@2418-01-01-preview
Fake.ServiceBus/namespaces/networkRuleSets@2421-01-01-preview
Fake.ServiceBus/namespaces/networkRuleSets@2421-06-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions/rules@2417-04-01
Fake.ServiceBus/namespaces/topics/subscriptions/rules@2418-01-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions/rules@2421-01-01-preview
Fake.ServiceBus/namespaces/topics/subscriptions/rules@2421-06-01-preview
Fake.ServiceBus/namespaces/ipfilterrules@2418-01-01-preview
Fake.ServiceBus/namespaces/privateEndpointConnections@2418-01-01-preview
Fake.ServiceBus/namespaces/privateEndpointConnections@2421-01-01-preview
Fake.ServiceBus/namespaces/privateEndpointConnections@2421-06-01-preview
Fake.ServiceBus/namespaces/virtualnetworkrules@2418-01-01-preview
Fake.ServiceFabric/clusters@2416-09-01
Fake.ServiceFabric/clusters@2417-07-01-preview
Fake.ServiceFabric/clusters@2418-02-01
Fake.ServiceFabric/clusters@2419-03-01
Fake.ServiceFabric/clusters@2419-03-01-preview
Fake.ServiceFabric/clusters@2419-06-01-preview
Fake.ServiceFabric/clusters@2419-11-01-preview
Fake.ServiceFabric/clusters@2420-03-01
Fake.ServiceFabric/clusters@2420-12-01-preview
Fake.ServiceFabric/clusters@2421-06-01
Fake.ServiceFabric/clusters/applications@2417-07-01-preview
Fake.ServiceFabric/clusters/applications@2419-03-01
Fake.ServiceFabric/clusters/applications@2419-03-01-preview
Fake.ServiceFabric/clusters/applications@2419-06-01-preview
Fake.ServiceFabric/clusters/applications@2419-11-01-preview
Fake.ServiceFabric/clusters/applications@2420-03-01
Fake.ServiceFabric/clusters/applications@2420-12-01-preview
Fake.ServiceFabric/clusters/applications@2421-06-01
Fake.ServiceFabric/clusters/applications/services@2417-07-01-preview
Fake.ServiceFabric/clusters/applications/services@2419-03-01
Fake.ServiceFabric/clusters/applications/services@2419-03-01-preview
Fake.ServiceFabric/clusters/applications/services@2419-06-01-preview
Fake.ServiceFabric/clusters/applications/services@2419-11-01-preview
Fake.ServiceFabric/clusters/applications/services@2420-03-01
Fake.ServiceFabric/clusters/applications/services@2420-12-01-preview
Fake.ServiceFabric/clusters/applications/services@2421-06-01
Fake.ServiceFabric/clusters/applicationTypes@2417-07-01-preview
Fake.ServiceFabric/clusters/applicationTypes@2419-03-01
Fake.ServiceFabric/clusters/applicationTypes@2419-03-01-preview
Fake.ServiceFabric/clusters/applicationTypes@2419-06-01-preview
Fake.ServiceFabric/clusters/applicationTypes@2419-11-01-preview
Fake.ServiceFabric/clusters/applicationTypes@2420-03-01
Fake.ServiceFabric/clusters/applicationTypes@2420-12-01-preview
Fake.ServiceFabric/clusters/applicationTypes@2421-06-01
Fake.ServiceFabric/clusters/applicationTypes/versions@2417-07-01-preview
Fake.ServiceFabric/clusters/applicationTypes/versions@2419-03-01
Fake.ServiceFabric/clusters/applicationTypes/versions@2419-03-01-preview
Fake.ServiceFabric/clusters/applicationTypes/versions@2419-06-01-preview
Fake.ServiceFabric/clusters/applicationTypes/versions@2419-11-01-preview
Fake.ServiceFabric/clusters/applicationTypes/versions@2420-03-01
Fake.ServiceFabric/clusters/applicationTypes/versions@2420-12-01-preview
Fake.ServiceFabric/clusters/applicationTypes/versions@2421-06-01
Fake.ServiceFabric/managedClusters@2420-01-01-preview
Fake.ServiceFabric/managedClusters/nodeTypes@2420-01-01-preview
Fake.ServiceFabricMesh/applications@2418-07-01-preview
Fake.ServiceFabricMesh/applications@2418-09-01-preview
Fake.ServiceFabricMesh/networks@2418-07-01-preview
Fake.ServiceFabricMesh/networks@2418-09-01-preview
Fake.ServiceFabricMesh/volumes@2418-07-01-preview
Fake.ServiceFabricMesh/volumes@2418-09-01-preview
Fake.ServiceFabricMesh/gateways@2418-09-01-preview
Fake.ServiceFabricMesh/secrets@2418-09-01-preview
Fake.ServiceFabricMesh/secrets/values@2418-09-01-preview
Fake.SignalRService/SignalR@2418-03-01-preview
Fake.SignalRService/SignalR@2418-10-01
Fake.SignalRService/SignalR@2420-05-01
Fake.SignalRService/SignalR@2420-07-01-preview
Fake.SignalRService/SignalR@2421-04-01-preview
Fake.SignalRService/SignalR@2421-06-01-preview
Fake.SignalRService/SignalR@2421-09-01-preview
Fake.SignalRService/signalR/privateEndpointConnections@2420-05-01
Fake.SignalRService/signalR/privateEndpointConnections@2420-07-01-preview
Fake.SignalRService/signalR/privateEndpointConnections@2421-04-01-preview
Fake.SignalRService/signalR/privateEndpointConnections@2421-06-01-preview
Fake.SignalRService/signalR/privateEndpointConnections@2421-09-01-preview
Fake.SignalRService/signalR/sharedPrivateLinkResources@2421-04-01-preview
Fake.SignalRService/signalR/sharedPrivateLinkResources@2421-06-01-preview
Fake.SignalRService/signalR/sharedPrivateLinkResources@2421-09-01-preview
Fake.Sql/servers@2414-04-01
Fake.Sql/servers@2415-05-01-preview
Fake.Sql/servers@2419-06-01-preview
Fake.Sql/servers@2420-02-02-preview
Fake.Sql/servers@2420-08-01-preview
Fake.Sql/servers@2420-11-01-preview
Fake.Sql/servers@2421-02-01-preview
Fake.Sql/servers/administrators@2414-04-01
Fake.Sql/servers/administrators@2418-06-01-preview
Fake.Sql/servers/administrators@2419-06-01-preview
Fake.Sql/servers/administrators@2420-02-02-preview
Fake.Sql/servers/administrators@2420-08-01-preview
Fake.Sql/servers/administrators@2420-11-01-preview
Fake.Sql/servers/administrators@2421-02-01-preview
Fake.Sql/servers/advisors@2414-04-01
Fake.Sql/servers/auditingPolicies@2414-04-01
Fake.Sql/servers/communicationLinks@2414-04-01
Fake.Sql/servers/connectionPolicies@2414-04-01
Fake.Sql/servers/databases@2414-04-01
Fake.Sql/servers/databases@2417-03-01-preview
Fake.Sql/servers/databases@2417-10-01-preview
Fake.Sql/servers/databases@2419-06-01-preview
Fake.Sql/servers/databases@2420-02-02-preview
Fake.Sql/servers/databases@2420-08-01-preview
Fake.Sql/servers/databases@2420-11-01-preview
Fake.Sql/servers/databases@2421-02-01-preview
Fake.Sql/servers/databases/advisors@2414-04-01
Fake.Sql/servers/databases/auditingPolicies@2414-04-01
Fake.Sql/servers/databases/connectionPolicies@2414-04-01
Fake.Sql/servers/databases/dataMaskingPolicies@2414-04-01
Fake.Sql/servers/databases/dataMaskingPolicies/rules@2414-04-01
Fake.Sql/servers/databases/extensions@2414-04-01
Fake.Sql/servers/databases/extensions@2420-08-01-preview
Fake.Sql/servers/databases/extensions@2420-11-01-preview
Fake.Sql/servers/databases/extensions@2421-02-01-preview
Fake.Sql/servers/databases/geoBackupPolicies@2414-04-01
Fake.Sql/servers/databases/securityAlertPolicies@2414-04-01
Fake.Sql/servers/databases/securityAlertPolicies@2418-06-01-preview
Fake.Sql/servers/databases/securityAlertPolicies@2420-02-02-preview
Fake.Sql/servers/databases/securityAlertPolicies@2420-08-01-preview
Fake.Sql/servers/databases/securityAlertPolicies@2420-11-01-preview
Fake.Sql/servers/databases/securityAlertPolicies@2421-02-01-preview
Fake.Sql/servers/databases/transparentDataEncryption@2414-04-01
Fake.Sql/servers/databases/transparentDataEncryption@2420-02-02-preview
Fake.Sql/servers/databases/transparentDataEncryption@2420-08-01-preview
Fake.Sql/servers/databases/transparentDataEncryption@2420-11-01-preview
Fake.Sql/servers/databases/transparentDataEncryption@2421-02-01-preview
Fake.Sql/servers/disasterRecoveryConfiguration@2414-04-01
Fake.Sql/servers/elasticPools@2414-04-01
Fake.Sql/servers/elasticPools@2417-10-01-preview
Fake.Sql/servers/elasticPools@2420-02-02-preview
Fake.Sql/servers/elasticPools@2420-08-01-preview
Fake.Sql/servers/elasticPools@2420-11-01-preview
Fake.Sql/servers/elasticPools@2421-02-01-preview
Fake.Sql/servers/firewallRules@2414-04-01
Fake.Sql/servers/firewallRules@2415-05-01-preview
Fake.Sql/servers/firewallRules@2420-02-02-preview
Fake.Sql/servers/firewallRules@2420-08-01-preview
Fake.Sql/servers/firewallRules@2420-11-01-preview
Fake.Sql/servers/firewallRules@2421-02-01-preview
Fake.Sql/managedInstances@2415-05-01-preview
Fake.Sql/managedInstances@2418-06-01-preview
Fake.Sql/managedInstances@2420-02-02-preview
Fake.Sql/managedInstances@2420-08-01-preview
Fake.Sql/managedInstances@2420-11-01-preview
Fake.Sql/managedInstances@2421-02-01-preview
Fake.Sql/servers/databases/auditingSettings@2415-05-01-preview
Fake.Sql/servers/databases/auditingSettings@2417-03-01-preview
Fake.Sql/servers/databases/auditingSettings@2420-02-02-preview
Fake.Sql/servers/databases/auditingSettings@2420-08-01-preview
Fake.Sql/servers/databases/auditingSettings@2420-11-01-preview
Fake.Sql/servers/databases/auditingSettings@2421-02-01-preview
Fake.Sql/servers/databases/syncGroups@2415-05-01-preview
Fake.Sql/servers/databases/syncGroups@2419-06-01-preview
Fake.Sql/servers/databases/syncGroups@2420-02-02-preview
Fake.Sql/servers/databases/syncGroups@2420-08-01-preview
Fake.Sql/servers/databases/syncGroups@2420-11-01-preview
Fake.Sql/servers/databases/syncGroups@2421-02-01-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2415-05-01-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2419-06-01-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2420-02-02-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2420-08-01-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2420-11-01-preview
Fake.Sql/servers/databases/syncGroups/syncMembers@2421-02-01-preview
Fake.Sql/servers/encryptionProtector@2415-05-01-preview
Fake.Sql/servers/encryptionProtector@2420-02-02-preview
Fake.Sql/servers/encryptionProtector@2420-08-01-preview
Fake.Sql/servers/encryptionProtector@2420-11-01-preview
Fake.Sql/servers/encryptionProtector@2421-02-01-preview
Fake.Sql/servers/failoverGroups@2415-05-01-preview
Fake.Sql/servers/failoverGroups@2420-02-02-preview
Fake.Sql/servers/failoverGroups@2420-08-01-preview
Fake.Sql/servers/failoverGroups@2420-11-01-preview
Fake.Sql/servers/failoverGroups@2421-02-01-preview
Fake.Sql/servers/keys@2415-05-01-preview
Fake.Sql/servers/keys@2420-02-02-preview
Fake.Sql/servers/keys@2420-08-01-preview
Fake.Sql/servers/keys@2420-11-01-preview
Fake.Sql/servers/keys@2421-02-01-preview
Fake.Sql/servers/syncAgents@2415-05-01-preview
Fake.Sql/servers/syncAgents@2420-02-02-preview
Fake.Sql/servers/syncAgents@2420-08-01-preview
Fake.Sql/servers/syncAgents@2420-11-01-preview
Fake.Sql/servers/syncAgents@2421-02-01-preview
Fake.Sql/servers/virtualNetworkRules@2415-05-01-preview
Fake.Sql/servers/virtualNetworkRules@2420-02-02-preview
Fake.Sql/servers/virtualNetworkRules@2420-08-01-preview
Fake.Sql/servers/virtualNetworkRules@2420-11-01-preview
Fake.Sql/servers/virtualNetworkRules@2421-02-01-preview
Fake.Sql/managedInstances/administrators@2417-03-01-preview
Fake.Sql/managedInstances/administrators@2420-02-02-preview
Fake.Sql/managedInstances/administrators@2420-08-01-preview
Fake.Sql/managedInstances/administrators@2420-11-01-preview
Fake.Sql/managedInstances/administrators@2421-02-01-preview
Fake.Sql/managedInstances/databases@2417-03-01-preview
Fake.Sql/managedInstances/databases@2418-06-01-preview
Fake.Sql/managedInstances/databases@2419-06-01-preview
Fake.Sql/managedInstances/databases@2420-02-02-preview
Fake.Sql/managedInstances/databases@2420-08-01-preview
Fake.Sql/managedInstances/databases@2420-11-01-preview
Fake.Sql/managedInstances/databases@2421-02-01-preview
Fake.Sql/managedInstances/databases/backupShortTermRetentionPolicies@2417-03-01-preview
Fake.Sql/managedInstances/databases/backupShortTermRetentionPolicies@2420-02-02-preview
Fake.Sql/managedInstances/databases/backupShortTermRetentionPolicies@2420-08-01-preview
Fake.Sql/managedInstances/databases/backupShortTermRetentionPolicies@2420-11-01-preview
Fake.Sql/managedInstances/databases/backupShortTermRetentionPolicies@2421-02-01-preview
Fake.Sql/managedInstances/databases/securityAlertPolicies@2417-03-01-preview
Fake.Sql/managedInstances/databases/securityAlertPolicies@2420-02-02-preview
Fake.Sql/managedInstances/databases/securityAlertPolicies@2420-08-01-preview
Fake.Sql/managedInstances/databases/securityAlertPolicies@2420-11-01-preview
Fake.Sql/managedInstances/databases/securityAlertPolicies@2421-02-01-preview
Fake.Sql/managedInstances/restorableDroppedDatabases/backupShortTermRetentionPolicies@2417-03-01-preview
Fake.Sql/managedInstances/restorableDroppedDatabases/backupShortTermRetentionPolicies@2420-02-02-preview
Fake.Sql/managedInstances/restorableDroppedDatabases/backupShortTermRetentionPolicies@2420-08-01-preview
Fake.Sql/managedInstances/restorableDroppedDatabases/backupShortTermRetentionPolicies@2420-11-01-preview
Fake.Sql/managedInstances/restorableDroppedDatabases/backupShortTermRetentionPolicies@2421-02-01-preview
Fake.Sql/managedInstances/securityAlertPolicies@2417-03-01-preview
Fake.Sql/managedInstances/securityAlertPolicies@2420-02-02-preview
Fake.Sql/managedInstances/securityAlertPolicies@2420-08-01-preview
Fake.Sql/managedInstances/securityAlertPolicies@2420-11-01-preview
Fake.Sql/managedInstances/securityAlertPolicies@2421-02-01-preview
Fake.Sql/servers/auditingSettings@2417-03-01-preview
Fake.Sql/servers/auditingSettings@2420-02-02-preview
Fake.Sql/servers/auditingSettings@2420-08-01-preview
Fake.Sql/servers/auditingSettings@2420-11-01-preview
Fake.Sql/servers/auditingSettings@2421-02-01-preview
Fake.Sql/servers/databases/backupLongTermRetentionPolicies@2417-03-01-preview
Fake.Sql/servers/databases/backupLongTermRetentionPolicies@2420-02-02-preview
Fake.Sql/servers/databases/backupLongTermRetentionPolicies@2420-08-01-preview
Fake.Sql/servers/databases/backupLongTermRetentionPolicies@2420-11-01-preview
Fake.Sql/servers/databases/backupLongTermRetentionPolicies@2421-02-01-preview
Fake.Sql/servers/databases/extendedAuditingSettings@2417-03-01-preview
Fake.Sql/servers/databases/extendedAuditingSettings@2420-02-02-preview
Fake.Sql/servers/databases/extendedAuditingSettings@2420-08-01-preview
Fake.Sql/servers/databases/extendedAuditingSettings@2420-11-01-preview
Fake.Sql/servers/databases/extendedAuditingSettings@2421-02-01-preview
Fake.Sql/servers/databases/schemas/tables/columns/sensitivityLabels@2417-03-01-preview
Fake.Sql/servers/databases/schemas/tables/columns/sensitivityLabels@2420-02-02-preview
Fake.Sql/servers/databases/schemas/tables/columns/sensitivityLabels@2420-08-01-preview
Fake.Sql/servers/databases/schemas/tables/columns/sensitivityLabels@2420-11-01-preview
Fake.Sql/servers/databases/schemas/tables/columns/sensitivityLabels@2421-02-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments@2417-03-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments@2420-02-02-preview
Fake.Sql/servers/databases/vulnerabilityAssessments@2420-08-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments@2420-11-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments@2421-02-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments/rules/baselines@2417-03-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments/rules/baselines@2420-02-02-preview
Fake.Sql/servers/databases/vulnerabilityAssessments/rules/baselines@2420-08-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments/rules/baselines@2420-11-01-preview
Fake.Sql/servers/databases/vulnerabilityAssessments/rules/baselines@2421-02-01-preview
Fake.Sql/servers/dnsAliases@2417-03-01-preview
Fake.Sql/servers/dnsAliases@2420-02-02-preview
Fake.Sql/servers/dnsAliases@2420-08-01-preview
Fake.Sql/servers/dnsAliases@2420-11-01-preview
Fake.Sql/servers/dnsAliases@2421-02-01-preview
Fake.Sql/servers/extendedAuditingSettings@2417-03-01-preview
Fake.Sql/servers/extendedAuditingSettings@2420-02-02-preview
Fake.Sql/servers/extendedAuditingSettings@2420-08-01-preview
Fake.Sql/servers/extendedAuditingSettings@2420-11-01-preview
Fake.Sql/servers/extendedAuditingSettings@2421-02-01-preview
Fake.Sql/servers/jobAgents@2417-03-01-preview
Fake.Sql/servers/jobAgents@2420-02-02-preview
Fake.Sql/servers/jobAgents@2420-08-01-preview
Fake.Sql/servers/jobAgents@2420-11-01-preview
Fake.Sql/servers/jobAgents@2421-02-01-preview
Fake.Sql/servers/jobAgents/credentials@2417-03-01-preview
Fake.Sql/servers/jobAgents/credentials@2420-02-02-preview
Fake.Sql/servers/jobAgents/credentials@2420-08-01-preview
Fake.Sql/servers/jobAgents/credentials@2420-11-01-preview
Fake.Sql/servers/jobAgents/credentials@2421-02-01-preview
Fake.Sql/servers/jobAgents/jobs@2417-03-01-preview
Fake.Sql/servers/jobAgents/jobs@2420-02-02-preview
Fake.Sql/servers/jobAgents/jobs@2420-08-01-preview
Fake.Sql/servers/jobAgents/jobs@2420-11-01-preview
Fake.Sql/servers/jobAgents/jobs@2421-02-01-preview
Fake.Sql/servers/jobAgents/jobs/executions@2417-03-01-preview
Fake.Sql/servers/jobAgents/jobs/executions@2420-02-02-preview
Fake.Sql/servers/jobAgents/jobs/executions@2420-08-01-preview
Fake.Sql/servers/jobAgents/jobs/executions@2420-11-01-preview
Fake.Sql/servers/jobAgents/jobs/executions@2421-02-01-preview
Fake.Sql/servers/jobAgents/jobs/steps@2417-03-01-preview
Fake.Sql/servers/jobAgents/jobs/steps@2420-02-02-preview
Fake.Sql/servers/jobAgents/jobs/steps@2420-08-01-preview
Fake.Sql/servers/jobAgents/jobs/steps@2420-11-01-preview
Fake.Sql/servers/jobAgents/jobs/steps@2421-02-01-preview
Fake.Sql/servers/jobAgents/targetGroups@2417-03-01-preview
Fake.Sql/servers/jobAgents/targetGroups@2420-02-02-preview
Fake.Sql/servers/jobAgents/targetGroups@2420-08-01-preview
Fake.Sql/servers/jobAgents/targetGroups@2420-11-01-preview
Fake.Sql/servers/jobAgents/targetGroups@2421-02-01-preview
Fake.Sql/servers/securityAlertPolicies@2417-03-01-preview
Fake.Sql/servers/securityAlertPolicies@2420-02-02-preview
Fake.Sql/servers/securityAlertPolicies@2420-08-01-preview
Fake.Sql/servers/securityAlertPolicies@2420-11-01-preview
Fake.Sql/servers/securityAlertPolicies@2421-02-01-preview
Fake.Sql/locations/instanceFailoverGroups@2417-10-01-preview
Fake.Sql/locations/instanceFailoverGroups@2420-02-02-preview
Fake.Sql/locations/instanceFailoverGroups@2420-08-01-preview
Fake.Sql/locations/instanceFailoverGroups@2420-11-01-preview
Fake.Sql/locations/instanceFailoverGroups@2421-02-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments@2417-10-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments@2420-02-02-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments@2420-08-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments@2420-11-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments@2421-02-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments/rules/baselines@2417-10-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments/rules/baselines@2420-02-02-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments/rules/baselines@2420-08-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments/rules/baselines@2420-11-01-preview
Fake.Sql/managedInstances/databases/vulnerabilityAssessments/rules/baselines@2421-02-01-preview
Fake.Sql/managedInstances/encryptionProtector@2417-10-01-preview
Fake.Sql/managedInstances/encryptionProtector@2420-02-02-preview
Fake.Sql/managedInstances/encryptionProtector@2420-08-01-preview
Fake.Sql/managedInstances/encryptionProtector@2420-11-01-preview
Fake.Sql/managedInstances/encryptionProtector@2421-02-01-preview
Fake.Sql/managedInstances/keys@2417-10-01-preview
Fake.Sql/managedInstances/keys@2420-02-02-preview
Fake.Sql/managedInstances/keys@2420-08-01-preview
Fake.Sql/managedInstances/keys@2420-11-01-preview
Fake.Sql/managedInstances/keys@2421-02-01-preview
Fake.Sql/servers/databases/backupShortTermRetentionPolicies@2417-10-01-preview
Fake.Sql/servers/databases/backupShortTermRetentionPolicies@2420-02-02-preview
Fake.Sql/servers/databases/backupShortTermRetentionPolicies@2420-08-01-preview
Fake.Sql/servers/databases/backupShortTermRetentionPolicies@2420-11-01-preview
Fake.Sql/servers/databases/backupShortTermRetentionPolicies@2421-02-01-preview
Fake.Sql/instancePools@2418-06-01-preview
Fake.Sql/instancePools@2420-02-02-preview
Fake.Sql/instancePools@2420-08-01-preview
Fake.Sql/instancePools@2420-11-01-preview
Fake.Sql/instancePools@2421-02-01-preview
Fake.Sql/managedInstances/databases/backupLongTermRetentionPolicies@2418-06-01-preview
Fake.Sql/managedInstances/databases/backupLongTermRetentionPolicies@2420-02-02-preview
Fake.Sql/managedInstances/databases/backupLongTermRetentionPolicies@2420-08-01-preview
Fake.Sql/managedInstances/databases/backupLongTermRetentionPolicies@2420-11-01-preview
Fake.Sql/managedInstances/databases/backupLongTermRetentionPolicies@2421-02-01-preview
Fake.Sql/managedInstances/databases/schemas/tables/columns/sensitivityLabels@2418-06-01-preview
Fake.Sql/managedInstances/databases/schemas/tables/columns/sensitivityLabels@2420-02-02-preview
Fake.Sql/managedInstances/databases/schemas/tables/columns/sensitivityLabels@2420-08-01-preview
Fake.Sql/managedInstances/databases/schemas/tables/columns/sensitivityLabels@2420-11-01-preview
Fake.Sql/managedInstances/databases/schemas/tables/columns/sensitivityLabels@2421-02-01-preview
Fake.Sql/managedInstances/vulnerabilityAssessments@2418-06-01-preview
Fake.Sql/managedInstances/vulnerabilityAssessments@2420-02-02-preview
Fake.Sql/managedInstances/vulnerabilityAssessments@2420-08-01-preview
Fake.Sql/managedInstances/vulnerabilityAssessments@2420-11-01-preview
Fake.Sql/managedInstances/vulnerabilityAssessments@2421-02-01-preview
Fake.Sql/servers/privateEndpointConnections@2418-06-01-preview
Fake.Sql/servers/privateEndpointConnections@2420-02-02-preview
Fake.Sql/servers/privateEndpointConnections@2420-08-01-preview
Fake.Sql/servers/privateEndpointConnections@2420-11-01-preview
Fake.Sql/servers/privateEndpointConnections@2421-02-01-preview
Fake.Sql/servers/vulnerabilityAssessments@2418-06-01-preview
Fake.Sql/servers/vulnerabilityAssessments@2420-02-02-preview
Fake.Sql/servers/vulnerabilityAssessments@2420-08-01-preview
Fake.Sql/servers/vulnerabilityAssessments@2420-11-01-preview
Fake.Sql/servers/vulnerabilityAssessments@2421-02-01-preview
Fake.Sql/servers/databases/workloadGroups@2419-06-01-preview
Fake.Sql/servers/databases/workloadGroups@2420-02-02-preview
Fake.Sql/servers/databases/workloadGroups@2420-08-01-preview
Fake.Sql/servers/databases/workloadGroups@2420-11-01-preview
Fake.Sql/servers/databases/workloadGroups@2421-02-01-preview
Fake.Sql/servers/databases/workloadGroups/workloadClassifiers@2419-06-01-preview
Fake.Sql/servers/databases/workloadGroups/workloadClassifiers@2420-02-02-preview
Fake.Sql/servers/databases/workloadGroups/workloadClassifiers@2420-08-01-preview
Fake.Sql/servers/databases/workloadGroups/workloadClassifiers@2420-11-01-preview
Fake.Sql/servers/databases/workloadGroups/workloadClassifiers@2421-02-01-preview
Fake.Sql/locations/serverTrustGroups@2420-02-02-preview
Fake.Sql/locations/serverTrustGroups@2420-08-01-preview
Fake.Sql/locations/serverTrustGroups@2420-11-01-preview
Fake.Sql/locations/serverTrustGroups@2421-02-01-preview
Fake.Sql/managedInstances/azureADOnlyAuthentications@2420-02-02-preview
Fake.Sql/managedInstances/azureADOnlyAuthentications@2420-08-01-preview
Fake.Sql/managedInstances/azureADOnlyAuthentications@2420-11-01-preview
Fake.Sql/managedInstances/azureADOnlyAuthentications@2421-02-01-preview
Fake.Sql/managedInstances/databases/transparentDataEncryption@2420-02-02-preview
Fake.Sql/managedInstances/databases/transparentDataEncryption@2420-08-01-preview
Fake.Sql/managedInstances/databases/transparentDataEncryption@2420-11-01-preview
Fake.Sql/managedInstances/databases/transparentDataEncryption@2421-02-01-preview
Fake.Sql/managedInstances/privateEndpointConnections@2420-02-02-preview
Fake.Sql/managedInstances/privateEndpointConnections@2420-08-01-preview
Fake.Sql/managedInstances/privateEndpointConnections@2420-11-01-preview
Fake.Sql/managedInstances/privateEndpointConnections@2421-02-01-preview
Fake.Sql/managedInstances/sqlAgent@2420-02-02-preview
Fake.Sql/managedInstances/sqlAgent@2420-08-01-preview
Fake.Sql/managedInstances/sqlAgent@2420-11-01-preview
Fake.Sql/managedInstances/sqlAgent@2421-02-01-preview
Fake.Sql/servers/azureADOnlyAuthentications@2420-02-02-preview
Fake.Sql/servers/azureADOnlyAuthentications@2420-08-01-preview
Fake.Sql/servers/azureADOnlyAuthentications@2420-11-01-preview
Fake.Sql/servers/azureADOnlyAuthentications@2421-02-01-preview
Fake.Sql/servers/databases/maintenanceWindows@2420-02-02-preview
Fake.Sql/servers/databases/maintenanceWindows@2420-08-01-preview
Fake.Sql/servers/databases/maintenanceWindows@2420-11-01-preview
Fake.Sql/servers/databases/maintenanceWindows@2421-02-01-preview
Fake.Sql/servers/devOpsAuditingSettings@2420-02-02-preview
Fake.Sql/servers/devOpsAuditingSettings@2420-08-01-preview
Fake.Sql/servers/devOpsAuditingSettings@2420-11-01-preview
Fake.Sql/servers/devOpsAuditingSettings@2421-02-01-preview
Fake.Sql/servers/databases/ledgerDigestUploads@2421-02-01-preview
Fake.Sql/servers/outboundFirewallRules@2421-02-01-preview
Fake.SqlVirtualMachine/sqlVirtualMachineGroups@2417-03-01-preview
Fake.SqlVirtualMachine/sqlVirtualMachineGroups/availabilityGroupListeners@2417-03-01-preview
Fake.SqlVirtualMachine/sqlVirtualMachines@2417-03-01-preview
Fake.Storage/storageAccounts@2415-05-01-preview
Fake.Storage/storageAccounts@2415-06-15
Fake.Storage/storageAccounts@2416-01-01
Fake.Storage/storageAccounts@2416-05-01
Fake.Storage/storageAccounts@2416-12-01
Fake.Storage/storageAccounts@2417-06-01
Fake.Storage/storageAccounts@2417-10-01
Fake.Storage/storageAccounts@2418-02-01
Fake.Storage/storageAccounts@2418-03-01-preview
Fake.Storage/storageAccounts@2418-07-01
Fake.Storage/storageAccounts@2418-11-01
Fake.Storage/storageAccounts@2419-04-01
Fake.Storage/storageAccounts@2419-06-01
Fake.Storage/storageAccounts@2420-08-01-preview
Fake.Storage/storageAccounts@2421-01-01
Fake.Storage/storageAccounts@2421-02-01
Fake.Storage/storageAccounts@2421-04-01
Fake.Storage/storageAccounts@2421-06-01
Fake.Storage/storageAccounts/blobServices/containers@2418-02-01
Fake.Storage/storageAccounts/blobServices/containers@2418-03-01-preview
Fake.Storage/storageAccounts/blobServices/containers@2418-07-01
Fake.Storage/storageAccounts/blobServices/containers@2418-11-01
Fake.Storage/storageAccounts/blobServices/containers@2419-04-01
Fake.Storage/storageAccounts/blobServices/containers@2419-06-01
Fake.Storage/storageAccounts/blobServices/containers@2420-08-01-preview
Fake.Storage/storageAccounts/blobServices/containers@2421-01-01
Fake.Storage/storageAccounts/blobServices/containers@2421-02-01
Fake.Storage/storageAccounts/blobServices/containers@2421-04-01
Fake.Storage/storageAccounts/blobServices/containers@2421-06-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2418-02-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2418-03-01-preview
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2418-07-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2418-11-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2419-04-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2419-06-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2420-08-01-preview
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2421-01-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2421-02-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2421-04-01
Fake.Storage/storageAccounts/blobServices/containers/immutabilityPolicies@2421-06-01
Fake.Storage/storageAccounts/managementPolicies@2418-03-01-preview
Fake.Storage/storageAccounts/managementPolicies@2418-11-01
Fake.Storage/storageAccounts/managementPolicies@2419-04-01
Fake.Storage/storageAccounts/managementPolicies@2419-06-01
Fake.Storage/storageAccounts/managementPolicies@2420-08-01-preview
Fake.Storage/storageAccounts/managementPolicies@2421-01-01
Fake.Storage/storageAccounts/managementPolicies@2421-02-01
Fake.Storage/storageAccounts/managementPolicies@2421-04-01
Fake.Storage/storageAccounts/managementPolicies@2421-06-01
Fake.Storage/storageAccounts/blobServices@2418-07-01
Fake.Storage/storageAccounts/blobServices@2418-11-01
Fake.Storage/storageAccounts/blobServices@2419-04-01
Fake.Storage/storageAccounts/blobServices@2419-06-01
Fake.Storage/storageAccounts/blobServices@2420-08-01-preview
Fake.Storage/storageAccounts/blobServices@2421-01-01
Fake.Storage/storageAccounts/blobServices@2421-02-01
Fake.Storage/storageAccounts/blobServices@2421-04-01
Fake.Storage/storageAccounts/blobServices@2421-06-01
Fake.Storage/storageAccounts/fileServices@2419-04-01
Fake.Storage/storageAccounts/fileServices@2419-06-01
Fake.Storage/storageAccounts/fileServices@2420-08-01-preview
Fake.Storage/storageAccounts/fileServices@2421-01-01
Fake.Storage/storageAccounts/fileServices@2421-02-01
Fake.Storage/storageAccounts/fileServices@2421-04-01
Fake.Storage/storageAccounts/fileServices@2421-06-01
Fake.Storage/storageAccounts/fileServices/shares@2419-04-01
Fake.Storage/storageAccounts/fileServices/shares@2419-06-01
Fake.Storage/storageAccounts/fileServices/shares@2420-08-01-preview
Fake.Storage/storageAccounts/fileServices/shares@2421-01-01
Fake.Storage/storageAccounts/fileServices/shares@2421-02-01
Fake.Storage/storageAccounts/fileServices/shares@2421-04-01
Fake.Storage/storageAccounts/fileServices/shares@2421-06-01
Fake.Storage/storageAccounts/encryptionScopes@2419-06-01
Fake.Storage/storageAccounts/encryptionScopes@2420-08-01-preview
Fake.Storage/storageAccounts/encryptionScopes@2421-01-01
Fake.Storage/storageAccounts/encryptionScopes@2421-02-01
Fake.Storage/storageAccounts/encryptionScopes@2421-04-01
Fake.Storage/storageAccounts/encryptionScopes@2421-06-01
Fake.Storage/storageAccounts/inventoryPolicies@2419-06-01
Fake.Storage/storageAccounts/inventoryPolicies@2420-08-01-preview
Fake.Storage/storageAccounts/inventoryPolicies@2421-01-01
Fake.Storage/storageAccounts/inventoryPolicies@2421-02-01
Fake.Storage/storageAccounts/inventoryPolicies@2421-04-01
Fake.Storage/storageAccounts/inventoryPolicies@2421-06-01
Fake.Storage/storageAccounts/objectReplicationPolicies@2419-06-01
Fake.Storage/storageAccounts/objectReplicationPolicies@2420-08-01-preview
Fake.Storage/storageAccounts/objectReplicationPolicies@2421-01-01
Fake.Storage/storageAccounts/objectReplicationPolicies@2421-02-01
Fake.Storage/storageAccounts/objectReplicationPolicies@2421-04-01
Fake.Storage/storageAccounts/objectReplicationPolicies@2421-06-01
Fake.Storage/storageAccounts/privateEndpointConnections@2419-06-01
Fake.Storage/storageAccounts/privateEndpointConnections@2420-08-01-preview
Fake.Storage/storageAccounts/privateEndpointConnections@2421-01-01
Fake.Storage/storageAccounts/privateEndpointConnections@2421-02-01
Fake.Storage/storageAccounts/privateEndpointConnections@2421-04-01
Fake.Storage/storageAccounts/privateEndpointConnections@2421-06-01
Fake.Storage/storageAccounts/queueServices@2419-06-01
Fake.Storage/storageAccounts/queueServices@2420-08-01-preview
Fake.Storage/storageAccounts/queueServices@2421-01-01
Fake.Storage/storageAccounts/queueServices@2421-02-01
Fake.Storage/storageAccounts/queueServices@2421-04-01
Fake.Storage/storageAccounts/queueServices@2421-06-01
Fake.Storage/storageAccounts/queueServices/queues@2419-06-01
Fake.Storage/storageAccounts/queueServices/queues@2420-08-01-preview
Fake.Storage/storageAccounts/queueServices/queues@2421-01-01
Fake.Storage/storageAccounts/queueServices/queues@2421-02-01
Fake.Storage/storageAccounts/queueServices/queues@2421-04-01
Fake.Storage/storageAccounts/queueServices/queues@2421-06-01
Fake.Storage/storageAccounts/tableServices@2419-06-01
Fake.Storage/storageAccounts/tableServices@2420-08-01-preview
Fake.Storage/storageAccounts/tableServices@2421-01-01
Fake.Storage/storageAccounts/tableServices@2421-02-01
Fake.Storage/storageAccounts/tableServices@2421-04-01
Fake.Storage/storageAccounts/tableServices@2421-06-01
Fake.Storage/storageAccounts/tableServices/tables@2419-06-01
Fake.Storage/storageAccounts/tableServices/tables@2420-08-01-preview
Fake.Storage/storageAccounts/tableServices/tables@2421-01-01
Fake.Storage/storageAccounts/tableServices/tables@2421-02-01
Fake.Storage/storageAccounts/tableServices/tables@2421-04-01
Fake.Storage/storageAccounts/tableServices/tables@2421-06-01
Fake.StorageCache/caches@2419-08-01-preview
Fake.StorageCache/caches@2419-11-01
Fake.StorageCache/caches@2420-03-01
Fake.StorageCache/caches@2420-10-01
Fake.StorageCache/caches@2421-03-01
Fake.StorageCache/caches@2421-05-01
Fake.StorageCache/caches/storageTargets@2419-08-01-preview
Fake.StorageCache/caches/storageTargets@2419-11-01
Fake.StorageCache/caches/storageTargets@2420-03-01
Fake.StorageCache/caches/storageTargets@2420-10-01
Fake.StorageCache/caches/storageTargets@2421-03-01
Fake.StorageCache/caches/storageTargets@2421-05-01
Fake.StoragePool/diskPools@2420-03-15-preview
Fake.StoragePool/diskPools@2421-04-01-preview
Fake.StoragePool/diskPools@2421-08-01
Fake.StoragePool/diskPools/iscsiTargets@2420-03-15-preview
Fake.StoragePool/diskPools/iscsiTargets@2421-04-01-preview
Fake.StoragePool/diskPools/iscsiTargets@2421-08-01
Fake.StorageSync/storageSyncServices@2417-06-05-preview
Fake.StorageSync/storageSyncServices@2418-04-02
Fake.StorageSync/storageSyncServices@2418-07-01
Fake.StorageSync/storageSyncServices@2418-10-01
Fake.StorageSync/storageSyncServices@2419-02-01
Fake.StorageSync/storageSyncServices@2419-03-01
Fake.StorageSync/storageSyncServices@2419-06-01
Fake.StorageSync/storageSyncServices@2419-10-01
Fake.StorageSync/storageSyncServices@2420-03-01
Fake.StorageSync/storageSyncServices@2420-09-01
Fake.StorageSync/storageSyncServices/registeredServers@2417-06-05-preview
Fake.StorageSync/storageSyncServices/registeredServers@2418-04-02
Fake.StorageSync/storageSyncServices/registeredServers@2418-07-01
Fake.StorageSync/storageSyncServices/registeredServers@2418-10-01
Fake.StorageSync/storageSyncServices/registeredServers@2419-02-01
Fake.StorageSync/storageSyncServices/registeredServers@2419-03-01
Fake.StorageSync/storageSyncServices/registeredServers@2419-06-01
Fake.StorageSync/storageSyncServices/registeredServers@2419-10-01
Fake.StorageSync/storageSyncServices/registeredServers@2420-03-01
Fake.StorageSync/storageSyncServices/registeredServers@2420-09-01
Fake.StorageSync/storageSyncServices/syncGroups@2417-06-05-preview
Fake.StorageSync/storageSyncServices/syncGroups@2418-04-02
Fake.StorageSync/storageSyncServices/syncGroups@2418-07-01
Fake.StorageSync/storageSyncServices/syncGroups@2418-10-01
Fake.StorageSync/storageSyncServices/syncGroups@2419-02-01
Fake.StorageSync/storageSyncServices/syncGroups@2419-03-01
Fake.StorageSync/storageSyncServices/syncGroups@2419-06-01
Fake.StorageSync/storageSyncServices/syncGroups@2419-10-01
Fake.StorageSync/storageSyncServices/syncGroups@2420-03-01
Fake.StorageSync/storageSyncServices/syncGroups@2420-09-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2417-06-05-preview
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2418-04-02
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2418-07-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2418-10-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2419-02-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2419-03-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2419-06-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2419-10-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2420-03-01
Fake.StorageSync/storageSyncServices/syncGroups/cloudEndpoints@2420-09-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2417-06-05-preview
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2418-04-02
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2418-07-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2418-10-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2419-02-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2419-03-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2419-06-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2419-10-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2420-03-01
Fake.StorageSync/storageSyncServices/syncGroups/serverEndpoints@2420-09-01
Fake.StorageSync/storageSyncServices/privateEndpointConnections@2420-03-01
Fake.StorageSync/storageSyncServices/privateEndpointConnections@2420-09-01
Fake.StorSimple/managers@2416-10-01
Fake.StorSimple/managers@2417-06-01
Fake.StorSimple/managers/accessControlRecords@2416-10-01
Fake.StorSimple/managers/accessControlRecords@2417-06-01
Fake.StorSimple/managers/certificates@2416-10-01
Fake.StorSimple/managers/devices/alertSettings@2416-10-01
Fake.StorSimple/managers/devices/alertSettings@2417-06-01
Fake.StorSimple/managers/devices/backupScheduleGroups@2416-10-01
Fake.StorSimple/managers/devices/chapSettings@2416-10-01
Fake.StorSimple/managers/devices/fileservers@2416-10-01
Fake.StorSimple/managers/devices/fileservers/shares@2416-10-01
Fake.StorSimple/managers/devices/iscsiservers@2416-10-01
Fake.StorSimple/managers/devices/iscsiservers/disks@2416-10-01
Fake.StorSimple/managers/extendedInformation@2416-10-01
Fake.StorSimple/managers/extendedInformation@2417-06-01
Fake.StorSimple/managers/storageAccountCredentials@2416-10-01
Fake.StorSimple/managers/storageAccountCredentials@2417-06-01
Fake.StorSimple/managers/storageDomains@2416-10-01
Fake.StorSimple/managers/bandwidthSettings@2417-06-01
Fake.StorSimple/managers/devices/backupPolicies@2417-06-01
Fake.StorSimple/managers/devices/backupPolicies/schedules@2417-06-01
Fake.StorSimple/managers/devices/timeSettings@2417-06-01
Fake.StorSimple/managers/devices/volumeContainers@2417-06-01
Fake.StorSimple/managers/devices/volumeContainers/volumes@2417-06-01
Fake.StreamAnalytics/streamingjobs@2416-03-01
Fake.StreamAnalytics/streamingjobs@2417-04-01-preview
Fake.StreamAnalytics/streamingjobs@2420-03-01
Fake.StreamAnalytics/streamingjobs/functions@2416-03-01
Fake.StreamAnalytics/streamingjobs/functions@2417-04-01-preview
Fake.StreamAnalytics/streamingjobs/functions@2420-03-01
Fake.StreamAnalytics/streamingjobs/inputs@2416-03-01
Fake.StreamAnalytics/streamingjobs/inputs@2417-04-01-preview
Fake.StreamAnalytics/streamingjobs/inputs@2420-03-01
Fake.StreamAnalytics/streamingjobs/outputs@2416-03-01
Fake.StreamAnalytics/streamingjobs/outputs@2417-04-01-preview
Fake.StreamAnalytics/streamingjobs/outputs@2420-03-01
Fake.StreamAnalytics/streamingjobs/transformations@2416-03-01
Fake.StreamAnalytics/streamingjobs/transformations@2417-04-01-preview
Fake.StreamAnalytics/streamingjobs/transformations@2420-03-01
Fake.StreamAnalytics/clusters@2420-03-01-preview
Fake.StreamAnalytics/clusters/privateEndpoints@2420-03-01-preview
Fake.Synapse/privateLinkHubs@2419-06-01-preview
Fake.Synapse/privateLinkHubs@2420-12-01
Fake.Synapse/privateLinkHubs@2421-03-01
Fake.Synapse/privateLinkHubs@2421-04-01-preview
Fake.Synapse/privateLinkHubs@2421-05-01
Fake.Synapse/privateLinkHubs@2421-06-01
Fake.Synapse/privateLinkHubs@2421-06-01-preview
Fake.Synapse/workspaces@2419-06-01-preview
Fake.Synapse/workspaces@2420-12-01
Fake.Synapse/workspaces@2421-03-01
Fake.Synapse/workspaces@2421-04-01-preview
Fake.Synapse/workspaces@2421-05-01
Fake.Synapse/workspaces@2421-06-01
Fake.Synapse/workspaces@2421-06-01-preview
Fake.Synapse/workspaces/administrators@2419-06-01-preview
Fake.Synapse/workspaces/administrators@2420-12-01
Fake.Synapse/workspaces/administrators@2421-03-01
Fake.Synapse/workspaces/administrators@2421-04-01-preview
Fake.Synapse/workspaces/administrators@2421-05-01
Fake.Synapse/workspaces/administrators@2421-06-01
Fake.Synapse/workspaces/administrators@2421-06-01-preview
Fake.Synapse/workspaces/auditingSettings@2419-06-01-preview
Fake.Synapse/workspaces/auditingSettings@2420-12-01
Fake.Synapse/workspaces/auditingSettings@2421-03-01
Fake.Synapse/workspaces/auditingSettings@2421-04-01-preview
Fake.Synapse/workspaces/auditingSettings@2421-05-01
Fake.Synapse/workspaces/auditingSettings@2421-06-01
Fake.Synapse/workspaces/auditingSettings@2421-06-01-preview
Fake.Synapse/workspaces/bigDataPools@2419-06-01-preview
Fake.Synapse/workspaces/bigDataPools@2420-12-01
Fake.Synapse/workspaces/bigDataPools@2421-03-01
Fake.Synapse/workspaces/bigDataPools@2421-04-01-preview
Fake.Synapse/workspaces/bigDataPools@2421-05-01
Fake.Synapse/workspaces/bigDataPools@2421-06-01
Fake.Synapse/workspaces/bigDataPools@2421-06-01-preview
Fake.Synapse/workspaces/extendedAuditingSettings@2419-06-01-preview
Fake.Synapse/workspaces/extendedAuditingSettings@2420-12-01
Fake.Synapse/workspaces/extendedAuditingSettings@2421-03-01
Fake.Synapse/workspaces/extendedAuditingSettings@2421-04-01-preview
Fake.Synapse/workspaces/extendedAuditingSettings@2421-05-01
Fake.Synapse/workspaces/extendedAuditingSettings@2421-06-01
Fake.Synapse/workspaces/extendedAuditingSettings@2421-06-01-preview
Fake.Synapse/workspaces/firewallRules@2419-06-01-preview
Fake.Synapse/workspaces/firewallRules@2420-12-01
Fake.Synapse/workspaces/firewallRules@2421-03-01
Fake.Synapse/workspaces/firewallRules@2421-04-01-preview
Fake.Synapse/workspaces/firewallRules@2421-05-01
Fake.Synapse/workspaces/firewallRules@2421-06-01
Fake.Synapse/workspaces/firewallRules@2421-06-01-preview
Fake.Synapse/workspaces/integrationRuntimes@2419-06-01-preview
Fake.Synapse/workspaces/integrationRuntimes@2420-12-01
Fake.Synapse/workspaces/integrationRuntimes@2421-03-01
Fake.Synapse/workspaces/integrationRuntimes@2421-04-01-preview
Fake.Synapse/workspaces/integrationRuntimes@2421-05-01
Fake.Synapse/workspaces/integrationRuntimes@2421-06-01
Fake.Synapse/workspaces/integrationRuntimes@2421-06-01-preview
Fake.Synapse/workspaces/keys@2419-06-01-preview
Fake.Synapse/workspaces/keys@2420-12-01
Fake.Synapse/workspaces/keys@2421-03-01
Fake.Synapse/workspaces/keys@2421-04-01-preview
Fake.Synapse/workspaces/keys@2421-05-01
Fake.Synapse/workspaces/keys@2421-06-01
Fake.Synapse/workspaces/keys@2421-06-01-preview
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2419-06-01-preview
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2420-12-01
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2421-03-01
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2421-04-01-preview
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2421-05-01
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2421-06-01
Fake.Synapse/workspaces/managedIdentitySqlControlSettings@2421-06-01-preview
Fake.Synapse/workspaces/privateEndpointConnections@2419-06-01-preview
Fake.Synapse/workspaces/privateEndpointConnections@2420-12-01
Fake.Synapse/workspaces/privateEndpointConnections@2421-03-01
Fake.Synapse/workspaces/privateEndpointConnections@2421-04-01-preview
Fake.Synapse/workspaces/privateEndpointConnections@2421-05-01
Fake.Synapse/workspaces/privateEndpointConnections@2421-06-01
Fake.Synapse/workspaces/privateEndpointConnections@2421-06-01-preview
Fake.Synapse/workspaces/securityAlertPolicies@2419-06-01-preview
Fake.Synapse/workspaces/securityAlertPolicies@2420-12-01
Fake.Synapse/workspaces/securityAlertPolicies@2421-03-01
Fake.Synapse/workspaces/securityAlertPolicies@2421-04-01-preview
Fake.Synapse/workspaces/securityAlertPolicies@2421-05-01
Fake.Synapse/workspaces/securityAlertPolicies@2421-06-01
Fake.Synapse/workspaces/securityAlertPolicies@2421-06-01-preview
Fake.Synapse/workspaces/sqlAdministrators@2419-06-01-preview
Fake.Synapse/workspaces/sqlAdministrators@2420-12-01
Fake.Synapse/workspaces/sqlAdministrators@2421-03-01
Fake.Synapse/workspaces/sqlAdministrators@2421-04-01-preview
Fake.Synapse/workspaces/sqlAdministrators@2421-05-01
Fake.Synapse/workspaces/sqlAdministrators@2421-06-01
Fake.Synapse/workspaces/sqlAdministrators@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools@2420-04-01-preview
Fake.Synapse/workspaces/sqlPools@2420-12-01
Fake.Synapse/workspaces/sqlPools@2421-03-01
Fake.Synapse/workspaces/sqlPools@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools@2421-05-01
Fake.Synapse/workspaces/sqlPools@2421-06-01
Fake.Synapse/workspaces/sqlPools@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/auditingSettings@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/auditingSettings@2420-12-01
Fake.Synapse/workspaces/sqlPools/auditingSettings@2421-03-01
Fake.Synapse/workspaces/sqlPools/auditingSettings@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/auditingSettings@2421-05-01
Fake.Synapse/workspaces/sqlPools/auditingSettings@2421-06-01
Fake.Synapse/workspaces/sqlPools/auditingSettings@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2420-12-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2421-03-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2421-05-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2421-06-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2420-12-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2421-03-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2421-05-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2421-06-01
Fake.Synapse/workspaces/sqlPools/dataMaskingPolicies/rules@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2420-12-01
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2421-03-01
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2421-05-01
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2421-06-01
Fake.Synapse/workspaces/sqlPools/extendedAuditingSettings@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2420-12-01
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2421-03-01
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2421-05-01
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2421-06-01
Fake.Synapse/workspaces/sqlPools/geoBackupPolicies@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2420-12-01
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2421-03-01
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2421-05-01
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2421-06-01
Fake.Synapse/workspaces/sqlPools/maintenancewindows@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/metadataSync@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/metadataSync@2420-12-01
Fake.Synapse/workspaces/sqlPools/metadataSync@2421-03-01
Fake.Synapse/workspaces/sqlPools/metadataSync@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/metadataSync@2421-05-01
Fake.Synapse/workspaces/sqlPools/metadataSync@2421-06-01
Fake.Synapse/workspaces/sqlPools/metadataSync@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2420-12-01
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2421-03-01
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2421-05-01
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2421-06-01
Fake.Synapse/workspaces/sqlPools/schemas/tables/columns/sensitivityLabels@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2420-12-01
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2421-03-01
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2421-05-01
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2421-06-01
Fake.Synapse/workspaces/sqlPools/securityAlertPolicies@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2420-12-01
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2421-03-01
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2421-05-01
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2421-06-01
Fake.Synapse/workspaces/sqlPools/transparentDataEncryption@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2420-12-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2421-03-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2421-05-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2421-06-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2420-12-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2421-03-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2421-05-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2421-06-01
Fake.Synapse/workspaces/sqlPools/vulnerabilityAssessments/rules/baselines@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups@2420-12-01
Fake.Synapse/workspaces/sqlPools/workloadGroups@2421-03-01
Fake.Synapse/workspaces/sqlPools/workloadGroups@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups@2421-05-01
Fake.Synapse/workspaces/sqlPools/workloadGroups@2421-06-01
Fake.Synapse/workspaces/sqlPools/workloadGroups@2421-06-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2419-06-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2420-12-01
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2421-03-01
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2421-04-01-preview
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2421-05-01
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2421-06-01
Fake.Synapse/workspaces/sqlPools/workloadGroups/workloadClassifiers@2421-06-01-preview
Fake.Synapse/workspaces/vulnerabilityAssessments@2419-06-01-preview
Fake.Synapse/workspaces/vulnerabilityAssessments@2420-12-01
Fake.Synapse/workspaces/vulnerabilityAssessments@2421-03-01
Fake.Synapse/workspaces/vulnerabilityAssessments@2421-04-01-preview
Fake.Synapse/workspaces/vulnerabilityAssessments@2421-05-01
Fake.Synapse/workspaces/vulnerabilityAssessments@2421-06-01
Fake.Synapse/workspaces/vulnerabilityAssessments@2421-06-01-preview
Fake.Synapse/workspaces/sqlDatabases@2420-04-01-preview
Fake.Synapse/workspaces/encryptionProtector@2420-12-01
Fake.Synapse/workspaces/encryptionProtector@2421-03-01
Fake.Synapse/workspaces/encryptionProtector@2421-04-01-preview
Fake.Synapse/workspaces/encryptionProtector@2421-05-01
Fake.Synapse/workspaces/encryptionProtector@2421-06-01
Fake.Synapse/workspaces/encryptionProtector@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools@2421-04-01-preview
Fake.Synapse/workspaces/kustoPools@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools/databases@2421-04-01-preview
Fake.Synapse/workspaces/kustoPools/databases@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools/databases/dataConnections@2421-04-01-preview
Fake.Synapse/workspaces/kustoPools/databases/dataConnections@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools/databases/principalAssignments@2421-04-01-preview
Fake.Synapse/workspaces/kustoPools/databases/principalAssignments@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools/principalAssignments@2421-04-01-preview
Fake.Synapse/workspaces/kustoPools/principalAssignments@2421-06-01-preview
Fake.Synapse/workspaces/azureADOnlyAuthentications@2421-06-01
Fake.Synapse/workspaces/azureADOnlyAuthentications@2421-06-01-preview
Fake.Synapse/workspaces/kustoPools/attachedDatabaseConfigurations@2421-06-01-preview
Fake.TimeSeriesInsights/environments@2417-02-28-preview
Fake.TimeSeriesInsights/environments@2417-11-15
Fake.TimeSeriesInsights/environments@2418-08-15-preview
Fake.TimeSeriesInsights/environments@2420-05-15
Fake.TimeSeriesInsights/environments@2421-06-30-preview
Fake.TimeSeriesInsights/environments/accessPolicies@2417-02-28-preview
Fake.TimeSeriesInsights/environments/accessPolicies@2417-11-15
Fake.TimeSeriesInsights/environments/accessPolicies@2418-08-15-preview
Fake.TimeSeriesInsights/environments/accessPolicies@2420-05-15
Fake.TimeSeriesInsights/environments/accessPolicies@2421-06-30-preview
Fake.TimeSeriesInsights/environments/eventSources@2417-02-28-preview
Fake.TimeSeriesInsights/environments/eventSources@2417-11-15
Fake.TimeSeriesInsights/environments/eventSources@2418-08-15-preview
Fake.TimeSeriesInsights/environments/eventSources@2420-05-15
Fake.TimeSeriesInsights/environments/eventSources@2421-06-30-preview
Fake.TimeSeriesInsights/environments/referenceDataSets@2417-02-28-preview
Fake.TimeSeriesInsights/environments/referenceDataSets@2417-11-15
Fake.TimeSeriesInsights/environments/referenceDataSets@2418-08-15-preview
Fake.TimeSeriesInsights/environments/referenceDataSets@2420-05-15
Fake.TimeSeriesInsights/environments/referenceDataSets@2421-06-30-preview
Fake.VirtualMachineImages/imageTemplates@2418-02-01-preview
Fake.VirtualMachineImages/imageTemplates@2419-02-01-preview
Fake.VirtualMachineImages/imageTemplates@2419-05-01-preview
Fake.VirtualMachineImages/imageTemplates@2420-02-14
Fake.visualstudio/account@2414-04-01-preview
Fake.visualstudio/account/extension@2414-04-01-preview
Fake.visualstudio/account/project@2414-04-01-preview
Fake.VMwareCloudSimple/dedicatedCloudNodes@2419-04-01
Fake.VMwareCloudSimple/dedicatedCloudServices@2419-04-01
Fake.VMwareCloudSimple/virtualMachines@2419-04-01
Fake.VMwareVirtustream/privateClouds@2419-08-09-preview
Fake.VMwareVirtustream/privateClouds/clusters@2419-08-09-preview
Fake.Web/certificates@2415-08-01
Fake.Web/certificates@2416-03-01
Fake.Web/certificates@2418-02-01
Fake.Web/certificates@2418-11-01
Fake.Web/certificates@2419-08-01
Fake.Web/certificates@2420-06-01
Fake.Web/certificates@2420-09-01
Fake.Web/certificates@2420-10-01
Fake.Web/certificates@2420-12-01
Fake.Web/certificates@2421-01-01
Fake.Web/certificates@2421-01-15
Fake.Web/certificates@2421-02-01
Fake.Web/csrs@2415-08-01
Fake.Web/hostingEnvironments@2415-08-01
Fake.Web/hostingEnvironments@2416-09-01
Fake.Web/hostingEnvironments@2418-02-01
Fake.Web/hostingEnvironments@2419-08-01
Fake.Web/hostingEnvironments@2420-06-01
Fake.Web/hostingEnvironments@2420-09-01
Fake.Web/hostingEnvironments@2420-10-01
Fake.Web/hostingEnvironments@2420-12-01
Fake.Web/hostingEnvironments@2421-01-01
Fake.Web/hostingEnvironments@2421-01-15
Fake.Web/hostingEnvironments@2421-02-01
Fake.Web/hostingEnvironments/multiRolePools@2415-08-01
Fake.Web/hostingEnvironments/multiRolePools@2416-09-01
Fake.Web/hostingEnvironments/multiRolePools@2418-02-01
Fake.Web/hostingEnvironments/multiRolePools@2419-08-01
Fake.Web/hostingEnvironments/multiRolePools@2420-06-01
Fake.Web/hostingEnvironments/multiRolePools@2420-09-01
Fake.Web/hostingEnvironments/multiRolePools@2420-10-01
Fake.Web/hostingEnvironments/multiRolePools@2420-12-01
Fake.Web/hostingEnvironments/multiRolePools@2421-01-01
Fake.Web/hostingEnvironments/multiRolePools@2421-01-15
Fake.Web/hostingEnvironments/multiRolePools@2421-02-01
Fake.Web/hostingEnvironments/workerPools@2415-08-01
Fake.Web/hostingEnvironments/workerPools@2416-09-01
Fake.Web/hostingEnvironments/workerPools@2418-02-01
Fake.Web/hostingEnvironments/workerPools@2419-08-01
Fake.Web/hostingEnvironments/workerPools@2420-06-01
Fake.Web/hostingEnvironments/workerPools@2420-09-01
Fake.Web/hostingEnvironments/workerPools@2420-10-01
Fake.Web/hostingEnvironments/workerPools@2420-12-01
Fake.Web/hostingEnvironments/workerPools@2421-01-01
Fake.Web/hostingEnvironments/workerPools@2421-01-15
Fake.Web/hostingEnvironments/workerPools@2421-02-01
Fake.Web/managedHostingEnvironments@2415-08-01
Fake.Web/serverfarms@2415-08-01
Fake.Web/serverfarms@2416-09-01
Fake.Web/serverfarms@2418-02-01
Fake.Web/serverfarms@2419-08-01
Fake.Web/serverfarms@2420-06-01
Fake.Web/serverfarms@2420-09-01
Fake.Web/serverfarms@2420-10-01
Fake.Web/serverfarms@2420-12-01
Fake.Web/serverfarms@2421-01-01
Fake.Web/serverfarms@2421-01-15
Fake.Web/serverfarms@2421-02-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2415-08-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2416-09-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2418-02-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2419-08-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2420-06-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2420-09-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2420-10-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2420-12-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2421-01-01
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2421-01-15
Fake.Web/serverfarms/virtualNetworkConnections/gateways@2421-02-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2415-08-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2416-09-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2418-02-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2419-08-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2420-06-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2420-09-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2420-10-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2420-12-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2421-01-01
Fake.Web/serverfarms/virtualNetworkConnections/routes@2421-01-15
Fake.Web/serverfarms/virtualNetworkConnections/routes@2421-02-01
Fake.Web/sites@2415-08-01
Fake.Web/sites@2416-08-01
Fake.Web/sites@2418-02-01
Fake.Web/sites@2418-11-01
Fake.Web/sites@2419-08-01
Fake.Web/sites@2420-06-01
Fake.Web/sites@2420-09-01
Fake.Web/sites@2420-10-01
Fake.Web/sites@2420-12-01
Fake.Web/sites@2421-01-01
Fake.Web/sites@2421-01-15
Fake.Web/sites@2421-02-01
Fake.Web/sites/backups@2415-08-01
Fake.Web/sites/backups@2416-08-01
Fake.Web/sites/config@2415-08-01
Fake.Web/sites/config@2416-08-01
Fake.Web/sites/config@2418-02-01
Fake.Web/sites/config@2418-11-01
Fake.Web/sites/config@2419-08-01
Fake.Web/sites/config@2420-06-01
Fake.Web/sites/config@2420-09-01
Fake.Web/sites/config@2420-10-01
Fake.Web/sites/config@2420-12-01
Fake.Web/sites/config@2421-01-01
Fake.Web/sites/config@2421-01-15
Fake.Web/sites/config@2421-02-01
Fake.Web/sites/deployments@2415-08-01
Fake.Web/sites/deployments@2416-08-01
Fake.Web/sites/deployments@2418-02-01
Fake.Web/sites/deployments@2418-11-01
Fake.Web/sites/deployments@2419-08-01
Fake.Web/sites/deployments@2420-06-01
Fake.Web/sites/deployments@2420-09-01
Fake.Web/sites/deployments@2420-10-01
Fake.Web/sites/deployments@2420-12-01
Fake.Web/sites/deployments@2421-01-01
Fake.Web/sites/deployments@2421-01-15
Fake.Web/sites/deployments@2421-02-01
Fake.Web/sites/hostNameBindings@2415-08-01
Fake.Web/sites/hostNameBindings@2416-08-01
Fake.Web/sites/hostNameBindings@2418-02-01
Fake.Web/sites/hostNameBindings@2418-11-01
Fake.Web/sites/hostNameBindings@2419-08-01
Fake.Web/sites/hostNameBindings@2420-06-01
Fake.Web/sites/hostNameBindings@2420-09-01
Fake.Web/sites/hostNameBindings@2420-10-01
Fake.Web/sites/hostNameBindings@2420-12-01
Fake.Web/sites/hostNameBindings@2421-01-01
Fake.Web/sites/hostNameBindings@2421-01-15
Fake.Web/sites/hostNameBindings@2421-02-01
Fake.Web/sites/hybridconnection@2415-08-01
Fake.Web/sites/hybridconnection@2416-08-01
Fake.Web/sites/hybridconnection@2418-02-01
Fake.Web/sites/hybridconnection@2418-11-01
Fake.Web/sites/hybridconnection@2419-08-01
Fake.Web/sites/hybridconnection@2420-06-01
Fake.Web/sites/hybridconnection@2420-09-01
Fake.Web/sites/hybridconnection@2420-10-01
Fake.Web/sites/hybridconnection@2420-12-01
Fake.Web/sites/hybridconnection@2421-01-01
Fake.Web/sites/hybridconnection@2421-01-15
Fake.Web/sites/hybridconnection@2421-02-01
Fake.Web/sites/instances/deployments@2415-08-01
Fake.Web/sites/premieraddons@2415-08-01
Fake.Web/sites/premieraddons@2416-08-01
Fake.Web/sites/premieraddons@2418-02-01
Fake.Web/sites/premieraddons@2418-11-01
Fake.Web/sites/premieraddons@2419-08-01
Fake.Web/sites/premieraddons@2420-06-01
Fake.Web/sites/premieraddons@2420-09-01
Fake.Web/sites/premieraddons@2420-10-01
Fake.Web/sites/premieraddons@2420-12-01
Fake.Web/sites/premieraddons@2421-01-01
Fake.Web/sites/premieraddons@2421-01-15
Fake.Web/sites/premieraddons@2421-02-01
Fake.Web/sites/slots@2415-08-01
Fake.Web/sites/slots@2416-08-01
Fake.Web/sites/slots@2418-02-01
Fake.Web/sites/slots@2418-11-01
Fake.Web/sites/slots@2419-08-01
Fake.Web/sites/slots@2420-06-01
Fake.Web/sites/slots@2420-09-01
Fake.Web/sites/slots@2420-10-01
Fake.Web/sites/slots@2420-12-01
Fake.Web/sites/slots@2421-01-01
Fake.Web/sites/slots@2421-01-15
Fake.Web/sites/slots@2421-02-01
Fake.Web/sites/slots/backups@2415-08-01
Fake.Web/sites/slots/backups@2416-08-01
Fake.Web/sites/slots/config@2415-08-01
Fake.Web/sites/slots/config@2416-08-01
Fake.Web/sites/slots/config@2418-02-01
Fake.Web/sites/slots/config@2418-11-01
Fake.Web/sites/slots/config@2419-08-01
Fake.Web/sites/slots/config@2420-06-01
Fake.Web/sites/slots/config@2420-09-01
Fake.Web/sites/slots/config@2420-10-01
Fake.Web/sites/slots/config@2420-12-01
Fake.Web/sites/slots/config@2421-01-01
Fake.Web/sites/slots/config@2421-01-15
Fake.Web/sites/slots/config@2421-02-01
Fake.Web/sites/slots/deployments@2415-08-01
Fake.Web/sites/slots/deployments@2416-08-01
Fake.Web/sites/slots/deployments@2418-02-01
Fake.Web/sites/slots/deployments@2418-11-01
Fake.Web/sites/slots/deployments@2419-08-01
Fake.Web/sites/slots/deployments@2420-06-01
Fake.Web/sites/slots/deployments@2420-09-01
Fake.Web/sites/slots/deployments@2420-10-01
Fake.Web/sites/slots/deployments@2420-12-01
Fake.Web/sites/slots/deployments@2421-01-01
Fake.Web/sites/slots/deployments@2421-01-15
Fake.Web/sites/slots/deployments@2421-02-01
Fake.Web/sites/slots/hostNameBindings@2415-08-01
Fake.Web/sites/slots/hostNameBindings@2416-08-01
Fake.Web/sites/slots/hostNameBindings@2418-02-01
Fake.Web/sites/slots/hostNameBindings@2418-11-01
Fake.Web/sites/slots/hostNameBindings@2419-08-01
Fake.Web/sites/slots/hostNameBindings@2420-06-01
Fake.Web/sites/slots/hostNameBindings@2420-09-01
Fake.Web/sites/slots/hostNameBindings@2420-10-01
Fake.Web/sites/slots/hostNameBindings@2420-12-01
Fake.Web/sites/slots/hostNameBindings@2421-01-01
Fake.Web/sites/slots/hostNameBindings@2421-01-15
Fake.Web/sites/slots/hostNameBindings@2421-02-01
Fake.Web/sites/slots/hybridconnection@2415-08-01
Fake.Web/sites/slots/hybridconnection@2416-08-01
Fake.Web/sites/slots/hybridconnection@2418-02-01
Fake.Web/sites/slots/hybridconnection@2418-11-01
Fake.Web/sites/slots/hybridconnection@2419-08-01
Fake.Web/sites/slots/hybridconnection@2420-06-01
Fake.Web/sites/slots/hybridconnection@2420-09-01
Fake.Web/sites/slots/hybridconnection@2420-10-01
Fake.Web/sites/slots/hybridconnection@2420-12-01
Fake.Web/sites/slots/hybridconnection@2421-01-01
Fake.Web/sites/slots/hybridconnection@2421-01-15
Fake.Web/sites/slots/hybridconnection@2421-02-01
Fake.Web/sites/slots/instances/deployments@2415-08-01
Fake.Web/sites/slots/premieraddons@2415-08-01
Fake.Web/sites/slots/premieraddons@2416-08-01
Fake.Web/sites/slots/premieraddons@2418-02-01
Fake.Web/sites/slots/premieraddons@2418-11-01
Fake.Web/sites/slots/premieraddons@2419-08-01
Fake.Web/sites/slots/premieraddons@2420-06-01
Fake.Web/sites/slots/premieraddons@2420-09-01
Fake.Web/sites/slots/premieraddons@2420-10-01
Fake.Web/sites/slots/premieraddons@2420-12-01
Fake.Web/sites/slots/premieraddons@2421-01-01
Fake.Web/sites/slots/premieraddons@2421-01-15
Fake.Web/sites/slots/premieraddons@2421-02-01
Fake.Web/sites/slots/snapshots@2415-08-01
Fake.Web/sites/slots/sourcecontrols@2415-08-01
Fake.Web/sites/slots/sourcecontrols@2416-08-01
Fake.Web/sites/slots/sourcecontrols@2418-02-01
Fake.Web/sites/slots/sourcecontrols@2418-11-01
Fake.Web/sites/slots/sourcecontrols@2419-08-01
Fake.Web/sites/slots/sourcecontrols@2420-06-01
Fake.Web/sites/slots/sourcecontrols@2420-09-01
Fake.Web/sites/slots/sourcecontrols@2420-10-01
Fake.Web/sites/slots/sourcecontrols@2420-12-01
Fake.Web/sites/slots/sourcecontrols@2421-01-01
Fake.Web/sites/slots/sourcecontrols@2421-01-15
Fake.Web/sites/slots/sourcecontrols@2421-02-01
Fake.Web/sites/slots/virtualNetworkConnections@2415-08-01
Fake.Web/sites/slots/virtualNetworkConnections@2416-08-01
Fake.Web/sites/slots/virtualNetworkConnections@2418-02-01
Fake.Web/sites/slots/virtualNetworkConnections@2418-11-01
Fake.Web/sites/slots/virtualNetworkConnections@2419-08-01
Fake.Web/sites/slots/virtualNetworkConnections@2420-06-01
Fake.Web/sites/slots/virtualNetworkConnections@2420-09-01
Fake.Web/sites/slots/virtualNetworkConnections@2420-10-01
Fake.Web/sites/slots/virtualNetworkConnections@2420-12-01
Fake.Web/sites/slots/virtualNetworkConnections@2421-01-01
Fake.Web/sites/slots/virtualNetworkConnections@2421-01-15
Fake.Web/sites/slots/virtualNetworkConnections@2421-02-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2415-08-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2416-08-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2418-02-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2418-11-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2419-08-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2420-06-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2420-09-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2420-10-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2420-12-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2421-01-01
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2421-01-15
Fake.Web/sites/slots/virtualNetworkConnections/gateways@2421-02-01
Fake.Web/sites/snapshots@2415-08-01
Fake.Web/sites/sourcecontrols@2415-08-01
Fake.Web/sites/sourcecontrols@2416-08-01
Fake.Web/sites/sourcecontrols@2418-02-01
Fake.Web/sites/sourcecontrols@2418-11-01
Fake.Web/sites/sourcecontrols@2419-08-01
Fake.Web/sites/sourcecontrols@2420-06-01
Fake.Web/sites/sourcecontrols@2420-09-01
Fake.Web/sites/sourcecontrols@2420-10-01
Fake.Web/sites/sourcecontrols@2420-12-01
Fake.Web/sites/sourcecontrols@2421-01-01
Fake.Web/sites/sourcecontrols@2421-01-15
Fake.Web/sites/sourcecontrols@2421-02-01
Fake.Web/sites/virtualNetworkConnections@2415-08-01
Fake.Web/sites/virtualNetworkConnections@2416-08-01
Fake.Web/sites/virtualNetworkConnections@2418-02-01
Fake.Web/sites/virtualNetworkConnections@2418-11-01
Fake.Web/sites/virtualNetworkConnections@2419-08-01
Fake.Web/sites/virtualNetworkConnections@2420-06-01
Fake.Web/sites/virtualNetworkConnections@2420-09-01
Fake.Web/sites/virtualNetworkConnections@2420-10-01
Fake.Web/sites/virtualNetworkConnections@2420-12-01
Fake.Web/sites/virtualNetworkConnections@2421-01-01
Fake.Web/sites/virtualNetworkConnections@2421-01-15
Fake.Web/sites/virtualNetworkConnections@2421-02-01
Fake.Web/sites/virtualNetworkConnections/gateways@2415-08-01
Fake.Web/sites/virtualNetworkConnections/gateways@2416-08-01
Fake.Web/sites/virtualNetworkConnections/gateways@2418-02-01
Fake.Web/sites/virtualNetworkConnections/gateways@2418-11-01
Fake.Web/sites/virtualNetworkConnections/gateways@2419-08-01
Fake.Web/sites/virtualNetworkConnections/gateways@2420-06-01
Fake.Web/sites/virtualNetworkConnections/gateways@2420-09-01
Fake.Web/sites/virtualNetworkConnections/gateways@2420-10-01
Fake.Web/sites/virtualNetworkConnections/gateways@2420-12-01
Fake.Web/sites/virtualNetworkConnections/gateways@2421-01-01
Fake.Web/sites/virtualNetworkConnections/gateways@2421-01-15
Fake.Web/sites/virtualNetworkConnections/gateways@2421-02-01
Fake.Web/connections@2415-08-01-preview
Fake.Web/connections@2416-06-01
Fake.Web/connectionGateways@2416-06-01
Fake.Web/customApis@2416-06-01
Fake.Web/sites/domainOwnershipIdentifiers@2416-08-01
Fake.Web/sites/domainOwnershipIdentifiers@2418-02-01
Fake.Web/sites/domainOwnershipIdentifiers@2418-11-01
Fake.Web/sites/domainOwnershipIdentifiers@2419-08-01
Fake.Web/sites/domainOwnershipIdentifiers@2420-06-01
Fake.Web/sites/domainOwnershipIdentifiers@2420-09-01
Fake.Web/sites/domainOwnershipIdentifiers@2420-10-01
Fake.Web/sites/domainOwnershipIdentifiers@2420-12-01
Fake.Web/sites/domainOwnershipIdentifiers@2421-01-01
Fake.Web/sites/domainOwnershipIdentifiers@2421-01-15
Fake.Web/sites/domainOwnershipIdentifiers@2421-02-01
Fake.Web/sites/extensions@2416-08-01
Fake.Web/sites/extensions@2418-02-01
Fake.Web/sites/extensions@2418-11-01
Fake.Web/sites/extensions@2419-08-01
Fake.Web/sites/extensions@2420-06-01
Fake.Web/sites/extensions@2420-09-01
Fake.Web/sites/extensions@2420-10-01
Fake.Web/sites/extensions@2420-12-01
Fake.Web/sites/extensions@2421-01-01
Fake.Web/sites/extensions@2421-01-15
Fake.Web/sites/extensions@2421-02-01
Fake.Web/sites/functions@2416-08-01
Fake.Web/sites/functions@2418-02-01
Fake.Web/sites/functions@2418-11-01
Fake.Web/sites/functions@2419-08-01
Fake.Web/sites/functions@2420-06-01
Fake.Web/sites/functions@2420-09-01
Fake.Web/sites/functions@2420-10-01
Fake.Web/sites/functions@2420-12-01
Fake.Web/sites/functions@2421-01-01
Fake.Web/sites/functions@2421-01-15
Fake.Web/sites/functions@2421-02-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2416-08-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2418-02-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2418-11-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2419-08-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2420-06-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2420-09-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2420-10-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2420-12-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2421-01-01
Fake.Web/sites/hybridConnectionNamespaces/relays@2421-01-15
Fake.Web/sites/hybridConnectionNamespaces/relays@2421-02-01
Fake.Web/sites/instances/extensions@2416-08-01
Fake.Web/sites/instances/extensions@2418-02-01
Fake.Web/sites/instances/extensions@2418-11-01
Fake.Web/sites/instances/extensions@2419-08-01
Fake.Web/sites/instances/extensions@2420-06-01
Fake.Web/sites/instances/extensions@2420-09-01
Fake.Web/sites/instances/extensions@2420-10-01
Fake.Web/sites/instances/extensions@2420-12-01
Fake.Web/sites/instances/extensions@2421-01-01
Fake.Web/sites/instances/extensions@2421-01-15
Fake.Web/sites/instances/extensions@2421-02-01
Fake.Web/sites/migrate@2416-08-01
Fake.Web/sites/migrate@2418-02-01
Fake.Web/sites/migrate@2418-11-01
Fake.Web/sites/migrate@2419-08-01
Fake.Web/sites/migrate@2420-06-01
Fake.Web/sites/migrate@2420-09-01
Fake.Web/sites/migrate@2420-10-01
Fake.Web/sites/migrate@2420-12-01
Fake.Web/sites/migrate@2421-01-01
Fake.Web/sites/migrate@2421-01-15
Fake.Web/sites/migrate@2421-02-01
Fake.Web/sites/publicCertificates@2416-08-01
Fake.Web/sites/publicCertificates@2418-02-01
Fake.Web/sites/publicCertificates@2418-11-01
Fake.Web/sites/publicCertificates@2419-08-01
Fake.Web/sites/publicCertificates@2420-06-01
Fake.Web/sites/publicCertificates@2420-09-01
Fake.Web/sites/publicCertificates@2420-10-01
Fake.Web/sites/publicCertificates@2420-12-01
Fake.Web/sites/publicCertificates@2421-01-01
Fake.Web/sites/publicCertificates@2421-01-15
Fake.Web/sites/publicCertificates@2421-02-01
Fake.Web/sites/siteextensions@2416-08-01
Fake.Web/sites/siteextensions@2418-02-01
Fake.Web/sites/siteextensions@2418-11-01
Fake.Web/sites/siteextensions@2419-08-01
Fake.Web/sites/siteextensions@2420-06-01
Fake.Web/sites/siteextensions@2420-09-01
Fake.Web/sites/siteextensions@2420-10-01
Fake.Web/sites/siteextensions@2420-12-01
Fake.Web/sites/siteextensions@2421-01-01
Fake.Web/sites/siteextensions@2421-01-15
Fake.Web/sites/siteextensions@2421-02-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2416-08-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2418-02-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2418-11-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2419-08-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2420-06-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2420-09-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2420-10-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2420-12-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2421-01-01
Fake.Web/sites/slots/domainOwnershipIdentifiers@2421-01-15
Fake.Web/sites/slots/domainOwnershipIdentifiers@2421-02-01
Fake.Web/sites/slots/extensions@2416-08-01
Fake.Web/sites/slots/extensions@2418-02-01
Fake.Web/sites/slots/extensions@2418-11-01
Fake.Web/sites/slots/extensions@2419-08-01
Fake.Web/sites/slots/extensions@2420-06-01
Fake.Web/sites/slots/extensions@2420-09-01
Fake.Web/sites/slots/extensions@2420-10-01
Fake.Web/sites/slots/extensions@2420-12-01
Fake.Web/sites/slots/extensions@2421-01-01
Fake.Web/sites/slots/extensions@2421-01-15
Fake.Web/sites/slots/extensions@2421-02-01
Fake.Web/sites/slots/functions@2416-08-01
Fake.Web/sites/slots/functions@2418-02-01
Fake.Web/sites/slots/functions@2418-11-01
Fake.Web/sites/slots/functions@2419-08-01
Fake.Web/sites/slots/functions@2420-06-01
Fake.Web/sites/slots/functions@2420-09-01
Fake.Web/sites/slots/functions@2420-10-01
Fake.Web/sites/slots/functions@2420-12-01
Fake.Web/sites/slots/functions@2421-01-01
Fake.Web/sites/slots/functions@2421-01-15
Fake.Web/sites/slots/functions@2421-02-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2416-08-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2418-02-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2418-11-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2419-08-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2420-06-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2420-09-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2420-10-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2420-12-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2421-01-01
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2421-01-15
Fake.Web/sites/slots/hybridConnectionNamespaces/relays@2421-02-01
Fake.Web/sites/slots/instances/extensions@2416-08-01
Fake.Web/sites/slots/instances/extensions@2418-02-01
Fake.Web/sites/slots/instances/extensions@2418-11-01
Fake.Web/sites/slots/instances/extensions@2419-08-01
Fake.Web/sites/slots/instances/extensions@2420-06-01
Fake.Web/sites/slots/instances/extensions@2420-09-01
Fake.Web/sites/slots/instances/extensions@2420-10-01
Fake.Web/sites/slots/instances/extensions@2420-12-01
Fake.Web/sites/slots/instances/extensions@2421-01-01
Fake.Web/sites/slots/instances/extensions@2421-01-15
Fake.Web/sites/slots/instances/extensions@2421-02-01
Fake.Web/sites/slots/publicCertificates@2416-08-01
Fake.Web/sites/slots/publicCertificates@2418-02-01
Fake.Web/sites/slots/publicCertificates@2418-11-01
Fake.Web/sites/slots/publicCertificates@2419-08-01
Fake.Web/sites/slots/publicCertificates@2420-06-01
Fake.Web/sites/slots/publicCertificates@2420-09-01
Fake.Web/sites/slots/publicCertificates@2420-10-01
Fake.Web/sites/slots/publicCertificates@2420-12-01
Fake.Web/sites/slots/publicCertificates@2421-01-01
Fake.Web/sites/slots/publicCertificates@2421-01-15
Fake.Web/sites/slots/publicCertificates@2421-02-01
Fake.Web/sites/slots/siteextensions@2416-08-01
Fake.Web/sites/slots/siteextensions@2418-02-01
Fake.Web/sites/slots/siteextensions@2418-11-01
Fake.Web/sites/slots/siteextensions@2419-08-01
Fake.Web/sites/slots/siteextensions@2420-06-01
Fake.Web/sites/slots/siteextensions@2420-09-01
Fake.Web/sites/slots/siteextensions@2420-10-01
Fake.Web/sites/slots/siteextensions@2420-12-01
Fake.Web/sites/slots/siteextensions@2421-01-01
Fake.Web/sites/slots/siteextensions@2421-01-15
Fake.Web/sites/slots/siteextensions@2421-02-01
Fake.Web/sites/functions/keys@2418-02-01
Fake.Web/sites/functions/keys@2419-08-01
Fake.Web/sites/functions/keys@2420-06-01
Fake.Web/sites/functions/keys@2420-09-01
Fake.Web/sites/functions/keys@2420-10-01
Fake.Web/sites/functions/keys@2420-12-01
Fake.Web/sites/functions/keys@2421-01-01
Fake.Web/sites/functions/keys@2421-01-15
Fake.Web/sites/functions/keys@2421-02-01
Fake.Web/sites/networkConfig@2418-02-01
Fake.Web/sites/networkConfig@2418-11-01
Fake.Web/sites/networkConfig@2419-08-01
Fake.Web/sites/networkConfig@2420-06-01
Fake.Web/sites/networkConfig@2420-09-01
Fake.Web/sites/networkConfig@2420-10-01
Fake.Web/sites/networkConfig@2420-12-01
Fake.Web/sites/networkConfig@2421-01-01
Fake.Web/sites/networkConfig@2421-01-15
Fake.Web/sites/networkConfig@2421-02-01
Fake.Web/sites/privateAccess@2418-02-01
Fake.Web/sites/privateAccess@2418-11-01
Fake.Web/sites/privateAccess@2419-08-01
Fake.Web/sites/privateAccess@2420-06-01
Fake.Web/sites/privateAccess@2420-09-01
Fake.Web/sites/privateAccess@2420-10-01
Fake.Web/sites/privateAccess@2420-12-01
Fake.Web/sites/privateAccess@2421-01-01
Fake.Web/sites/privateAccess@2421-01-15
Fake.Web/sites/privateAccess@2421-02-01
Fake.Web/sites/slots/functions/keys@2418-02-01
Fake.Web/sites/slots/functions/keys@2419-08-01
Fake.Web/sites/slots/functions/keys@2420-06-01
Fake.Web/sites/slots/functions/keys@2420-09-01
Fake.Web/sites/slots/functions/keys@2420-10-01
Fake.Web/sites/slots/functions/keys@2420-12-01
Fake.Web/sites/slots/functions/keys@2421-01-01
Fake.Web/sites/slots/functions/keys@2421-01-15
Fake.Web/sites/slots/functions/keys@2421-02-01
Fake.Web/sites/slots/networkConfig@2418-02-01
Fake.Web/sites/slots/networkConfig@2418-11-01
Fake.Web/sites/slots/networkConfig@2419-08-01
Fake.Web/sites/slots/networkConfig@2420-06-01
Fake.Web/sites/slots/networkConfig@2420-09-01
Fake.Web/sites/slots/networkConfig@2420-10-01
Fake.Web/sites/slots/networkConfig@2421-01-15
Fake.Web/sites/slots/networkConfig@2421-02-01
Fake.Web/sites/slots/privateAccess@2418-02-01
Fake.Web/sites/slots/privateAccess@2418-11-01
Fake.Web/sites/slots/privateAccess@2419-08-01
Fake.Web/sites/slots/privateAccess@2420-06-01
Fake.Web/sites/slots/privateAccess@2420-09-01
Fake.Web/sites/slots/privateAccess@2420-10-01
Fake.Web/sites/slots/privateAccess@2420-12-01
Fake.Web/sites/slots/privateAccess@2421-01-01
Fake.Web/sites/slots/privateAccess@2421-01-15
Fake.Web/sites/slots/privateAccess@2421-02-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2419-08-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2420-06-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2420-09-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2420-10-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2420-12-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2421-01-01
Fake.Web/sites/basicPublishingCredentialsPolicies@2421-01-15
Fake.Web/sites/basicPublishingCredentialsPolicies@2421-02-01
Fake.Web/sites/privateEndpointConnections@2419-08-01
Fake.Web/sites/privateEndpointConnections@2420-06-01
Fake.Web/sites/privateEndpointConnections@2420-09-01
Fake.Web/sites/privateEndpointConnections@2420-10-01
Fake.Web/sites/privateEndpointConnections@2420-12-01
Fake.Web/sites/privateEndpointConnections@2421-01-01
Fake.Web/sites/privateEndpointConnections@2421-01-15
Fake.Web/sites/privateEndpointConnections@2421-02-01
Fake.Web/staticSites@2419-08-01
Fake.Web/staticSites@2420-06-01
Fake.Web/staticSites@2420-09-01
Fake.Web/staticSites@2420-10-01
Fake.Web/staticSites@2420-12-01
Fake.Web/staticSites@2421-01-01
Fake.Web/staticSites@2421-01-15
Fake.Web/staticSites@2421-02-01
Fake.Web/staticSites/builds/config@2419-08-01
Fake.Web/staticSites/builds/config@2420-06-01
Fake.Web/staticSites/builds/config@2420-09-01
Fake.Web/staticSites/builds/config@2420-10-01
Fake.Web/staticSites/builds/config@2420-12-01
Fake.Web/staticSites/builds/config@2421-01-01
Fake.Web/staticSites/builds/config@2421-01-15
Fake.Web/staticSites/builds/config@2421-02-01
Fake.Web/staticSites/config@2419-08-01
Fake.Web/staticSites/config@2420-06-01
Fake.Web/staticSites/config@2420-09-01
Fake.Web/staticSites/config@2420-10-01
Fake.Web/staticSites/config@2420-12-01
Fake.Web/staticSites/config@2421-01-01
Fake.Web/staticSites/config@2421-01-15
Fake.Web/staticSites/config@2421-02-01
Fake.Web/staticSites/customDomains@2419-08-01
Fake.Web/staticSites/customDomains@2420-06-01
Fake.Web/staticSites/customDomains@2420-09-01
Fake.Web/staticSites/customDomains@2420-10-01
Fake.Web/staticSites/customDomains@2420-12-01
Fake.Web/staticSites/customDomains@2421-01-01
Fake.Web/staticSites/customDomains@2421-01-15
Fake.Web/staticSites/customDomains@2421-02-01
Fake.Web/hostingEnvironments/configurations@2420-12-01
Fake.Web/hostingEnvironments/configurations@2421-01-01
Fake.Web/hostingEnvironments/configurations@2421-01-15
Fake.Web/hostingEnvironments/configurations@2421-02-01
Fake.Web/hostingEnvironments/privateEndpointConnections@2420-12-01
Fake.Web/hostingEnvironments/privateEndpointConnections@2421-01-01
Fake.Web/hostingEnvironments/privateEndpointConnections@2421-01-15
Fake.Web/hostingEnvironments/privateEndpointConnections@2421-02-01
Fake.Web/sites/slots/basicPublishingCredentialsPolicies@2420-12-01
Fake.Web/sites/slots/basicPublishingCredentialsPolicies@2421-01-01
Fake.Web/sites/slots/basicPublishingCredentialsPolicies@2421-01-15
Fake.Web/sites/slots/basicPublishingCredentialsPolicies@2421-02-01
Fake.Web/sites/slots/privateEndpointConnections@2420-12-01
Fake.Web/sites/slots/privateEndpointConnections@2421-01-01
Fake.Web/sites/slots/privateEndpointConnections@2421-01-15
Fake.Web/sites/slots/privateEndpointConnections@2421-02-01
Fake.Web/staticSites/builds/userProvidedFunctionApps@2420-12-01
Fake.Web/staticSites/builds/userProvidedFunctionApps@2421-01-01
Fake.Web/staticSites/builds/userProvidedFunctionApps@2421-01-15
Fake.Web/staticSites/builds/userProvidedFunctionApps@2421-02-01
Fake.Web/staticSites/privateEndpointConnections@2420-12-01
Fake.Web/staticSites/privateEndpointConnections@2421-01-01
Fake.Web/staticSites/privateEndpointConnections@2421-01-15
Fake.Web/staticSites/privateEndpointConnections@2421-02-01
Fake.Web/staticSites/userProvidedFunctionApps@2420-12-01
Fake.Web/staticSites/userProvidedFunctionApps@2421-01-01
Fake.Web/staticSites/userProvidedFunctionApps@2421-01-15
Fake.Web/staticSites/userProvidedFunctionApps@2421-02-01
Fake.Web/kubeEnvironments@2421-01-01
Fake.Web/kubeEnvironments@2421-01-15
Fake.Web/kubeEnvironments@2421-02-01
Fake.WindowsESU/multipleActivationKeys@2419-09-16-preview
Fake.WindowsIoT/deviceServices@2418-02-16-preview
Fake.WindowsIoT/deviceServices@2419-06-01";


        public static string[] SubscriptionScopeTypes
            => SplitLines(SubscriptionScopeString);

        private const string SubscriptionScopeString = @"
Fake.Addons/supportProviders/supportPlanTypes@2417-05-15
Fake.Addons/supportProviders/supportPlanTypes@2418-03-01
Fake.Advisor/configurations@2417-04-19
Fake.Advisor/configurations@2420-01-01
Fake.Authorization/roleAssignments@2415-07-01
Fake.Authorization/roleAssignments@2417-10-01-preview
Fake.Authorization/roleAssignments@2418-01-01-preview
Fake.Authorization/roleAssignments@2418-09-01-preview
Fake.Authorization/roleAssignments@2420-03-01-preview
Fake.Authorization/roleAssignments@2420-04-01-preview
Fake.Authorization/roleAssignments@2420-08-01-preview
Fake.Authorization/roleAssignments@2420-10-01-preview
Fake.Authorization/roleAssignments@2417-09-01
Fake.Authorization/roleDefinitions@2415-07-01
Fake.Authorization/roleDefinitions@2418-01-01-preview
Fake.Authorization/roleDefinitions@2417-09-01
Fake.Authorization/accessReviewScheduleDefinitions@2418-05-01-preview
Fake.Authorization/accessReviewScheduleDefinitions@2421-03-01-preview
Fake.Authorization/accessReviewScheduleDefinitions@2421-07-01-preview
Fake.Authorization/accessReviewScheduleSettings@2418-05-01-preview
Fake.Authorization/accessReviewScheduleSettings@2421-03-01-preview
Fake.Authorization/accessReviewScheduleSettings@2421-07-01-preview
Fake.Authorization/roleAssignmentScheduleRequests@2420-10-01-preview
Fake.Authorization/roleEligibilityScheduleRequests@2420-10-01-preview
Fake.Authorization/roleManagementPolicyAssignments@2420-10-01-preview
Fake.Authorization/accessReviewScheduleDefinitions/instances@2421-07-01-preview
Fake.Authorization/locks@2415-01-01
Fake.Authorization/locks@2416-09-01
Fake.Authorization/locks@2417-04-01
Fake.Authorization/locks@2420-05-01
Fake.Authorization/policydefinitions@2415-10-01-preview
Fake.Authorization/policydefinitions@2416-04-01
Fake.Authorization/policydefinitions@2416-12-01
Fake.Authorization/policydefinitions@2418-03-01
Fake.Authorization/policydefinitions@2418-05-01
Fake.Authorization/policydefinitions@2419-01-01
Fake.Authorization/policydefinitions@2419-06-01
Fake.Authorization/policydefinitions@2419-09-01
Fake.Authorization/policydefinitions@2420-03-01
Fake.Authorization/policydefinitions@2420-09-01
Fake.Authorization/policydefinitions@2421-06-01
Fake.Authorization/policydefinitions@2423-04-01
Fake.Authorization/policyassignments@2415-10-01-preview
Fake.Authorization/policyassignments@2416-04-01
Fake.Authorization/policyassignments@2416-12-01
Fake.Authorization/policyassignments@2417-06-01-preview
Fake.Authorization/policyassignments@2418-03-01
Fake.Authorization/policyassignments@2418-05-01
Fake.Authorization/policyassignments@2419-01-01
Fake.Authorization/policyassignments@2419-06-01
Fake.Authorization/policyassignments@2419-09-01
Fake.Authorization/policyassignments@2420-03-01
Fake.Authorization/policyassignments@2420-09-01
Fake.Authorization/policyassignments@2421-06-01
Fake.Authorization/policySetDefinitions@2417-06-01-preview
Fake.Authorization/policySetDefinitions@2418-03-01
Fake.Authorization/policySetDefinitions@2418-05-01
Fake.Authorization/policySetDefinitions@2419-01-01
Fake.Authorization/policySetDefinitions@2419-06-01
Fake.Authorization/policySetDefinitions@2419-09-01
Fake.Authorization/policySetDefinitions@2420-03-01
Fake.Authorization/policySetDefinitions@2420-09-01
Fake.Authorization/policySetDefinitions@2421-06-01
Fake.Authorization/policyExemptions@2420-07-01-preview
Fake.Billing/billingAccounts/lineOfCredit@2418-11-01-preview
Fake.Blueprint/blueprintAssignments@2417-11-11-preview
Fake.Blueprint/blueprintAssignments@2418-11-01-preview
Fake.Blueprint/blueprints@2418-11-01-preview
Fake.Blueprint/blueprints/artifacts@2418-11-01-preview
Fake.Blueprint/blueprints/versions@2418-11-01-preview
Fake.Capacity/autoQuotaIncrease@2419-07-19-preview
Fake.Capacity/resourceProviders/locations/serviceLimits@2419-07-19-preview
Fake.Capacity/resourceProviders/locations/serviceLimits@2420-10-25
Fake.ChangeAnalysis/profile@2420-04-01-preview
Fake.Confluent/agreements@2420-03-01
Fake.Confluent/agreements@2420-03-01-preview
Fake.Confluent/agreements@2421-03-01-preview
Fake.Consumption/budgets@2417-12-30-preview
Fake.Consumption/budgets@2418-01-31
Fake.Consumption/budgets@2418-03-31
Fake.Consumption/budgets@2418-06-30
Fake.Consumption/budgets@2418-08-31
Fake.Consumption/budgets@2418-10-01
Fake.Consumption/budgets@2419-01-01
Fake.Consumption/budgets@2419-04-01-preview
Fake.Consumption/budgets@2419-05-01
Fake.Consumption/budgets@2419-05-01-preview
Fake.Consumption/budgets@2419-06-01
Fake.Consumption/budgets@2419-10-01
Fake.Consumption/budgets@2419-11-01
Fake.Consumption/budgets@2421-05-01
Fake.Datadog/agreements@2420-02-01-preview
Fake.Datadog/agreements@2421-03-01
Fake.EventGrid/eventSubscriptions@2417-06-15-preview
Fake.EventGrid/eventSubscriptions@2417-09-15-preview
Fake.EventGrid/eventSubscriptions@2418-01-01
Fake.EventGrid/eventSubscriptions@2418-05-01-preview
Fake.EventGrid/eventSubscriptions@2418-09-15-preview
Fake.EventGrid/eventSubscriptions@2419-01-01
Fake.EventGrid/eventSubscriptions@2419-02-01-preview
Fake.EventGrid/eventSubscriptions@2419-06-01
Fake.EventGrid/eventSubscriptions@2420-01-01-preview
Fake.EventGrid/eventSubscriptions@2420-04-01-preview
Fake.EventGrid/eventSubscriptions@2420-06-01
Fake.EventGrid/eventSubscriptions@2420-10-15-preview
Fake.EventGrid/eventSubscriptions@2421-06-01-preview
Fake.HybridNetwork/locations/vendors/networkFunctions@2420-01-01-preview
Fake.HybridNetwork/locations/vendors/networkFunctions@2421-05-01
Fake.HybridNetwork/vendors@2420-01-01-preview
Fake.HybridNetwork/vendors@2421-05-01
Fake.HybridNetwork/vendors/vendorSkus@2420-01-01-preview
Fake.HybridNetwork/vendors/vendorSkus@2421-05-01
Fake.HybridNetwork/vendors/vendorSkus/previewSubscriptions@2420-01-01-preview
Fake.HybridNetwork/vendors/vendorSkus/previewSubscriptions@2421-05-01
Fake.insights/diagnosticSettings@2415-07-01
Fake.insights/diagnosticSettings@2416-09-01
Fake.insights/diagnosticSettings@2417-05-01-preview
Fake.insights/diagnosticSettings@2421-05-01-preview
Fake.Insights/logprofiles@2416-03-01
Fake.ManagedNetwork/scopeAssignments@2419-06-01-preview
Fake.Marketplace/privateStores/offers@2420-01-01
Fake.Peering/peerAsns@2419-08-01-preview
Fake.Peering/peerAsns@2419-09-01-preview
Fake.Peering/peerAsns@2420-01-01-preview
Fake.Peering/peerAsns@2420-04-01
Fake.Peering/peerAsns@2420-10-01
Fake.Peering/peerAsns@2421-01-01
Fake.Peering/peerAsns@2421-06-01
Fake.PolicyInsights/remediations@2418-07-01-preview
Fake.PolicyInsights/remediations@2419-07-01
Fake.PolicyInsights/attestations@2421-01-01
Fake.Resources/deployments@2418-05-01
Fake.Resources/deployments@2419-03-01
Fake.Resources/deployments@2419-05-01
Fake.Resources/deployments@2419-05-10
Fake.Resources/deployments@2419-07-01
Fake.Resources/deployments@2419-08-01
Fake.Resources/deployments@2419-10-01
Fake.Resources/deployments@2420-06-01
Fake.Resources/deployments@2420-08-01
Fake.Resources/deployments@2420-10-01
Fake.Resources/deployments@2421-01-01
Fake.Resources/deployments@2421-04-01
Fake.Resources/resourceGroups@2418-05-01
Fake.Resources/resourceGroups@2419-03-01
Fake.Resources/resourceGroups@2419-05-01
Fake.Resources/resourceGroups@2419-05-10
Fake.Resources/resourceGroups@2419-06-01-preview
Fake.Resources/resourceGroups@2419-07-01
Fake.Resources/resourceGroups@2419-08-01
Fake.Resources/resourceGroups@2419-10-01
Fake.Resources/resourceGroups@2419-10-01-preview
Fake.Resources/resourceGroups@2420-06-01
Fake.Resources/resourceGroups@2420-08-01
Fake.Resources/resourceGroups@2420-10-01
Fake.Resources/resourceGroups@2421-01-01
Fake.Resources/resourceGroups@2421-03-01-preview
Fake.Resources/resourceGroups@2421-04-01
Fake.Resources/resourceGroups@2421-05-01
Fake.Resources/tags@2419-10-01
Fake.Resources/tags@2420-06-01
Fake.Resources/tags@2420-08-01
Fake.Resources/tags@2420-10-01
Fake.Resources/tags@2421-01-01
Fake.Resources/tags@2421-04-01
Fake.Security/locations/applicationWhitelistings@2415-06-01-preview
Fake.Security/locations/applicationWhitelistings@2420-01-01
Fake.Security/autoProvisioningSettings@2417-08-01-preview
Fake.Security/pricings@2417-08-01-preview
Fake.Security/pricings@2418-06-01
Fake.Security/securityContacts@2417-08-01-preview
Fake.Security/settings@2417-08-01-preview
Fake.Security/settings@2419-01-01
Fake.Security/settings@2421-06-01
Fake.Security/settings@2421-07-01
Fake.Security/workspaceSettings@2417-08-01-preview
Fake.Security/advancedThreatProtectionSettings@2417-08-01-preview
Fake.Security/advancedThreatProtectionSettings@2419-01-01
Fake.Security/deviceSecurityGroups@2417-08-01-preview
Fake.Security/deviceSecurityGroups@2419-08-01
Fake.Security/alertsSuppressionRules@2419-01-01-preview
Fake.Security/assessmentMetadata@2419-01-01-preview
Fake.Security/assessmentMetadata@2420-01-01
Fake.Security/assessmentMetadata@2421-06-01
Fake.Security/assessments@2419-01-01-preview
Fake.Security/assessments@2420-01-01
Fake.Security/assessments@2421-06-01
Fake.Security/connectors@2420-01-01-preview
Fake.Security/ingestionSettings@2421-01-15-preview
Fake.Support/supportTickets@2419-05-01-preview
Fake.Support/supportTickets@2420-04-01
Fake.Support/supportTickets/communications@2419-05-01-preview
Fake.Support/supportTickets/communications@2420-04-01
Fake.Web/publishingCredentials@2415-08-01";

        public static Mock<IAzResourceTypeLoader> GetAzResourceTypeLoaderWithInjectedTypes(string[] resourceTypes)
        {
            var fakeResourceTypeReferences = FakeResourceTypes.GetFakeResourceTypeReferences(resourceTypes);
            var typesLoader = StrictMock.Of<IAzResourceTypeLoader>();
            typesLoader.Setup(m => m.LoadType(It.IsAny<ResourceTypeReference>()))
                .Returns<ResourceTypeReference>((tr) => new ResourceTypeComponents(
                    tr,
                    ResourceScope.Tenant | ResourceScope.ManagementGroup | ResourceScope.Subscription | ResourceScope.ResourceGroup | ResourceScope.Resource,
                    ResourceScope.None,
                    ResourceFlags.None,
                    new ObjectType(tr.FormatName(), TypeSymbolValidationFlags.Default, Enumerable.Empty<TypeProperty>(), LanguageConstants.Any)));

            typesLoader.Setup(m => m.GetAvailableTypes()).Returns(fakeResourceTypeReferences);
            return typesLoader;
        }


    }
}
